@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -19,23 +19,29 @@
 #include <crm/crm.h>
 #include <crm/common/ipc_internal.h>
 #include <crm/common/mainloop.h>
+#include <crm/msg_xml.h>
 
 #include "pacemaker-attrd.h"
 
 cib_t *the_cib = NULL;
 
-static bool requesting_shutdown = FALSE;
-static bool shutting_down = FALSE;
+static bool requesting_shutdown = false;
+static bool shutting_down = false;
 static GMainLoop *mloop = NULL;
 
+/* A hash table storing information on the protocol version of each peer attrd.
+ * The key is the peer's uname, and the value is the protocol version number.
+ */
+GHashTable *peer_protocol_vers = NULL;
+
 /*!
  * \internal
  * \brief  Set requesting_shutdown state
  */
 void
-attrd_set_requesting_shutdown()
+attrd_set_requesting_shutdown(void)
 {
-    requesting_shutdown = TRUE;
+    requesting_shutdown = true;
 }
 
 /*!
@@ -43,19 +49,19 @@ attrd_set_requesting_shutdown()
  * \brief  Clear requesting_shutdown state
  */
 void
-attrd_clear_requesting_shutdown()
+attrd_clear_requesting_shutdown(void)
 {
-    requesting_shutdown = FALSE;
+    requesting_shutdown = false;
 }
 
 /*!
  * \internal
  * \brief Check whether we're currently requesting shutdown
  *
- * \return TRUE if requesting shutdown, FALSE otherwise
+ * \return true if requesting shutdown, false otherwise
  */
-gboolean
-attrd_requesting_shutdown()
+bool
+attrd_requesting_shutdown(void)
 {
     return requesting_shutdown;
 }
@@ -64,10 +70,10 @@ attrd_requesting_shutdown()
  * \internal
  * \brief Check whether we're currently shutting down
  *
- * \return TRUE if shutting down, FALSE otherwise
+ * \return true if shutting down, false otherwise
  */
-gboolean
-attrd_shutting_down()
+bool
+attrd_shutting_down(void)
 {
     return shutting_down;
 }
@@ -82,7 +88,7 @@ void
 attrd_shutdown(int nsig)
 {
     // Tell various functions not to do anthing
-    shutting_down = TRUE;
+    shutting_down = true;
 
     // Don't respond to signals while shutting down
     mainloop_destroy_signal(SIGTERM);
@@ -92,6 +98,14 @@ attrd_shutdown(int nsig)
     mainloop_destroy_signal(SIGUSR2);
     mainloop_destroy_signal(SIGTRAP);
 
+    attrd_free_waitlist();
+    attrd_free_confirmations();
+
+    if (peer_protocol_vers != NULL) {
+        g_hash_table_destroy(peer_protocol_vers);
+        peer_protocol_vers = NULL;
+    }
+
     if ((mloop == NULL) || !g_main_loop_is_running(mloop)) {
         /* If there's no main loop active, just exit. This should be possible
          * only if we get SIGTERM in brief windows at start-up and shutdown.
@@ -108,7 +122,7 @@ attrd_shutdown(int nsig)
  * \brief Create a main loop for attrd
  */
 void
-attrd_init_mainloop()
+attrd_init_mainloop(void)
 {
     mloop = g_main_loop_new(NULL, FALSE);
 }
@@ -118,123 +132,42 @@ attrd_init_mainloop()
  * \brief Run attrd main loop
  */
 void
-attrd_run_mainloop()
+attrd_run_mainloop(void)
 {
     g_main_loop_run(mloop);
 }
 
-/*!
- * \internal
- * \brief Accept a new client IPC connection
- *
- * \param[in] c    New connection
- * \param[in] uid  Client user id
- * \param[in] gid  Client group id
- *
- * \return pcmk_ok on success, -errno otherwise
- */
-static int32_t
-attrd_ipc_accept(qb_ipcs_connection_t *c, uid_t uid, gid_t gid)
-{
-    crm_trace("New client connection %p", c);
-    if (shutting_down) {
-        crm_info("Ignoring new connection from pid %d during shutdown",
-                 pcmk__client_pid(c));
-        return -EPERM;
-    }
-
-    if (pcmk__new_client(c, uid, gid) == NULL) {
-        return -EIO;
-    }
-    return pcmk_ok;
-}
-
-/*!
- * \internal
- * \brief Destroy a client IPC connection
- *
- * \param[in] c  Connection to destroy
- *
- * \return FALSE (i.e. do not re-run this callback)
- */
-static int32_t
-attrd_ipc_closed(qb_ipcs_connection_t *c)
-{
-    pcmk__client_t *client = pcmk__find_client(c);
-
-    if (client == NULL) {
-        crm_trace("Ignoring request to clean up unknown connection %p", c);
-    } else {
-        crm_trace("Cleaning up closed client connection %p", c);
-        pcmk__free_client(client);
-    }
-    return FALSE;
-}
-
-/*!
- * \internal
- * \brief Destroy a client IPC connection
- *
- * \param[in] c  Connection to destroy
- *
- * \note We handle a destroyed connection the same as a closed one,
- *       but we need a separate handler because the return type is different.
- */
-static void
-attrd_ipc_destroy(qb_ipcs_connection_t *c)
-{
-    crm_trace("Destroying client connection %p", c);
-    attrd_ipc_closed(c);
-}
-
-/*!
- * \internal
- * \brief Set up attrd IPC communication
- *
- * \param[out] ipcs         Will be set to newly allocated server connection
- * \param[in]  dispatch_fn  Handler for new messages on connection
- */
 void
-attrd_init_ipc(qb_ipcs_service_t **ipcs, qb_ipcs_msg_process_fn dispatch_fn)
-{
-
-    static struct qb_ipcs_service_handlers ipc_callbacks = {
-        .connection_accept = attrd_ipc_accept,
-        .connection_created = NULL,
-        .msg_process = NULL,
-        .connection_closed = attrd_ipc_closed,
-        .connection_destroyed = attrd_ipc_destroy
-    };
-
-    ipc_callbacks.msg_process = dispatch_fn;
-    pcmk__serve_attrd_ipc(ipcs, &ipc_callbacks);
-}
-
-void
-attrd_cib_disconnect()
+attrd_cib_disconnect(void)
 {
     CRM_CHECK(the_cib != NULL, return);
     the_cib->cmds->del_notify_callback(the_cib, T_CIB_REPLACE_NOTIFY, attrd_cib_replaced_cb);
-    the_cib->cmds->del_notify_callback(the_cib, T_CIB_DIFF_NOTIFY, attrd_cib_updated_cb); 
-    the_cib->cmds->signoff(the_cib);
-    cib_delete(the_cib);
-    the_cib = NULL;
+    the_cib->cmds->del_notify_callback(the_cib, T_CIB_DIFF_NOTIFY, attrd_cib_updated_cb);
+    cib__clean_up_connection(&the_cib);
 }
 
 void
 attrd_cib_replaced_cb(const char *event, xmlNode * msg)
 {
+    int change_section = cib_change_section_nodes | cib_change_section_status | cib_change_section_alerts;
+
     if (attrd_requesting_shutdown() || attrd_shutting_down()) {
         return;
     }
 
+    crm_element_value_int(msg, F_CIB_CHANGE_SECTION, &change_section);
+
     if (attrd_election_won()) {
-        crm_notice("Updating all attributes after %s event", event);
-        write_attributes(TRUE, FALSE);
+        if (change_section & (cib_change_section_nodes | cib_change_section_status)) {
+            crm_notice("Updating all attributes after %s event", event);
+            attrd_write_attributes(true, false);
+        }
     }
 
-    // Check for changes in alerts
-    mainloop_set_trigger(attrd_config_read);
+    if (change_section & cib_change_section_alerts) {
+        // Check for changes in alerts
+        mainloop_set_trigger(attrd_config_read);
+    }
 }
 
 /* strlen("value") */
@@ -246,9 +179,9 @@ attrd_cib_replaced_cb(const char *event, xmlNode * msg)
  *
  * \param[in] value  Attribute value to check
  *
- * \return TRUE if value needs expansion, FALSE otherwise
+ * \return true if value needs expansion, false otherwise
  */
-gboolean
+bool
 attrd_value_needs_expansion(const char *value)
 {
     return ((strlen(value) >= (plus_plus_len + 2))
@@ -322,3 +255,108 @@ attrd_failure_regex(regex_t *regex, const char *rsc, const char *op,
 
     return (rc == 0)? pcmk_ok : -EINVAL;
 }
+
+void
+attrd_free_attribute_value(gpointer data)
+{
+    attribute_value_t *v = data;
+
+    free(v->nodename);
+    free(v->current);
+    free(v->requested);
+    free(v);
+}
+
+void
+attrd_free_attribute(gpointer data)
+{
+    attribute_t *a = data;
+    if(a) {
+        free(a->id);
+        free(a->set_id);
+        free(a->set_type);
+        free(a->uuid);
+        free(a->user);
+
+        mainloop_timer_del(a->timer);
+        g_hash_table_destroy(a->values);
+
+        free(a);
+    }
+}
+
+/*!
+ * \internal
+ * \brief When a peer node leaves the cluster, stop tracking its protocol version.
+ *
+ * \param[in] host  The peer node's uname to be removed
+ */
+void
+attrd_remove_peer_protocol_ver(const char *host)
+{
+    if (peer_protocol_vers != NULL) {
+        g_hash_table_remove(peer_protocol_vers, host);
+    }
+}
+
+/*!
+ * \internal
+ * \brief When a peer node broadcasts a message with its protocol version, keep
+ *        track of that information.
+ *
+ * We keep track of each peer's protocol version so we know which peers to
+ * expect confirmation messages from when handling cluster-wide sync points.
+ * We additionally keep track of the lowest protocol version supported by all
+ * peers so we know when we can send IPC messages containing more than one
+ * request.
+ *
+ * \param[in] host  The peer node's uname to be tracked
+ * \param[in] value The peer node's protocol version
+ */
+void
+attrd_update_minimum_protocol_ver(const char *host, const char *value)
+{
+    int ver;
+
+    if (peer_protocol_vers == NULL) {
+        peer_protocol_vers = pcmk__strkey_table(free, NULL);
+    }
+
+    pcmk__scan_min_int(value, &ver, 0);
+
+    if (ver > 0) {
+        char *host_name = strdup(host);
+
+        /* Record the peer attrd's protocol version. */
+        CRM_ASSERT(host_name != NULL);
+        g_hash_table_insert(peer_protocol_vers, host_name, GINT_TO_POINTER(ver));
+
+        /* If the protocol version is a new minimum, record it as such. */
+        if (minimum_protocol_version == -1 || ver < minimum_protocol_version) {
+            minimum_protocol_version = ver;
+            crm_trace("Set minimum attrd protocol version to %d",
+                      minimum_protocol_version);
+        }
+    }
+}
+
+void
+attrd_copy_xml_attributes(xmlNode *src, xmlNode *dest)
+{
+    /* Copy attributes from the wrapper parent node into the child node.
+     * We can't just use copy_in_properties because we want to skip any
+     * attributes that are already set on the child.  For instance, if
+     * we were told to use a specific node, there will already be a node
+     * attribute on the child.  Copying the parent's node attribute over
+     * could result in the wrong value.
+     */
+    for (xmlAttrPtr a = pcmk__xe_first_attr(src); a != NULL; a = a->next) {
+        const char *p_name = (const char *) a->name;
+        const char *p_value = ((a == NULL) || (a->children == NULL)) ? NULL :
+                              (const char *) a->children->content;
+
+        if (crm_element_value(dest, p_name) == NULL) {
+            crm_xml_add(dest, p_name, p_value);
+        }
+    }
+}
