@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,20 +22,16 @@
 
 #include <pacemaker-controld.h>
 
-qb_ipcs_service_t *ipcs = NULL;
+static qb_ipcs_service_t *ipcs = NULL;
+
+static crm_trigger_t *config_read_trigger = NULL;
 
 #if SUPPORT_COROSYNC
 extern gboolean crm_connect_corosync(crm_cluster_t * cluster);
 #endif
 
 void crm_shutdown(int nsig);
-gboolean crm_read_options(gpointer user_data);
-
-gboolean fsa_has_quorum = FALSE;
-crm_trigger_t *fsa_source = NULL;
-crm_trigger_t *config_read = NULL;
-bool no_quorum_suicide_escalation = FALSE;
-bool controld_shutdown_lock_enabled = false;
+static gboolean crm_read_options(gpointer user_data);
 
 /*	 A_HA_CONNECT	*/
 void
@@ -48,7 +44,7 @@ do_ha_control(long long action,
     static crm_cluster_t *cluster = NULL;
 
     if (cluster == NULL) {
-        cluster = calloc(1, sizeof(crm_cluster_t));
+        cluster = pcmk_cluster_new();
     }
 
     if (action & A_HA_DISCONNECT) {
@@ -62,23 +58,23 @@ do_ha_control(long long action,
         crm_set_status_callback(&peer_update_callback);
         crm_set_autoreap(FALSE);
 
-        if (is_corosync_cluster()) {
 #if SUPPORT_COROSYNC
+        if (is_corosync_cluster()) {
             registered = crm_connect_corosync(cluster);
-#endif
         }
+#endif // SUPPORT_COROSYNC
 
-        if (registered == TRUE) {
+        if (registered) {
             controld_election_init(cluster->uname);
-            fsa_our_uname = cluster->uname;
-            fsa_our_uuid = cluster->uuid;
+            controld_globals.our_nodename = cluster->uname;
+            controld_globals.our_uuid = cluster->uuid;
             if(cluster->uuid == NULL) {
                 crm_err("Could not obtain local uuid");
                 registered = FALSE;
             }
         }
 
-        if (registered == FALSE) {
+        if (!registered) {
             controld_set_fsa_input_flags(R_HA_DISCONNECTED);
             register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
             return;
@@ -118,7 +114,7 @@ do_shutdown_req(long long action,
     controld_set_fsa_input_flags(R_SHUTDOWN);
     //controld_set_fsa_input_flags(R_STAYDOWN);
     crm_info("Sending shutdown request to all peers (DC is %s)",
-             (fsa_our_dc? fsa_our_dc : "not set"));
+             pcmk__s(controld_globals.dc_name, "not set"));
     msg = create_request(CRM_OP_SHUTDOWN_REQ, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
 
     if (send_cluster_message(NULL, crm_msg_crmd, msg, TRUE) == FALSE) {
@@ -127,30 +123,26 @@ do_shutdown_req(long long action,
     free_xml(msg);
 }
 
-extern char *max_generation_from;
-extern xmlNode *max_generation_xml;
-extern GHashTable *resource_history;
-extern GHashTable *voted;
-extern pcmk__output_t *logger_out;
-
 void
 crmd_fast_exit(crm_exit_t exit_code)
 {
-    if (pcmk_is_set(fsa_input_register, R_STAYDOWN)) {
+    if (pcmk_is_set(controld_globals.fsa_input_register, R_STAYDOWN)) {
         crm_warn("Inhibiting respawn "CRM_XS" remapping exit code %d to %d",
                  exit_code, CRM_EX_FATAL);
         exit_code = CRM_EX_FATAL;
 
     } else if ((exit_code == CRM_EX_OK)
-               && pcmk_is_set(fsa_input_register, R_IN_RECOVERY)) {
+               && pcmk_is_set(controld_globals.fsa_input_register,
+                              R_IN_RECOVERY)) {
         crm_err("Could not recover from internal error");
         exit_code = CRM_EX_ERROR;
     }
 
-    if (logger_out != NULL) {
-        logger_out->finish(logger_out, exit_code, true, NULL);
-        pcmk__output_free(logger_out);
-        logger_out = NULL;
+    if (controld_globals.logger_out != NULL) {
+        controld_globals.logger_out->finish(controld_globals.logger_out,
+                                            exit_code, true, NULL);
+        pcmk__output_free(controld_globals.logger_out);
+        controld_globals.logger_out = NULL;
     }
 
     crm_exit(exit_code);
@@ -159,8 +151,7 @@ crmd_fast_exit(crm_exit_t exit_code)
 crm_exit_t
 crmd_exit(crm_exit_t exit_code)
 {
-    GList *gIter = NULL;
-    GMainLoop *mloop = crmd_mainloop;
+    GMainLoop *mloop = controld_globals.mainloop;
 
     static bool in_progress = FALSE;
 
@@ -194,7 +185,7 @@ crmd_exit(crm_exit_t exit_code)
     controld_shutdown_schedulerd_ipc();
     controld_disconnect_fencer(TRUE);
 
-    if ((exit_code == CRM_EX_OK) && (crmd_mainloop == NULL)) {
+    if ((exit_code == CRM_EX_OK) && (controld_globals.mainloop == NULL)) {
         crm_debug("No mainloop detected");
         exit_code = CRM_EX_ERROR;
     }
@@ -214,20 +205,22 @@ crmd_exit(crm_exit_t exit_code)
 
 /* Clean up as much memory as possible for valgrind */
 
-    for (gIter = fsa_message_queue; gIter != NULL; gIter = gIter->next) {
-        fsa_data_t *fsa_data = gIter->data;
+    for (GList *iter = controld_globals.fsa_message_queue; iter != NULL;
+         iter = iter->next) {
+        fsa_data_t *fsa_data = (fsa_data_t *) iter->data;
 
         crm_info("Dropping %s: [ state=%s cause=%s origin=%s ]",
                  fsa_input2string(fsa_data->fsa_input),
-                 fsa_state2string(fsa_state),
+                 fsa_state2string(controld_globals.fsa_state),
                  fsa_cause2string(fsa_data->fsa_cause), fsa_data->origin);
         delete_fsa_input(fsa_data);
     }
 
     controld_clear_fsa_input_flags(R_MEMBERSHIP);
-    g_list_free(fsa_message_queue); fsa_message_queue = NULL;
 
-    metadata_cache_fini();
+    g_list_free(controld_globals.fsa_message_queue);
+    controld_globals.fsa_message_queue = NULL;
+
     controld_election_fini();
 
     /* Tear down the CIB manager connection, but don't free it yet -- it could
@@ -236,15 +229,15 @@ crmd_exit(crm_exit_t exit_code)
 
     controld_disconnect_cib_manager();
 
-    verify_stopped(fsa_state, LOG_WARNING);
+    verify_stopped(controld_globals.fsa_state, LOG_WARNING);
     controld_clear_fsa_input_flags(R_LRM_CONNECTED);
     lrm_state_destroy_all();
 
-    /* This basically will not work, since mainloop has a reference to it */
-    mainloop_destroy_trigger(fsa_source); fsa_source = NULL;
+    mainloop_destroy_trigger(config_read_trigger);
+    config_read_trigger = NULL;
 
-    mainloop_destroy_trigger(config_read); config_read = NULL;
-    mainloop_destroy_trigger(transition_trigger); transition_trigger = NULL;
+    controld_destroy_fsa_trigger();
+    controld_destroy_transition_trigger();
 
     pcmk__client_cleanup();
     crm_peer_destroy();
@@ -253,19 +246,28 @@ crmd_exit(crm_exit_t exit_code)
     te_cleanup_stonith_history_sync(NULL, TRUE);
     controld_free_sched_timer();
 
-    free(fsa_our_dc_version); fsa_our_dc_version = NULL;
-    free(fsa_our_uname); fsa_our_uname = NULL;
-    free(fsa_our_uuid); fsa_our_uuid = NULL;
-    free(fsa_our_dc); fsa_our_dc = NULL;
+    free(controld_globals.our_nodename);
+    controld_globals.our_nodename = NULL;
 
-    free(fsa_cluster_name); fsa_cluster_name = NULL;
+    free(controld_globals.our_uuid);
+    controld_globals.our_uuid = NULL;
 
-    free(te_uuid); te_uuid = NULL;
-    free(failed_stop_offset); failed_stop_offset = NULL;
-    free(failed_start_offset); failed_start_offset = NULL;
+    free(controld_globals.dc_name);
+    controld_globals.dc_name = NULL;
 
-    free(max_generation_from); max_generation_from = NULL;
-    free_xml(max_generation_xml); max_generation_xml = NULL;
+    free(controld_globals.dc_version);
+    controld_globals.dc_version = NULL;
+
+    free(controld_globals.cluster_name);
+    controld_globals.cluster_name = NULL;
+
+    free(controld_globals.te_uuid);
+    controld_globals.te_uuid = NULL;
+
+    free_max_generation();
+    controld_destroy_cib_replacements_table();
+    controld_destroy_failed_sync_table();
+    controld_destroy_outside_events_table();
 
     mainloop_destroy_signal(SIGPIPE);
     mainloop_destroy_signal(SIGUSR1);
@@ -274,10 +276,10 @@ crmd_exit(crm_exit_t exit_code)
     /* leave SIGCHLD engaged as we might still want to drain some service-actions */
 
     if (mloop) {
-        GMainContext *ctx = g_main_loop_get_context(crmd_mainloop);
+        GMainContext *ctx = g_main_loop_get_context(controld_globals.mainloop);
 
         /* Don't re-enter this block */
-        crmd_mainloop = NULL;
+        controld_globals.mainloop = NULL;
 
         /* no signals on final draining anymore */
         mainloop_destroy_signal(SIGCHLD);
@@ -303,8 +305,8 @@ crmd_exit(crm_exit_t exit_code)
         mainloop_destroy_signal(SIGCHLD);
     }
 
-    cib_delete(fsa_cib_conn);
-    fsa_cib_conn = NULL;
+    cib_delete(controld_globals.cib_conn);
+    controld_globals.cib_conn = NULL;
 
     throttle_fini();
 
@@ -350,12 +352,14 @@ do_startup(long long action,
     mainloop_add_signal(SIGTERM, crm_shutdown);
     mainloop_add_signal(SIGPIPE, sigpipe_ignore);
 
-    fsa_source = mainloop_add_trigger(G_PRIORITY_HIGH, crm_fsa_trigger, NULL);
-    config_read = mainloop_add_trigger(G_PRIORITY_HIGH, crm_read_options, NULL);
-    transition_trigger = mainloop_add_trigger(G_PRIORITY_LOW, te_graph_trigger, NULL);
+    config_read_trigger = mainloop_add_trigger(G_PRIORITY_HIGH,
+                                               crm_read_options, NULL);
+
+    controld_init_fsa_trigger();
+    controld_init_transition_trigger();
 
     crm_debug("Creating CIB manager and executor objects");
-    fsa_cib_conn = cib_new();
+    controld_globals.cib_conn = cib_new();
 
     lrm_state_init_local();
     if (controld_init_fsa_timers() == FALSE) {
@@ -385,10 +389,10 @@ dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
     xmlNode *msg = pcmk__client_data2xml(client, data, &id, &flags);
 
     if (msg == NULL) {
-        pcmk__ipc_send_ack(client, id, flags, "ack", CRM_EX_PROTOCOL);
+        pcmk__ipc_send_ack(client, id, flags, "ack", NULL, CRM_EX_PROTOCOL);
         return 0;
     }
-    pcmk__ipc_send_ack(client, id, flags, "ack", CRM_EX_INDETERMINATE);
+    pcmk__ipc_send_ack(client, id, flags, "ack", NULL, CRM_EX_INDETERMINATE);
 
     CRM_ASSERT(client->user != NULL);
     pcmk__update_acl_user(msg, F_CRM_USER, client->user);
@@ -400,13 +404,13 @@ dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
         route_message(C_IPC_MESSAGE, msg);
     }
 
-    trigger_fsa();
+    controld_trigger_fsa();
     free_xml(msg);
     return 0;
 }
 
 static int32_t
-crmd_ipc_closed(qb_ipcs_connection_t * c)
+ipc_client_disconnected(qb_ipcs_connection_t *c)
 {
     pcmk__client_t *client = pcmk__find_client(c);
 
@@ -416,16 +420,16 @@ crmd_ipc_closed(qb_ipcs_connection_t * c)
                   c, client);
         free(client->userdata);
         pcmk__free_client(client);
-        trigger_fsa();
+        controld_trigger_fsa();
     }
     return 0;
 }
 
 static void
-crmd_ipc_destroy(qb_ipcs_connection_t * c)
+ipc_connection_destroyed(qb_ipcs_connection_t *c)
 {
     crm_trace("Connection %p", c);
-    crmd_ipc_closed(c);
+    ipc_client_disconnected(c);
 }
 
 /*	 A_STOP	*/
@@ -449,39 +453,43 @@ do_started(long long action,
         .connection_accept = accept_controller_client,
         .connection_created = NULL,
         .msg_process = dispatch_controller_ipc,
-        .connection_closed = crmd_ipc_closed,
-        .connection_destroyed = crmd_ipc_destroy
+        .connection_closed = ipc_client_disconnected,
+        .connection_destroyed = ipc_connection_destroyed
     };
 
     if (cur_state != S_STARTING) {
         crm_err("Start cancelled... %s", fsa_state2string(cur_state));
         return;
 
-    } else if (!pcmk_is_set(fsa_input_register, R_MEMBERSHIP)) {
+    } else if (!pcmk_is_set(controld_globals.fsa_input_register,
+                            R_MEMBERSHIP)) {
         crm_info("Delaying start, no membership data (%.16llx)", R_MEMBERSHIP);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (!pcmk_is_set(fsa_input_register, R_LRM_CONNECTED)) {
+    } else if (!pcmk_is_set(controld_globals.fsa_input_register,
+                            R_LRM_CONNECTED)) {
         crm_info("Delaying start, not connected to executor (%.16llx)", R_LRM_CONNECTED);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (!pcmk_is_set(fsa_input_register, R_CIB_CONNECTED)) {
+    } else if (!pcmk_is_set(controld_globals.fsa_input_register,
+                            R_CIB_CONNECTED)) {
         crm_info("Delaying start, CIB not connected (%.16llx)", R_CIB_CONNECTED);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (!pcmk_is_set(fsa_input_register, R_READ_CONFIG)) {
+    } else if (!pcmk_is_set(controld_globals.fsa_input_register,
+                            R_READ_CONFIG)) {
         crm_info("Delaying start, Config not read (%.16llx)", R_READ_CONFIG);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (!pcmk_is_set(fsa_input_register, R_PEER_DATA)) {
+    } else if (!pcmk_is_set(controld_globals.fsa_input_register, R_PEER_DATA)) {
 
         crm_info("Delaying start, No peer data (%.16llx)", R_PEER_DATA);
         crmd_fsa_stall(TRUE);
@@ -514,7 +522,7 @@ do_recover(long long action,
     register_fsa_input(C_FSA_INTERNAL, I_TERMINATE, NULL);
 }
 
-static pcmk__cluster_option_t crmd_opts[] = {
+static pcmk__cluster_option_t controller_options[] = {
     /* name, old name, type, allowed values,
      * default value, validator,
      * short description,
@@ -522,167 +530,173 @@ static pcmk__cluster_option_t crmd_opts[] = {
      */
     {
         "dc-version", NULL, "string", NULL, PCMK__VALUE_NONE, NULL,
-        "Pacemaker version on cluster node elected Designated Controller (DC)",
-        "Includes a hash which identifies the exact changeset the code was "
-            "built from. Used for diagnostic purposes."
+        N_("Pacemaker version on cluster node elected Designated Controller (DC)"),
+        N_("Includes a hash which identifies the exact changeset the code was "
+            "built from. Used for diagnostic purposes.")
     },
     {
         "cluster-infrastructure", NULL, "string", NULL, "corosync", NULL,
-        "The messaging stack on which Pacemaker is currently running",
-        "Used for informational and diagnostic purposes."
+        N_("The messaging stack on which Pacemaker is currently running"),
+        N_("Used for informational and diagnostic purposes.")
     },
     {
         "cluster-name", NULL, "string", NULL, NULL, NULL,
-        "An arbitrary name for the cluster",
-        "This optional value is mostly for users' convenience as desired "
+        N_("An arbitrary name for the cluster"),
+        N_("This optional value is mostly for users' convenience as desired "
             "in administration, but may also be used in Pacemaker "
             "configuration rules via the #cluster-name node attribute, and "
-            "by higher-level tools and resource agents."
+            "by higher-level tools and resource agents.")
     },
     {
         XML_CONFIG_ATTR_DC_DEADTIME, NULL, "time",
         NULL, "20s", pcmk__valid_interval_spec,
-        "How long to wait for a response from other nodes during start-up",
-        "The optimal value will depend on the speed and load of your network "
-            "and the type of switches used."
+        N_("How long to wait for a response from other nodes during start-up"),
+        N_("The optimal value will depend on the speed and load of your network "
+            "and the type of switches used.")
     },
     {
         XML_CONFIG_ATTR_RECHECK, NULL, "time",
         N_("Zero disables polling, while positive values are an interval in seconds"
             "(unless other units are specified, for example \"5min\")"),
         "15min", pcmk__valid_interval_spec,
-        "Polling interval to recheck cluster state and evaluate rules "
-            "with date specifications",
-        "Pacemaker is primarily event-driven, and looks ahead to know when to "
+        N_("Polling interval to recheck cluster state and evaluate rules "
+            "with date specifications"),
+        N_("Pacemaker is primarily event-driven, and looks ahead to know when to "
             "recheck cluster state for failure timeouts and most time-based "
             "rules. However, it will also recheck the cluster after this "
             "amount of inactivity, to evaluate rules with date specifications "
-            "and serve as a fail-safe for certain types of scheduler bugs."
+            "and serve as a fail-safe for certain types of scheduler bugs.")
     },
     {
         "load-threshold", NULL, "percentage", NULL,
         "80%", pcmk__valid_percentage,
-        "Maximum amount of system load that should be used by cluster nodes",
-        "The cluster will slow down its recovery process when the amount of "
-            "system resources used (currently CPU) approaches this limit",
+        N_("Maximum amount of system load that should be used by cluster nodes"),
+        N_("The cluster will slow down its recovery process when the amount of "
+            "system resources used (currently CPU) approaches this limit"),
     },
     {
         "node-action-limit", NULL, "integer", NULL,
         "0", pcmk__valid_number,
-        "Maximum number of jobs that can be scheduled per node "
-            "(defaults to 2x cores)"
+        N_("Maximum number of jobs that can be scheduled per node "
+            "(defaults to 2x cores)")
     },
     { XML_CONFIG_ATTR_FENCE_REACTION, NULL, "string", NULL, "stop", NULL,
-        "How a cluster node should react if notified of its own fencing",
-        "A cluster node may receive notification of its own fencing if fencing "
+        N_("How a cluster node should react if notified of its own fencing"),
+        N_("A cluster node may receive notification of its own fencing if fencing "
         "is misconfigured, or if fabric fencing is in use that doesn't cut "
         "cluster communication. Allowed values are \"stop\" to attempt to "
         "immediately stop Pacemaker and stay stopped, or \"panic\" to attempt "
-        "to immediately reboot the local node, falling back to stop on failure."
+        "to immediately reboot the local node, falling back to stop on failure.")
     },
     {
         XML_CONFIG_ATTR_ELECTION_FAIL, NULL, "time", NULL,
         "2min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "Declare an election failed if it is not decided within this much "
+        N_("Declare an election failed if it is not decided within this much "
             "time. If you need to adjust this value, it probably indicates "
-            "the presence of a bug."
+            "the presence of a bug.")
     },
     {
         XML_CONFIG_ATTR_FORCE_QUIT, NULL, "time", NULL,
         "20min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "Exit immediately if shutdown does not complete within this much "
+        N_("Exit immediately if shutdown does not complete within this much "
             "time. If you need to adjust this value, it probably indicates "
-            "the presence of a bug."
+            "the presence of a bug.")
     },
     {
         "join-integration-timeout", "crmd-integration-timeout", "time", NULL,
         "3min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "If you need to adjust this value, it probably indicates "
-            "the presence of a bug."
+        N_("If you need to adjust this value, it probably indicates "
+            "the presence of a bug.")
     },
     {
         "join-finalization-timeout", "crmd-finalization-timeout", "time", NULL,
         "30min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "If you need to adjust this value, it probably indicates "
-            "the presence of a bug."
+        N_("If you need to adjust this value, it probably indicates "
+            "the presence of a bug.")
     },
     {
         "transition-delay", "crmd-transition-delay", "time", NULL,
         "0s", pcmk__valid_interval_spec,
-        "*** Advanced Use Only *** Enabling this option will slow down "
-            "cluster recovery under all conditions",
-        "Delay cluster recovery for this much time to allow for additional "
+        N_("*** Advanced Use Only *** Enabling this option will slow down "
+            "cluster recovery under all conditions"),
+        N_("Delay cluster recovery for this much time to allow for additional "
             "events to occur. Useful if your configuration is sensitive to "
-            "the order in which ping updates arrive."
+            "the order in which ping updates arrive.")
     },
     {
         "stonith-watchdog-timeout", NULL, "time", NULL,
         "0", controld_verify_stonith_watchdog_timeout,
-        "How long to wait before we can assume nodes are safely down "
-            "when watchdog-based self-fencing via SBD is in use",
-        "If nonzero, along with `have-watchdog=true` automatically set by the "
-            "cluster, when fencing is required, watchdog-based self-fencing "
-            "will be performed via SBD without requiring a fencing resource "
-            "explicitly configured. "
-            "If `stonith-watchdog-timeout` is set to a positive value, unseen "
-            "nodes are assumed to self-fence within this much time. +WARNING:+ "
-            "It must be ensured that this value is larger than the "
-            "`SBD_WATCHDOG_TIMEOUT` environment variable on all nodes. "
-            "Pacemaker verifies the settings individually on all nodes and "
-            "prevents startup or shuts down if configured wrongly on the fly. "
-            "It's strongly recommended that `SBD_WATCHDOG_TIMEOUT` is set to "
-            "the same value on all nodes. "
-            "If `stonith-watchdog-timeout` is set to a negative value, and "
-            "`SBD_WATCHDOG_TIMEOUT` is set, twice that value will be used. "
-            "+WARNING:+ In this case, it's essential (currently not verified by "
-            "Pacemaker) that `SBD_WATCHDOG_TIMEOUT` is set to the same value on "
-            "all nodes."
+        N_("How long before nodes can be assumed to be safely down when "
+           "watchdog-based self-fencing via SBD is in use"),
+        N_("If this is set to a positive value, lost nodes are assumed to "
+           "self-fence using watchdog-based SBD within this much time. This "
+           "does not require a fencing resource to be explicitly configured, "
+           "though a fence_watchdog resource can be configured, to limit use "
+           "to specific nodes. If this is set to 0 (the default), the cluster "
+           "will never assume watchdog-based self-fencing. If this is set to a "
+           "negative value, the cluster will use twice the local value of the "
+           "`SBD_WATCHDOG_TIMEOUT` environment variable if that is positive, "
+           "or otherwise treat this as 0. WARNING: When used, this timeout "
+           "must be larger than `SBD_WATCHDOG_TIMEOUT` on all nodes that use "
+           "watchdog-based SBD, and Pacemaker will refuse to start on any of "
+           "those nodes where this is not true for the local value or SBD is "
+           "not active. When this is set to a negative value, "
+           "`SBD_WATCHDOG_TIMEOUT` must be set to the same value on all nodes "
+           "that use SBD, otherwise data corruption or loss could occur.")
     },
     {
         "stonith-max-attempts", NULL, "integer", NULL,
         "10", pcmk__valid_positive_number,
-        "How many times fencing can fail before it will no longer be "
-            "immediately re-attempted on a target"
+        N_("How many times fencing can fail before it will no longer be "
+            "immediately re-attempted on a target")
     },
 
     // Already documented in libpe_status (other values must be kept identical)
     {
-        "no-quorum-policy", NULL, "select", "stop, freeze, ignore, demote, suicide",
-        "stop", pcmk__valid_quorum, NULL, NULL
+        "no-quorum-policy", NULL, "select",
+        "stop, freeze, ignore, demote, suicide", "stop", pcmk__valid_quorum,
+        N_("What to do when the cluster does not have quorum"), NULL
     },
     {
         XML_CONFIG_ATTR_SHUTDOWN_LOCK, NULL, "boolean", NULL,
-        "false", pcmk__valid_boolean, NULL, NULL
+        "false", pcmk__valid_boolean,
+        N_("Whether to lock resources to a cleanly shut down node"),
+        N_("When true, resources active on a node when it is cleanly shut down "
+            "are kept \"locked\" to that node (not allowed to run elsewhere) "
+            "until they start again on that node after it rejoins (or for at "
+            "most shutdown-lock-limit, if set). Stonith resources and "
+            "Pacemaker Remote connections are never locked. Clone and bundle "
+            "instances and the promoted role of promotable clones are "
+            "currently never locked, though support could be added in a future "
+            "release.")
+    },
+    {
+        XML_CONFIG_ATTR_SHUTDOWN_LOCK_LIMIT, NULL, "time", NULL,
+        "0", pcmk__valid_interval_spec,
+        N_("Do not lock resources to a cleanly shut down node longer than "
+           "this"),
+        N_("If shutdown-lock is true and this is set to a nonzero time "
+            "duration, shutdown locks will expire after this much time has "
+            "passed since the shutdown was initiated, even if the node has not "
+            "rejoined.")
     },
 };
 
 void
 crmd_metadata(void)
 {
-    char *s = pcmk__format_option_metadata("pacemaker-controld",
-                                           "Pacemaker controller options",
-                                           "Cluster options used by Pacemaker's "
-                                               "controller",
-                                           crmd_opts, PCMK__NELEM(crmd_opts));
+    const char *desc_short = "Pacemaker controller options";
+    const char *desc_long = "Cluster options used by Pacemaker's controller";
+
+    gchar *s = pcmk__format_option_metadata("pacemaker-controld", desc_short,
+                                            desc_long, controller_options,
+                                            PCMK__NELEM(controller_options));
     printf("%s", s);
-    free(s);
-}
-
-static void
-verify_crmd_options(GHashTable * options)
-{
-    pcmk__validate_cluster_options(options, crmd_opts, PCMK__NELEM(crmd_opts));
-}
-
-static const char *
-crmd_pref(GHashTable * options, const char *name)
-{
-    return pcmk__cluster_option(options, crmd_opts, PCMK__NELEM(crmd_opts),
-                                name);
+    g_free(s);
 }
 
 static void
@@ -726,82 +740,73 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     pe_unpack_nvpairs(crmconfig, crmconfig, XML_CIB_TAG_PROPSET, NULL,
                       config_hash, CIB_OPTIONS_FIRST, FALSE, now, NULL);
 
-    verify_crmd_options(config_hash);
+    // Validate all options, and use defaults if not already present in hash
+    pcmk__validate_cluster_options(config_hash, controller_options,
+                                   PCMK__NELEM(controller_options));
 
-    value = crmd_pref(config_hash, XML_CONFIG_ATTR_DC_DEADTIME);
-    election_trigger->period_ms = crm_parse_interval_spec(value);
-
-    value = crmd_pref(config_hash, "node-action-limit"); /* Also checks migration-limit */
-    throttle_update_job_max(value);
-
-    value = crmd_pref(config_hash, "load-threshold");
-    if(value) {
-        throttle_set_load_target(strtof(value, NULL) / 100.0);
-    }
-
-    value = crmd_pref(config_hash, "no-quorum-policy");
+    value = g_hash_table_lookup(config_hash, "no-quorum-policy");
     if (pcmk__str_eq(value, "suicide", pcmk__str_casei) && pcmk__locate_sbd()) {
-        no_quorum_suicide_escalation = TRUE;
+        controld_set_global_flags(controld_no_quorum_suicide);
     }
 
-    set_fence_reaction(crmd_pref(config_hash, XML_CONFIG_ATTR_FENCE_REACTION));
+    value = g_hash_table_lookup(config_hash, XML_CONFIG_ATTR_SHUTDOWN_LOCK);
+    if (crm_is_true(value)) {
+        controld_set_global_flags(controld_shutdown_lock_enabled);
+    } else {
+        controld_clear_global_flags(controld_shutdown_lock_enabled);
+    }
 
-    value = crmd_pref(config_hash,"stonith-max-attempts");
-    update_stonith_max_attempts(value);
-
-    value = crmd_pref(config_hash, XML_CONFIG_ATTR_FORCE_QUIT);
-    shutdown_escalation_timer->period_ms = crm_parse_interval_spec(value);
-    crm_debug("Shutdown escalation occurs if DC has not responded to request in %ums",
-              shutdown_escalation_timer->period_ms);
-
-    value = crmd_pref(config_hash, XML_CONFIG_ATTR_ELECTION_FAIL);
-    controld_set_election_period(value);
-
-    value = crmd_pref(config_hash, XML_CONFIG_ATTR_RECHECK);
-    recheck_interval_ms = crm_parse_interval_spec(value);
-    crm_debug("Re-run scheduler after %dms of inactivity", recheck_interval_ms);
-
-    value = crmd_pref(config_hash, "transition-delay");
-    transition_timer->period_ms = crm_parse_interval_spec(value);
-
-    value = crmd_pref(config_hash, "join-integration-timeout");
-    integration_timer->period_ms = crm_parse_interval_spec(value);
-
-    value = crmd_pref(config_hash, "join-finalization-timeout");
-    finalization_timer->period_ms = crm_parse_interval_spec(value);
-
-    value = crmd_pref(config_hash, XML_CONFIG_ATTR_SHUTDOWN_LOCK);
-    controld_shutdown_lock_enabled = crm_is_true(value);
-
-    free(fsa_cluster_name);
-    fsa_cluster_name = NULL;
+    value = g_hash_table_lookup(config_hash,
+                                XML_CONFIG_ATTR_SHUTDOWN_LOCK_LIMIT);
+    controld_globals.shutdown_lock_limit = crm_parse_interval_spec(value)
+                                           / 1000;
 
     value = g_hash_table_lookup(config_hash, "cluster-name");
-    if (value) {
-        fsa_cluster_name = strdup(value);
-    }
+    pcmk__str_update(&(controld_globals.cluster_name), value);
+
+    // Let subcomponents initialize their own static variables
+    controld_configure_election(config_hash);
+    controld_configure_fencing(config_hash);
+    controld_configure_fsa_timers(config_hash);
+    controld_configure_throttle(config_hash);
 
     alerts = first_named_child(output, XML_CIB_TAG_ALERTS);
     crmd_unpack_alerts(alerts);
 
     controld_set_fsa_input_flags(R_READ_CONFIG);
-    crm_trace("Triggering FSA: %s", __func__);
-    mainloop_set_trigger(fsa_source);
+    controld_trigger_fsa();
 
     g_hash_table_destroy(config_hash);
   bail:
     crm_time_free(now);
 }
 
+/*!
+ * \internal
+ * \brief Trigger read and processing of the configuration
+ *
+ * \param[in] fn    Calling function name
+ * \param[in] line  Line number where call occurred
+ */
+void
+controld_trigger_config_as(const char *fn, int line)
+{
+    if (config_read_trigger != NULL) {
+        crm_trace("%s:%d - Triggered config processing", fn, line);
+        mainloop_set_trigger(config_read_trigger);
+    }
+}
+
 gboolean
 crm_read_options(gpointer user_data)
 {
-    int call_id =
-        fsa_cib_conn->cmds->query(fsa_cib_conn,
-            "//" XML_CIB_TAG_CRMCONFIG " | //" XML_CIB_TAG_ALERTS,
-            NULL, cib_xpath | cib_scope_local);
+    cib_t *cib_conn = controld_globals.cib_conn;
+    int call_id = cib_conn->cmds->query(cib_conn,
+                                        "//" XML_CIB_TAG_CRMCONFIG
+                                        " | //" XML_CIB_TAG_ALERTS,
+                                        NULL, cib_xpath|cib_scope_local);
 
-    fsa_register_cib_callback(call_id, FALSE, NULL, config_query_callback);
+    fsa_register_cib_callback(call_id, NULL, config_query_callback);
     crm_trace("Querying the CIB... call %d", call_id);
     return TRUE;
 }
@@ -814,18 +819,22 @@ do_read_config(long long action,
                enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
     throttle_init();
-    mainloop_set_trigger(config_read);
+    controld_trigger_config();
 }
 
 void
 crm_shutdown(int nsig)
 {
-    if ((crmd_mainloop == NULL) || !g_main_loop_is_running(crmd_mainloop)) {
+    const char *value = NULL;
+    guint default_period_ms = 0;
+
+    if ((controld_globals.mainloop == NULL)
+        || !g_main_loop_is_running(controld_globals.mainloop)) {
         crmd_exit(CRM_EX_OK);
         return;
     }
 
-    if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+    if (pcmk_is_set(controld_globals.fsa_input_register, R_SHUTDOWN)) {
         crm_err("Escalating shutdown");
         register_fsa_input_before(C_SHUTDOWN, I_ERROR, NULL);
         return;
@@ -834,13 +843,15 @@ crm_shutdown(int nsig)
     controld_set_fsa_input_flags(R_SHUTDOWN);
     register_fsa_input(C_SHUTDOWN, I_SHUTDOWN, NULL);
 
-    if (shutdown_escalation_timer->period_ms == 0) {
-        const char *value = crmd_pref(NULL, XML_CONFIG_ATTR_FORCE_QUIT);
-
-        shutdown_escalation_timer->period_ms = crm_parse_interval_spec(value);
-    }
-
-    crm_notice("Initiating controller shutdown sequence " CRM_XS
-               " limit=%ums", shutdown_escalation_timer->period_ms);
-    controld_start_timer(shutdown_escalation_timer);
+    /* If shutdown timer doesn't have a period set, use the default
+     *
+     * @TODO: Evaluate whether this is still necessary. As long as
+     * config_query_callback() has been run at least once, it doesn't look like
+     * anything could have changed the timer period since then.
+     */
+    value = pcmk__cluster_option(NULL, controller_options,
+                                 PCMK__NELEM(controller_options),
+                                 XML_CONFIG_ATTR_FORCE_QUIT);
+    default_period_ms = crm_parse_interval_spec(value);
+    controld_shutdown_start_countdown(default_period_ms);
 }
