@@ -24,24 +24,9 @@
 #include <crm/cluster.h>
 
 #include <pacemaker-controld.h>
-#include <controld_matrix.h>
 
-char *fsa_our_dc = NULL;
-cib_t *fsa_cib_conn = NULL;
-char *fsa_our_dc_version = NULL;
-
-char *fsa_our_uuid = NULL;
-char *fsa_our_uname = NULL;
-
-char *fsa_cluster_name = NULL;
-
-gboolean do_fsa_stall = FALSE;
-uint64_t fsa_input_register = 0;
-uint64_t fsa_actions = A_NOTHING;
-enum crmd_fsa_state fsa_state = S_STARTING;
-
-extern uint highest_born_on;
-extern uint num_join_invites;
+//! Triggers an FSA invocation
+static crm_trigger_t *fsa_trigger = NULL;
 
 #define DOT_PREFIX "actions:trace: "
 #define do_dot_log(fmt, args...)     crm_trace( fmt, ##args)
@@ -107,7 +92,8 @@ do_fsa_action(fsa_data_t * fsa_data, long long an_action,
 {
     controld_clear_fsa_action_flags(an_action);
     crm_trace(DOT_PREFIX "\t// %s", fsa_action2string(an_action));
-    function(an_action, fsa_data->fsa_cause, fsa_state, fsa_data->fsa_input, fsa_data);
+    function(an_action, fsa_data->fsa_cause, controld_globals.fsa_state,
+             fsa_data->fsa_input, fsa_data);
 }
 
 static const uint64_t startup_actions =
@@ -155,32 +141,76 @@ do_log(long long action, enum crmd_fsa_cause cause,
     }
 }
 
+/*!
+ * \internal
+ * \brief Initialize the FSA trigger
+ */
+void
+controld_init_fsa_trigger(void)
+{
+    fsa_trigger = mainloop_add_trigger(G_PRIORITY_HIGH, crm_fsa_trigger, NULL);
+}
+
+/*!
+ * \internal
+ * \brief Destroy the FSA trigger
+ */
+void
+controld_destroy_fsa_trigger(void)
+{
+    // This basically will not work, since mainloop has a reference to it
+    mainloop_destroy_trigger(fsa_trigger);
+    fsa_trigger = NULL;
+}
+
+/*!
+ * \internal
+ * \brief Trigger an FSA invocation
+ *
+ * \param[in] fn    Calling function name
+ * \param[in] line  Line number where call occurred
+ */
+void
+controld_trigger_fsa_as(const char *fn, int line)
+{
+    if (fsa_trigger != NULL) {
+        crm_trace("%s:%d - Triggered FSA invocation", fn, line);
+        mainloop_set_trigger(fsa_trigger);
+    }
+}
+
 enum crmd_fsa_state
 s_crmd_fsa(enum crmd_fsa_cause cause)
 {
+    controld_globals_t *globals = &controld_globals;
     fsa_data_t *fsa_data = NULL;
-    uint64_t register_copy = fsa_input_register;
+    uint64_t register_copy = controld_globals.fsa_input_register;
     uint64_t new_actions = A_NOTHING;
     enum crmd_fsa_state last_state;
 
     crm_trace("FSA invoked with Cause: %s\tState: %s",
-              fsa_cause2string(cause), fsa_state2string(fsa_state));
+              fsa_cause2string(cause),
+              fsa_state2string(globals->fsa_state));
 
-    fsa_dump_actions(fsa_actions, "Initial");
+    fsa_dump_actions(controld_globals.fsa_actions, "Initial");
 
-    do_fsa_stall = FALSE;
-    if ((fsa_message_queue == NULL) && (fsa_actions != A_NOTHING)) {
+    controld_clear_global_flags(controld_fsa_is_stalled);
+    if ((controld_globals.fsa_message_queue == NULL)
+        && (controld_globals.fsa_actions != A_NOTHING)) {
         /* fake the first message so we can get into the loop */
         fsa_data = calloc(1, sizeof(fsa_data_t));
         fsa_data->fsa_input = I_NULL;
         fsa_data->fsa_cause = C_FSA_INTERNAL;
         fsa_data->origin = __func__;
         fsa_data->data_type = fsa_dt_none;
-        fsa_message_queue = g_list_append(fsa_message_queue, fsa_data);
+        controld_globals.fsa_message_queue
+            = g_list_append(controld_globals.fsa_message_queue, fsa_data);
         fsa_data = NULL;
     }
-    while ((fsa_message_queue != NULL) && !do_fsa_stall) {
-        crm_trace("Checking messages (%d remaining)", g_list_length(fsa_message_queue));
+    while ((controld_globals.fsa_message_queue != NULL)
+           && !pcmk_is_set(controld_globals.flags, controld_fsa_is_stalled)) {
+        crm_trace("Checking messages (%d remaining)",
+                  g_list_length(controld_globals.fsa_message_queue));
 
         fsa_data = get_message();
         if(fsa_data == NULL) {
@@ -194,36 +224,37 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
         fsa_dump_actions(fsa_data->actions, "Restored actions");
 
         /* get the next batch of actions */
-        new_actions = crmd_fsa_actions[fsa_data->fsa_input][fsa_state];
+        new_actions = controld_fsa_get_action(fsa_data->fsa_input);
         controld_set_fsa_action_flags(new_actions);
         fsa_dump_actions(new_actions, "New actions");
 
         if (fsa_data->fsa_input != I_NULL && fsa_data->fsa_input != I_ROUTER) {
             crm_debug("Processing %s: [ state=%s cause=%s origin=%s ]",
                       fsa_input2string(fsa_data->fsa_input),
-                      fsa_state2string(fsa_state),
+                      fsa_state2string(globals->fsa_state),
                       fsa_cause2string(fsa_data->fsa_cause), fsa_data->origin);
         }
 
         /* logging : *before* the state is changed */
-        if (pcmk_is_set(fsa_actions, A_ERROR)) {
+        if (pcmk_is_set(controld_globals.fsa_actions, A_ERROR)) {
             do_fsa_action(fsa_data, A_ERROR, do_log);
         }
-        if (pcmk_is_set(fsa_actions, A_WARN)) {
+        if (pcmk_is_set(controld_globals.fsa_actions, A_WARN)) {
             do_fsa_action(fsa_data, A_WARN, do_log);
         }
-        if (pcmk_is_set(fsa_actions, A_LOG)) {
+        if (pcmk_is_set(controld_globals.fsa_actions, A_LOG)) {
             do_fsa_action(fsa_data, A_LOG, do_log);
         }
 
         /* update state variables */
-        last_state = fsa_state;
-        fsa_state = crmd_fsa_state[fsa_data->fsa_input][fsa_state];
+        last_state = globals->fsa_state;
+        globals->fsa_state = controld_fsa_get_next_state(fsa_data->fsa_input);
 
         /*
          * Remove certain actions during shutdown
          */
-        if (fsa_state == S_STOPPING || ((fsa_input_register & R_SHUTDOWN) == R_SHUTDOWN)) {
+        if ((globals->fsa_state == S_STOPPING)
+            || pcmk_is_set(controld_globals.fsa_input_register, R_SHUTDOWN)) {
             controld_clear_fsa_action_flags(startup_actions);
         }
 
@@ -231,12 +262,12 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
          * Hook for change of state.
          * Allows actions to be added or removed when entering a state
          */
-        if (last_state != fsa_state) {
-            do_state_transition(last_state, fsa_state, fsa_data);
+        if (last_state != globals->fsa_state) {
+            do_state_transition(last_state, globals->fsa_state, fsa_data);
         } else {
             do_dot_log(DOT_PREFIX "\t// FSA input: State=%s \tCause=%s"
                        " \tInput=%s \tOrigin=%s() \tid=%d",
-                       fsa_state2string(fsa_state),
+                       fsa_state2string(globals->fsa_state),
                        fsa_cause2string(fsa_data->fsa_cause),
                        fsa_input2string(fsa_data->fsa_input), fsa_data->origin, fsa_data->id);
         }
@@ -247,27 +278,32 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
         fsa_data = NULL;
     }
 
-    if ((fsa_message_queue != NULL) || (fsa_actions != A_NOTHING)
-        || do_fsa_stall) {
+    if ((controld_globals.fsa_message_queue != NULL)
+        || (controld_globals.fsa_actions != A_NOTHING)
+        || pcmk_is_set(controld_globals.flags, controld_fsa_is_stalled)) {
+
         crm_debug("Exiting the FSA: queue=%d, fsa_actions=%#llx, stalled=%s",
-                  g_list_length(fsa_message_queue),
-                  (unsigned long long) fsa_actions, pcmk__btoa(do_fsa_stall));
+                  g_list_length(controld_globals.fsa_message_queue),
+                  (unsigned long long) controld_globals.fsa_actions,
+                  pcmk__btoa(pcmk_is_set(controld_globals.flags,
+                                         controld_fsa_is_stalled)));
     } else {
         crm_trace("Exiting the FSA");
     }
 
     /* cleanup inputs? */
-    if (register_copy != fsa_input_register) {
-        uint64_t same = register_copy & fsa_input_register;
+    if (register_copy != controld_globals.fsa_input_register) {
+        uint64_t same = register_copy & controld_globals.fsa_input_register;
 
-        fsa_dump_inputs(LOG_DEBUG, "Added", fsa_input_register ^ same);
+        fsa_dump_inputs(LOG_DEBUG, "Added",
+                        controld_globals.fsa_input_register ^ same);
         fsa_dump_inputs(LOG_DEBUG, "Removed", register_copy ^ same);
     }
 
-    fsa_dump_actions(fsa_actions, "Remaining");
+    fsa_dump_actions(controld_globals.fsa_actions, "Remaining");
     fsa_dump_queue(LOG_DEBUG);
 
-    return fsa_state;
+    return globals->fsa_state;
 }
 
 void
@@ -278,122 +314,156 @@ s_crmd_fsa_actions(fsa_data_t * fsa_data)
      * action at a time to avoid complicating the ordering.
      */
     CRM_CHECK(fsa_data != NULL, return);
-    while (fsa_actions != A_NOTHING && do_fsa_stall == FALSE) {
+    while ((controld_globals.fsa_actions != A_NOTHING)
+           && !pcmk_is_set(controld_globals.flags, controld_fsa_is_stalled)) {
 
         /* regular action processing in order of action priority
          *
          * Make sure all actions that connect to required systems
          * are performed first
          */
-        if (fsa_actions & A_ERROR) {
+        if (pcmk_is_set(controld_globals.fsa_actions, A_ERROR)) {
             do_fsa_action(fsa_data, A_ERROR, do_log);
-        } else if (fsa_actions & A_WARN) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_WARN)) {
             do_fsa_action(fsa_data, A_WARN, do_log);
-        } else if (fsa_actions & A_LOG) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_LOG)) {
             do_fsa_action(fsa_data, A_LOG, do_log);
 
             /* get out of here NOW! before anything worse happens */
-        } else if (fsa_actions & A_EXIT_1) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_EXIT_1)) {
             do_fsa_action(fsa_data, A_EXIT_1, do_exit);
 
             /* sub-system restart */
-        } else if ((fsa_actions & O_LRM_RECONNECT) == O_LRM_RECONNECT) {
+        } else if (pcmk_all_flags_set(controld_globals.fsa_actions,
+                                      O_LRM_RECONNECT)) {
             do_fsa_action(fsa_data, O_LRM_RECONNECT, do_lrm_control);
-        } else if ((fsa_actions & O_CIB_RESTART) == O_CIB_RESTART) {
+
+        } else if (pcmk_all_flags_set(controld_globals.fsa_actions,
+                                      O_CIB_RESTART)) {
             do_fsa_action(fsa_data, O_CIB_RESTART, do_cib_control);
-        } else if ((fsa_actions & O_PE_RESTART) == O_PE_RESTART) {
+
+        } else if (pcmk_all_flags_set(controld_globals.fsa_actions,
+                                      O_PE_RESTART)) {
             do_fsa_action(fsa_data, O_PE_RESTART, do_pe_control);
-        } else if ((fsa_actions & O_TE_RESTART) == O_TE_RESTART) {
+
+        } else if (pcmk_all_flags_set(controld_globals.fsa_actions,
+                                      O_TE_RESTART)) {
             do_fsa_action(fsa_data, O_TE_RESTART, do_te_control);
 
             /* essential start tasks */
-        } else if (fsa_actions & A_STARTUP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_STARTUP)) {
             do_fsa_action(fsa_data, A_STARTUP, do_startup);
-        } else if (fsa_actions & A_CIB_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_CIB_START)) {
             do_fsa_action(fsa_data, A_CIB_START, do_cib_control);
-        } else if (fsa_actions & A_HA_CONNECT) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_HA_CONNECT)) {
             do_fsa_action(fsa_data, A_HA_CONNECT, do_ha_control);
-        } else if (fsa_actions & A_READCONFIG) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_READCONFIG)) {
             do_fsa_action(fsa_data, A_READCONFIG, do_read_config);
 
             /* sub-system start/connect */
-        } else if (fsa_actions & A_LRM_CONNECT) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_LRM_CONNECT)) {
             do_fsa_action(fsa_data, A_LRM_CONNECT, do_lrm_control);
-
-        } else if (fsa_actions & A_TE_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_TE_START)) {
             do_fsa_action(fsa_data, A_TE_START, do_te_control);
-        } else if (fsa_actions & A_PE_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_PE_START)) {
             do_fsa_action(fsa_data, A_PE_START, do_pe_control);
 
             /* Timers */
-/* 		else if(fsa_actions & O_DC_TIMER_RESTART) {
-		do_fsa_action(fsa_data, O_DC_TIMER_RESTART,	     do_timer_control) */ ;
-        } else if (fsa_actions & A_DC_TIMER_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_DC_TIMER_STOP)) {
             do_fsa_action(fsa_data, A_DC_TIMER_STOP, do_timer_control);
-        } else if (fsa_actions & A_INTEGRATE_TIMER_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_INTEGRATE_TIMER_STOP)) {
             do_fsa_action(fsa_data, A_INTEGRATE_TIMER_STOP, do_timer_control);
-        } else if (fsa_actions & A_INTEGRATE_TIMER_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_INTEGRATE_TIMER_START)) {
             do_fsa_action(fsa_data, A_INTEGRATE_TIMER_START, do_timer_control);
-        } else if (fsa_actions & A_FINALIZE_TIMER_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_FINALIZE_TIMER_STOP)) {
             do_fsa_action(fsa_data, A_FINALIZE_TIMER_STOP, do_timer_control);
-        } else if (fsa_actions & A_FINALIZE_TIMER_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_FINALIZE_TIMER_START)) {
             do_fsa_action(fsa_data, A_FINALIZE_TIMER_START, do_timer_control);
 
             /*
              * Highest priority actions
              */
-        } else if (fsa_actions & A_MSG_ROUTE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_MSG_ROUTE)) {
             do_fsa_action(fsa_data, A_MSG_ROUTE, do_msg_route);
-        } else if (fsa_actions & A_RECOVER) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_RECOVER)) {
             do_fsa_action(fsa_data, A_RECOVER, do_recover);
-        } else if (fsa_actions & A_CL_JOIN_RESULT) {
-            do_fsa_action(fsa_data, A_CL_JOIN_RESULT, do_cl_join_finalize_respond);
-        } else if (fsa_actions & A_CL_JOIN_REQUEST) {
-            do_fsa_action(fsa_data, A_CL_JOIN_REQUEST, do_cl_join_offer_respond);
-        } else if (fsa_actions & A_SHUTDOWN_REQ) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_CL_JOIN_RESULT)) {
+            do_fsa_action(fsa_data, A_CL_JOIN_RESULT,
+                          do_cl_join_finalize_respond);
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_CL_JOIN_REQUEST)) {
+            do_fsa_action(fsa_data, A_CL_JOIN_REQUEST,
+                          do_cl_join_offer_respond);
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_SHUTDOWN_REQ)) {
             do_fsa_action(fsa_data, A_SHUTDOWN_REQ, do_shutdown_req);
-        } else if (fsa_actions & A_ELECTION_VOTE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_ELECTION_VOTE)) {
             do_fsa_action(fsa_data, A_ELECTION_VOTE, do_election_vote);
-        } else if (fsa_actions & A_ELECTION_COUNT) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_ELECTION_COUNT)) {
             do_fsa_action(fsa_data, A_ELECTION_COUNT, do_election_count_vote);
-        } else if (fsa_actions & A_LRM_EVENT) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_LRM_EVENT)) {
             do_fsa_action(fsa_data, A_LRM_EVENT, do_lrm_event);
 
             /*
              * High priority actions
              */
-        } else if (fsa_actions & A_STARTED) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_STARTED)) {
             do_fsa_action(fsa_data, A_STARTED, do_started);
-        } else if (fsa_actions & A_CL_JOIN_QUERY) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_CL_JOIN_QUERY)) {
             do_fsa_action(fsa_data, A_CL_JOIN_QUERY, do_cl_join_query);
-        } else if (fsa_actions & A_DC_TIMER_START) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_TIMER_START)) {
             do_fsa_action(fsa_data, A_DC_TIMER_START, do_timer_control);
 
             /*
              * Medium priority actions
              * - Membership
              */
-        } else if (fsa_actions & A_DC_TAKEOVER) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_DC_TAKEOVER)) {
             do_fsa_action(fsa_data, A_DC_TAKEOVER, do_dc_takeover);
-        } else if (fsa_actions & A_DC_RELEASE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_DC_RELEASE)) {
             do_fsa_action(fsa_data, A_DC_RELEASE, do_dc_release);
-        } else if (fsa_actions & A_DC_JOIN_FINAL) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_DC_JOIN_FINAL)) {
             do_fsa_action(fsa_data, A_DC_JOIN_FINAL, do_dc_join_final);
-        } else if (fsa_actions & A_ELECTION_CHECK) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_ELECTION_CHECK)) {
             do_fsa_action(fsa_data, A_ELECTION_CHECK, do_election_check);
-        } else if (fsa_actions & A_ELECTION_START) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_ELECTION_START)) {
             do_fsa_action(fsa_data, A_ELECTION_START, do_election_vote);
-        } else if (fsa_actions & A_DC_JOIN_OFFER_ALL) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_JOIN_OFFER_ALL)) {
             do_fsa_action(fsa_data, A_DC_JOIN_OFFER_ALL, do_dc_join_offer_all);
-        } else if (fsa_actions & A_DC_JOIN_OFFER_ONE) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_JOIN_OFFER_ONE)) {
             do_fsa_action(fsa_data, A_DC_JOIN_OFFER_ONE, do_dc_join_offer_one);
-        } else if (fsa_actions & A_DC_JOIN_PROCESS_REQ) {
-            do_fsa_action(fsa_data, A_DC_JOIN_PROCESS_REQ, do_dc_join_filter_offer);
-        } else if (fsa_actions & A_DC_JOIN_PROCESS_ACK) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_JOIN_PROCESS_REQ)) {
+            do_fsa_action(fsa_data, A_DC_JOIN_PROCESS_REQ,
+                          do_dc_join_filter_offer);
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_JOIN_PROCESS_ACK)) {
             do_fsa_action(fsa_data, A_DC_JOIN_PROCESS_ACK, do_dc_join_ack);
-        } else if (fsa_actions & A_DC_JOIN_FINALIZE) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_DC_JOIN_FINALIZE)) {
             do_fsa_action(fsa_data, A_DC_JOIN_FINALIZE, do_dc_join_finalize);
-        } else if (fsa_actions & A_CL_JOIN_ANNOUNCE) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_CL_JOIN_ANNOUNCE)) {
             do_fsa_action(fsa_data, A_CL_JOIN_ANNOUNCE, do_cl_join_announce);
 
             /*
@@ -401,44 +471,46 @@ s_crmd_fsa_actions(fsa_data_t * fsa_data)
              * Make sure the CIB is always updated before invoking the
              * scheduler, and the scheduler before the transition engine.
              */
-        } else if (fsa_actions & A_TE_HALT) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_TE_HALT)) {
             do_fsa_action(fsa_data, A_TE_HALT, do_te_invoke);
-        } else if (fsa_actions & A_TE_CANCEL) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_TE_CANCEL)) {
             do_fsa_action(fsa_data, A_TE_CANCEL, do_te_invoke);
-        } else if (fsa_actions & A_LRM_INVOKE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_LRM_INVOKE)) {
             do_fsa_action(fsa_data, A_LRM_INVOKE, do_lrm_invoke);
-        } else if (fsa_actions & A_PE_INVOKE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_PE_INVOKE)) {
             do_fsa_action(fsa_data, A_PE_INVOKE, do_pe_invoke);
-        } else if (fsa_actions & A_TE_INVOKE) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_TE_INVOKE)) {
             do_fsa_action(fsa_data, A_TE_INVOKE, do_te_invoke);
 
             /* Shutdown actions */
-        } else if (fsa_actions & A_DC_RELEASED) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_DC_RELEASED)) {
             do_fsa_action(fsa_data, A_DC_RELEASED, do_dc_release);
-        } else if (fsa_actions & A_PE_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_PE_STOP)) {
             do_fsa_action(fsa_data, A_PE_STOP, do_pe_control);
-        } else if (fsa_actions & A_TE_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_TE_STOP)) {
             do_fsa_action(fsa_data, A_TE_STOP, do_te_control);
-        } else if (fsa_actions & A_SHUTDOWN) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_SHUTDOWN)) {
             do_fsa_action(fsa_data, A_SHUTDOWN, do_shutdown);
-        } else if (fsa_actions & A_LRM_DISCONNECT) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions,
+                               A_LRM_DISCONNECT)) {
             do_fsa_action(fsa_data, A_LRM_DISCONNECT, do_lrm_control);
-        } else if (fsa_actions & A_HA_DISCONNECT) {
+
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_HA_DISCONNECT)) {
             do_fsa_action(fsa_data, A_HA_DISCONNECT, do_ha_control);
-        } else if (fsa_actions & A_CIB_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_CIB_STOP)) {
             do_fsa_action(fsa_data, A_CIB_STOP, do_cib_control);
-        } else if (fsa_actions & A_STOP) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_STOP)) {
             do_fsa_action(fsa_data, A_STOP, do_stop);
 
             /* exit gracefully */
-        } else if (fsa_actions & A_EXIT_0) {
+        } else if (pcmk_is_set(controld_globals.fsa_actions, A_EXIT_0)) {
             do_fsa_action(fsa_data, A_EXIT_0, do_exit);
 
             /* Error checking and reporting */
         } else {
             crm_err("Action %s not supported "CRM_XS" %#llx",
-                    fsa_action2string(fsa_actions),
-                    (unsigned long long) fsa_actions);
+                    fsa_action2string(controld_globals.fsa_actions),
+                    (unsigned long long) controld_globals.fsa_actions);
             register_fsa_error_adv(C_FSA_INTERNAL, I_ERROR, fsa_data, NULL,
                                    __func__);
         }
@@ -493,9 +565,9 @@ check_join_counts(fsa_data_t *msg_data)
                 "than are in membership (%d > %u)", count, npeers);
         register_fsa_input(C_FSA_INTERNAL, I_ELECTION, NULL);
 
-    } else if (saved_ccm_membership_id != crm_peer_seq) {
+    } else if (controld_globals.membership_id != crm_peer_seq) {
         crm_info("New join needed because membership changed (%llu -> %llu)",
-                 saved_ccm_membership_id, crm_peer_seq);
+                 controld_globals.membership_id, crm_peer_seq);
         register_fsa_input_before(C_FSA_INTERNAL, I_NODE_JOIN, NULL);
 
     } else {
@@ -513,7 +585,7 @@ do_state_transition(enum crmd_fsa_state cur_state,
     int count = 0;
     gboolean clear_recovery_bit = TRUE;
 #if 0
-    uint64_t original_fsa_actions = fsa_actions;
+    uint64_t original_fsa_actions = controld_globals.fsa_actions;
 #endif
 
     enum crmd_fsa_cause cause = msg_data->fsa_cause;
@@ -546,10 +618,10 @@ do_state_transition(enum crmd_fsa_state cur_state,
                msg_data->origin);
 
     if (next_state != S_ELECTION && cur_state != S_RELEASE_DC) {
-        controld_stop_election_timer();
+        controld_stop_current_election_timeout();
     }
 #if 0
-    if ((fsa_input_register & R_SHUTDOWN)) {
+    if ((controld_globals.fsa_input_register & R_SHUTDOWN)) {
         controld_set_fsa_action_flags(A_DC_TIMER_STOP);
     }
 #endif
@@ -568,11 +640,8 @@ do_state_transition(enum crmd_fsa_state cur_state,
     if (next_state != S_PENDING) {
         controld_set_fsa_action_flags(A_DC_TIMER_STOP);
     }
-    if (next_state != S_ELECTION) {
-        highest_born_on = 0;
-    }
     if (next_state != S_IDLE) {
-        controld_stop_timer(recheck_timer);
+        controld_stop_recheck_timer();
     }
 
     if (cur_state == S_FINALIZE_JOIN && next_state == S_POLICY_ENGINE) {
@@ -581,7 +650,10 @@ do_state_transition(enum crmd_fsa_state cur_state,
 
     switch (next_state) {
         case S_PENDING:
-            fsa_cib_conn->cmds->set_secondary(fsa_cib_conn, cib_scope_local);
+            {
+                cib_t *cib_conn = controld_globals.cib_conn;
+                cib_conn->cmds->set_secondary(cib_conn, cib_scope_local);
+            }
             update_dc(NULL);
             break;
 
@@ -590,15 +662,15 @@ do_state_transition(enum crmd_fsa_state cur_state,
             break;
 
         case S_NOT_DC:
-            election_trigger->counter = 0;
+            controld_reset_counter_election_timer();
             purge_stonith_cleanup();
 
-            if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+            if (pcmk_is_set(controld_globals.fsa_input_register, R_SHUTDOWN)) {
                 crm_info("(Re)Issuing shutdown request now" " that we have a new DC");
                 controld_set_fsa_action_flags(A_SHUTDOWN_REQ);
             }
-            CRM_LOG_ASSERT(fsa_our_dc != NULL);
-            if (fsa_our_dc == NULL) {
+            CRM_LOG_ASSERT(controld_globals.dc_name != NULL);
+            if (controld_globals.dc_name == NULL) {
                 crm_err("Reached S_NOT_DC without a DC" " being recorded");
             }
             break;
@@ -626,7 +698,7 @@ do_state_transition(enum crmd_fsa_state cur_state,
             break;
 
         case S_POLICY_ENGINE:
-            election_trigger->counter = 0;
+            controld_reset_counter_election_timer();
             CRM_LOG_ASSERT(AM_I_DC);
             if (cause == C_TIMER_POPPED) {
                 crm_info("Progressed to state %s after %s",
@@ -643,7 +715,7 @@ do_state_transition(enum crmd_fsa_state cur_state,
 
         case S_IDLE:
             CRM_LOG_ASSERT(AM_I_DC);
-            if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+            if (pcmk_is_set(controld_globals.fsa_input_register, R_SHUTDOWN)) {
                 crm_info("(Re)Issuing shutdown request now" " that we are the DC");
                 controld_set_fsa_action_flags(A_SHUTDOWN_REQ);
             }
@@ -661,8 +733,9 @@ do_state_transition(enum crmd_fsa_state cur_state,
     }
 
 #if 0
-    if (original_fsa_actions != fsa_actions) {
-        fsa_dump_actions(original_fsa_actions ^ fsa_actions, "New actions");
+    if (original_fsa_actions != controld_globals.fsa_actions) {
+        fsa_dump_actions(original_fsa_actions ^ controld_globals.fsa_actions,
+                         "New actions");
     }
 #endif
 }
