@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2020 the Pacemaker project contributors
+ * Copyright 2016-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,6 +13,7 @@
 
 #include <crm/crm.h>
 #include <crm/services.h>
+#include <crm/services_internal.h>
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_internal.h>
 #include <crm/common/alerts_internal.h>
@@ -28,17 +29,17 @@ static inline void
 add_inflight_alert(int call_id, int timeout)
 {
     if (inflight_alerts == NULL) {
-        inflight_alerts = g_hash_table_new(g_direct_hash, g_direct_equal);
+        inflight_alerts = pcmk__intkey_table(NULL);
     }
-    g_hash_table_insert(inflight_alerts, GINT_TO_POINTER(call_id),
-                        GINT_TO_POINTER(timeout));
+    pcmk__intkey_table_insert(inflight_alerts, call_id,
+                              GINT_TO_POINTER(timeout));
 }
 
 static inline void
 remove_inflight_alert(int call_id)
 {
     if (inflight_alerts != NULL) {
-        g_hash_table_remove(inflight_alerts, GINT_TO_POINTER(call_id));
+        pcmk__intkey_table_remove(inflight_alerts, call_id);
     }
 }
 
@@ -70,9 +71,29 @@ alert_complete(svc_action_t *action)
 {
     struct alert_cb_s *cb_data = (struct alert_cb_s *) (action->cb_data);
 
+    CRM_CHECK(cb_data != NULL, return);
+
     remove_inflight_alert(cb_data->call_id);
-    crm_debug("Alert pid %d for %s completed with rc=%d",
-              action->pid, cb_data->client_id, action->rc);
+
+    if (action->status != PCMK_EXEC_DONE) {
+        const char *reason = services__exit_reason(action);
+
+        crm_notice("Could not send alert: %s%s%s%s " CRM_XS " client=%s",
+                   pcmk_exec_status_str(action->status),
+                   (reason == NULL)? "" : " (",
+                   (reason == NULL)? "" : reason,
+                   (reason == NULL)? "" : ")",
+                   cb_data->client_id);
+
+    } else if (action->rc != 0) {
+        crm_notice("Alert [%d] completed but exited with status %d "
+                   CRM_XS " client=%s",
+                   action->pid, action->rc, cb_data->client_id);
+
+    } else {
+        crm_debug("Alert [%d] completed " CRM_XS " client=%s",
+                  action->pid, cb_data->client_id);
+    }
 
     free(cb_data->client_id);
     free(action->cb_data);
@@ -110,18 +131,27 @@ process_lrmd_alert_exec(pcmk__client_t *client, uint32_t id, xmlNode *request)
                             ++alert_sequence_no);
 
     cb_data = calloc(1, sizeof(struct alert_cb_s));
-    CRM_CHECK(cb_data != NULL,
-              rc = -ENOMEM; goto err);
+    if (cb_data == NULL) {
+        rc = -errno;
+        goto err;
+    }
 
     /* coverity[deref_ptr] False Positive */
     cb_data->client_id = strdup(client->id);
-    CRM_CHECK(cb_data->client_id != NULL,
-              rc = -ENOMEM; goto err);
+    if (cb_data->client_id == NULL) {
+        rc = -errno;
+        goto err;
+    }
 
     crm_element_value_int(request, F_LRMD_CALLID, &(cb_data->call_id));
 
     action = services_alert_create(alert_id, alert_path, alert_timeout, params,
                                    alert_sequence_no, cb_data);
+    if (action->rc != PCMK_OCF_UNKNOWN) {
+        rc = -E2BIG;
+        goto err;
+    }
+
     rc = services_action_user(action, CRM_DAEMON_USER);
     if (rc < 0) {
         goto err;
@@ -140,9 +170,7 @@ err:
         }
         free(cb_data);
     }
-    if (action) {
-        services_action_free(action);
-    }
+    services_action_free(action);
     return rc;
 }
 
