@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2021 the Pacemaker project contributors
+ * Copyright 2012-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,6 +8,7 @@
  */
 
 #include <crm_internal.h>
+#include <crm/fencing/internal.h>
 
 #include <glib.h>
 
@@ -21,6 +22,7 @@
 #include <unistd.h>
 
 #include <crm/crm.h>
+#include <crm/fencing/internal.h>
 #include <crm/services.h>
 #include <crm/services_internal.h>
 #include <crm/common/mainloop.h>
@@ -96,7 +98,7 @@ typedef struct lrmd_cmd_s {
 } lrmd_cmd_t;
 
 static void cmd_finalize(lrmd_cmd_t * cmd, lrmd_rsc_t * rsc);
-static gboolean lrmd_rsc_dispatch(gpointer user_data);
+static gboolean execute_resource_action(gpointer user_data);
 static void cancel_all_recurring(lrmd_rsc_t * rsc, const char *client_id);
 
 #ifdef PCMK__TIME_USE_CGT
@@ -110,7 +112,7 @@ static void cancel_all_recurring(lrmd_rsc_t * rsc, const char *client_id);
  * \return true if timespec has been set (i.e. is nonzero), false otherwise
  */
 static inline bool
-time_is_set(struct timespec *timespec)
+time_is_set(const struct timespec *timespec)
 {
     return (timespec != NULL) &&
            ((timespec->tv_sec != 0) || (timespec->tv_nsec != 0));
@@ -145,7 +147,7 @@ get_current_time(struct timespec *t_current, struct timespec *t_orig)
  *       24 days or more.
  */
 static int
-time_diff_ms(struct timespec *now, struct timespec *old)
+time_diff_ms(const struct timespec *now, const struct timespec *old)
 {
     int diff_ms = 0;
 
@@ -170,7 +172,7 @@ time_diff_ms(struct timespec *now, struct timespec *old)
  * command, so we report the entire time since then and not just the time since
  * the most recent command (for recurring and systemd operations).
  *
- * \param[in] cmd  Executor command object to reset
+ * \param[in,out] cmd  Executor command object to reset
  *
  * \note It's not obvious what the queued time should be for a systemd
  *       start/stop operation, which might go like this:
@@ -191,7 +193,7 @@ cmd_original_times(lrmd_cmd_t * cmd)
 #endif
 
 static inline bool
-action_matches(lrmd_cmd_t *cmd, const char *action, guint interval_ms)
+action_matches(const lrmd_cmd_t *cmd, const char *action, guint interval_ms)
 {
     return (cmd->interval_ms == interval_ms)
            && pcmk__str_eq(cmd->action, action, pcmk__str_casei);
@@ -206,7 +208,7 @@ action_matches(lrmd_cmd_t *cmd, const char *action, guint interval_ms)
  * \param[in] queue_time_ms  Queue time in milliseconds, if known
  */
 static void
-log_finished(lrmd_cmd_t *cmd, int exec_time_ms, int queue_time_ms)
+log_finished(const lrmd_cmd_t *cmd, int exec_time_ms, int queue_time_ms)
 {
     int log_level = LOG_INFO;
     GString *str = g_string_sized_new(100); // reasonable starting size
@@ -215,8 +217,8 @@ log_finished(lrmd_cmd_t *cmd, int exec_time_ms, int queue_time_ms)
         log_level = LOG_DEBUG;
     }
 
-    g_string_printf(str, "%s %s (call %d",
-                    cmd->rsc_id, cmd->action, cmd->call_id);
+    g_string_append_printf(str, "%s %s (call %d",
+                           cmd->rsc_id, cmd->action, cmd->call_id);
     if (cmd->last_pid != 0) {
         g_string_append_printf(str, ", PID %d", cmd->last_pid);
     }
@@ -224,21 +226,22 @@ log_finished(lrmd_cmd_t *cmd, int exec_time_ms, int queue_time_ms)
         g_string_append_printf(str, ") exited with status %d",
                                cmd->result.exit_status);
     } else {
-        g_string_append_printf(str, ") could not be executed: %s",
-                               pcmk_exec_status_str(cmd->result.execution_status));
+        pcmk__g_strcat(str, ") could not be executed: ",
+                       pcmk_exec_status_str(cmd->result.execution_status),
+                       NULL);
     }
     if (cmd->result.exit_reason != NULL) {
-        g_string_append_printf(str, " (%s)", cmd->result.exit_reason);
+        pcmk__g_strcat(str, " (", cmd->result.exit_reason, ")", NULL);
     }
 
 #ifdef PCMK__TIME_USE_CGT
-    g_string_append_printf(str, " (execution time %s",
-                           pcmk__readable_interval(exec_time_ms));
+    pcmk__g_strcat(str, " (execution time ",
+                   pcmk__readable_interval(exec_time_ms), NULL);
     if (queue_time_ms > 0) {
-        g_string_append_printf(str, " after being queued %s",
-                               pcmk__readable_interval(queue_time_ms));
+        pcmk__g_strcat(str, " after being queued ",
+                       pcmk__readable_interval(queue_time_ms), NULL);
     }
-    g_string_append(str, ")");
+    g_string_append_c(str, ')');
 #endif
 
     do_crm_log(log_level, "%s", str->str);
@@ -282,8 +285,12 @@ build_rsc_from_xml(xmlNode * msg)
     rsc->class = crm_element_value_copy(rsc_xml, F_LRMD_CLASS);
     rsc->provider = crm_element_value_copy(rsc_xml, F_LRMD_PROVIDER);
     rsc->type = crm_element_value_copy(rsc_xml, F_LRMD_TYPE);
-    rsc->work = mainloop_add_trigger(G_PRIORITY_HIGH, lrmd_rsc_dispatch, rsc);
-    rsc->st_probe_rc = -ENODEV; // if stonith, initialize to "not running"
+    rsc->work = mainloop_add_trigger(G_PRIORITY_HIGH, execute_resource_action,
+                                     rsc);
+
+    // Initialize fence device probes (to return "not running")
+    pcmk__set_result(&rsc->fence_probe_result, CRM_EX_ERROR,
+                     PCMK_EXEC_NO_FENCE_DEVICE, NULL);
     return rsc;
 }
 
@@ -413,11 +420,14 @@ start_delay_helper(gpointer data)
 /*!
  * \internal
  * \brief Check whether a list already contains the equivalent of a given action
+ *
+ * \param[in] action_list  List to search
+ * \param[in] cmd          Action to search for
  */
 static lrmd_cmd_t *
-find_duplicate_action(GList *action_list, lrmd_cmd_t *cmd)
+find_duplicate_action(const GList *action_list, const lrmd_cmd_t *cmd)
 {
-    for (GList *item = action_list; item != NULL; item = item->next) {
+    for (const GList *item = action_list; item != NULL; item = item->next) {
         lrmd_cmd_t *dup = item->data;
 
         if (action_matches(cmd, dup->action, dup->interval_ms)) {
@@ -590,17 +600,14 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
 #endif
     log_finished(cmd, exec_time, queue_time);
 
-    /* if the first notify result for a cmd has already been sent earlier, and the
-     * the option to only send notifies on result changes is set. Check to see
-     * if the last result is the same as the new one. If so, suppress this update */
-    if (cmd->first_notify_sent && (cmd->call_opts & lrmd_opt_notify_changes_only)) {
-        if ((cmd->last_notify_rc == cmd->result.exit_status) &&
-            (cmd->last_notify_op_status == cmd->result.execution_status)) {
-
-            /* only send changes */
-            return;
-        }
-
+    /* If the originator requested to be notified only for changes in recurring
+     * operation results, skip the notification if the result hasn't changed.
+     */
+    if (cmd->first_notify_sent
+        && pcmk_is_set(cmd->call_opts, lrmd_opt_notify_changes_only)
+        && (cmd->last_notify_rc == cmd->result.exit_status)
+        && (cmd->last_notify_op_status == cmd->result.execution_status)) {
+        return;
     }
 
     cmd->first_notify_sent = true;
@@ -656,10 +663,12 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
             hash2smartfield((gpointer) key, (gpointer) value, args);
         }
     }
-    if (cmd->client_id && (cmd->call_opts & lrmd_opt_notify_orig_only)) {
+    if ((cmd->client_id != NULL)
+        && pcmk_is_set(cmd->call_opts, lrmd_opt_notify_orig_only)) {
+
         pcmk__client_t *client = pcmk__find_client_by_id(cmd->client_id);
 
-        if (client) {
+        if (client != NULL) {
             send_client_notify(client->id, client, notify);
         }
     } else {
@@ -747,55 +756,6 @@ cmd_finalize(lrmd_cmd_t * cmd, lrmd_rsc_t * rsc)
     }
 }
 
-static int
-stonith2uniform_rc(const char *action, int rc)
-{
-    switch (rc) {
-        case pcmk_ok:
-            rc = PCMK_OCF_OK;
-            break;
-
-        case -ENODEV:
-            /* This should be possible only for probes in practice, but
-             * interpret for all actions to be safe.
-             */
-            if (pcmk__str_eq(action, "monitor", pcmk__str_casei)) {
-                rc = PCMK_OCF_NOT_RUNNING;
-            } else if (pcmk__str_eq(action, "stop", pcmk__str_casei)) {
-                rc = PCMK_OCF_OK;
-            } else {
-                rc = PCMK_OCF_NOT_INSTALLED;
-            }
-            break;
-
-        case -EOPNOTSUPP:
-            rc = PCMK_OCF_UNIMPLEMENT_FEATURE;
-            break;
-
-        default:
-            rc = PCMK_OCF_UNKNOWN_ERROR;
-            break;
-    }
-    return rc;
-}
-
-static int
-action_get_uniform_rc(svc_action_t *action)
-{
-    lrmd_cmd_t *cmd = action->cb_data;
-
-    if (pcmk__str_eq(action->standard, PCMK_RESOURCE_CLASS_STONITH,
-                            pcmk__str_casei)) {
-        return stonith2uniform_rc(cmd->action, action->rc);
-    } else {
-        enum ocf_exitcode code = services_result2ocf(action->standard,
-                                                     cmd->action, action->rc);
-
-        // Cast variable instead of function return to keep compilers happy
-        return (int) code;
-    }
-}
-
 struct notify_new_client_data {
     xmlNode *notify;
     pcmk__client_t *new_client;
@@ -834,7 +794,7 @@ client_disconnect_cleanup(const char *client_id)
 
     g_hash_table_iter_init(&iter, rsc_list);
     while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & rsc)) {
-        if (rsc->call_opts & lrmd_opt_drop_recurring) {
+        if (pcmk_all_flags_set(rsc->call_opts, lrmd_opt_drop_recurring)) {
             /* This client is disconnecting, drop any recurring operations
              * it may have initiated on the resource */
             cancel_all_recurring(rsc, client_id);
@@ -847,6 +807,7 @@ action_complete(svc_action_t * action)
 {
     lrmd_rsc_t *rsc;
     lrmd_cmd_t *cmd = action->cb_data;
+    enum ocf_exitcode code;
 
 #ifdef PCMK__TIME_USE_CGT
     const char *rclass = NULL;
@@ -866,8 +827,12 @@ action_complete(svc_action_t * action)
 #endif
 
     cmd->last_pid = action->pid;
-    pcmk__set_result(&(cmd->result), action_get_uniform_rc(action),
+
+    // Cast variable instead of function return to keep compilers happy
+    code = services_result2ocf(action->standard, cmd->action, action->rc);
+    pcmk__set_result(&(cmd->result), (int) code,
                      action->status, services__exit_reason(action));
+
     rsc = cmd->rsc_id ? g_hash_table_lookup(rsc_list, cmd->rsc_id) : NULL;
 
 #ifdef PCMK__TIME_USE_CGT
@@ -878,7 +843,7 @@ action_complete(svc_action_t * action)
     }
 
     if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
-        if ((cmd->result.exit_status == PCMK_OCF_OK)
+        if (pcmk__result_ok(&(cmd->result))
             && pcmk__strcase_any_of(cmd->action, "start", "stop", NULL)) {
             /* systemd returns from start and stop actions after the action
              * begins, not after it completes. We have to jump through a few
@@ -894,7 +859,7 @@ action_complete(svc_action_t * action)
             if (cmd->result.execution_status == PCMK_EXEC_PENDING) {
                 goagain = true;
 
-            } else if ((cmd->result.exit_status == PCMK_OCF_OK)
+            } else if (pcmk__result_ok(&(cmd->result))
                        && pcmk__str_eq(cmd->real_action, "stop", pcmk__str_casei)) {
                 goagain = true;
 
@@ -927,12 +892,12 @@ action_complete(svc_action_t * action)
 #if SUPPORT_NAGIOS
     if (rsc && pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
         if (action_matches(cmd, "monitor", 0)
-            && (cmd->result.exit_status == PCMK_OCF_OK)) {
+            && pcmk__result_ok(&(cmd->result))) {
             /* Successfully executed --version for the nagios plugin */
             cmd->result.exit_status = PCMK_OCF_NOT_RUNNING;
 
         } else if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)
-                   && (cmd->result.exit_status != PCMK_OCF_OK)) {
+                   && !pcmk__result_ok(&(cmd->result))) {
 #ifdef PCMK__TIME_USE_CGT
             goagain = true;
 #endif
@@ -955,7 +920,7 @@ action_complete(svc_action_t * action)
             cmd->start_delay = delay;
             cmd->timeout = timeout_left;
 
-            if (cmd->result.exit_status == PCMK_OCF_OK) {
+            if (pcmk__result_ok(&(cmd->result))) {
                 crm_debug("%s %s may still be in progress: re-scheduling (elapsed=%dms, remaining=%dms, start_delay=%dms)",
                           cmd->rsc_id, cmd->real_action, time_sum, timeout_left, delay);
 
@@ -1001,83 +966,83 @@ action_complete(svc_action_t * action)
 
 /*!
  * \internal
- * \brief Determine operation status of a stonith operation
+ * \brief Process the result of a fence device action (start, stop, or monitor)
  *
- * Non-stonith resource operations get their operation status directly from the
- * service library, but the fencer does not have an equivalent, so we must infer
- * an operation status from the fencer API's return code.
- *
- * \param[in] action       Name of action performed on stonith resource
- * \param[in] interval_ms  Action interval
- * \param[in] rc           Action result from fencer
- *
- * \return Operation status corresponding to fencer API return code
+ * \param[in,out] cmd               Fence device action that completed
+ * \param[in]     exit_status       Fencer API exit status for action
+ * \param[in]     execution_status  Fencer API execution status for action
+ * \param[in]     exit_reason       Human-friendly detail, if action failed
  */
-static int
-stonith_rc2status(const char *action, guint interval_ms, int rc)
-{
-    int status = PCMK_EXEC_DONE;
-
-    switch (rc) {
-        case pcmk_ok:
-            break;
-
-        case -EOPNOTSUPP:
-        case -EPROTONOSUPPORT:
-            status = PCMK_EXEC_NOT_SUPPORTED;
-            break;
-
-        case -ETIME:
-        case -ETIMEDOUT:
-            status = PCMK_EXEC_TIMEOUT;
-            break;
-
-        case -ENOTCONN:
-        case -ECOMM:
-            // Couldn't talk to fencer
-            status = PCMK_EXEC_ERROR;
-            break;
-
-        case -ENODEV:
-            // The device is not registered with the fencer
-            status = PCMK_EXEC_ERROR;
-            break;
-
-        default:
-            break;
-    }
-    return status;
-}
-
 static void
-stonith_action_complete(lrmd_cmd_t * cmd, int rc)
+stonith_action_complete(lrmd_cmd_t *cmd, int exit_status,
+                        enum pcmk_exec_status execution_status,
+                        const char *exit_reason)
 {
     // This can be NULL if resource was removed before command completed
     lrmd_rsc_t *rsc = g_hash_table_lookup(rsc_list, cmd->rsc_id);
 
-    cmd->result.exit_status = stonith2uniform_rc(cmd->action, rc);
+    // Simplify fencer exit status to uniform exit status
+    if (exit_status != CRM_EX_OK) {
+        exit_status = PCMK_OCF_UNKNOWN_ERROR;
+    }
 
-    /* This function may be called with status already set to cancelled, if a
-     * pending action was aborted. Otherwise, we need to determine status from
-     * the fencer return code.
-     */
-    if (cmd->result.execution_status != PCMK_EXEC_CANCELLED) {
-        cmd->result.execution_status = stonith_rc2status(cmd->action,
-                                                         cmd->interval_ms, rc);
+    if (cmd->result.execution_status == PCMK_EXEC_CANCELLED) {
+        /* An in-flight fence action was cancelled. The execution status is
+         * already correct, so don't overwrite it.
+         */
+        execution_status = PCMK_EXEC_CANCELLED;
 
-        // Certain successful actions change the known state of the resource
-        if ((rsc != NULL) && (cmd->result.exit_status == PCMK_OCF_OK)) {
-            if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)) {
-                rsc->st_probe_rc = pcmk_ok; // maps to PCMK_OCF_OK
-            } else if (pcmk__str_eq(cmd->action, "stop", pcmk__str_casei)) {
-                rsc->st_probe_rc = -ENODEV; // maps to PCMK_OCF_NOT_RUNNING
-            }
+    } else {
+        /* Some execution status codes have specific meanings for the fencer
+         * that executor clients may not expect, so map them to a simple error
+         * status.
+         */
+        switch (execution_status) {
+            case PCMK_EXEC_NOT_CONNECTED:
+            case PCMK_EXEC_INVALID:
+                execution_status = PCMK_EXEC_ERROR;
+                break;
+
+            case PCMK_EXEC_NO_FENCE_DEVICE:
+                /* This should be possible only for probes in practice, but
+                 * interpret for all actions to be safe.
+                 */
+                if (pcmk__str_eq(cmd->action, CRMD_ACTION_STATUS,
+                                 pcmk__str_none)) {
+                    exit_status = PCMK_OCF_NOT_RUNNING;
+
+                } else if (pcmk__str_eq(cmd->action, CRMD_ACTION_STOP,
+                                        pcmk__str_none)) {
+                    exit_status = PCMK_OCF_OK;
+
+                } else {
+                    exit_status = PCMK_OCF_NOT_INSTALLED;
+                }
+                execution_status = PCMK_EXEC_ERROR;
+                break;
+
+            case PCMK_EXEC_NOT_SUPPORTED:
+                exit_status = PCMK_OCF_UNIMPLEMENT_FEATURE;
+                break;
+
+            default:
+                break;
         }
     }
 
-    // Give the user more detail than an OCF code
-    if (rc != -pcmk_err_generic) {
-        cmd->result.exit_reason = strdup(pcmk_strerror(rc));
+    pcmk__set_result(&cmd->result, exit_status, execution_status, exit_reason);
+
+    // Certain successful actions change the known state of the resource
+    if ((rsc != NULL) && pcmk__result_ok(&(cmd->result))) {
+
+        if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)) {
+            pcmk__set_result(&rsc->fence_probe_result, CRM_EX_OK,
+                             PCMK_EXEC_DONE, NULL); // "running"
+
+        } else if (pcmk__str_eq(cmd->action, "stop", pcmk__str_casei)) {
+            pcmk__set_result(&rsc->fence_probe_result, CRM_EX_ERROR,
+                             PCMK_EXEC_NO_FENCE_DEVICE, NULL); // "not running"
+        }
     }
 
     /* The recurring timer should not be running at this point in any case, but
@@ -1100,56 +1065,70 @@ stonith_action_complete(lrmd_cmd_t * cmd, int rc)
 static void
 lrmd_stonith_callback(stonith_t * stonith, stonith_callback_data_t * data)
 {
-    stonith_action_complete(data->userdata, data->rc);
+    if ((data == NULL) || (data->userdata == NULL)) {
+        crm_err("Ignoring fence action result: "
+                "Invalid callback arguments (bug?)");
+    } else {
+        stonith_action_complete((lrmd_cmd_t *) data->userdata,
+                                stonith__exit_status(data),
+                                stonith__execution_status(data),
+                                stonith__exit_reason(data));
+    }
 }
 
 void
 stonith_connection_failed(void)
 {
     GHashTableIter iter;
-    GList *cmd_list = NULL;
-    GList *cmd_iter = NULL;
     lrmd_rsc_t *rsc = NULL;
-    char *key = NULL;
+
+    crm_warn("Connection to fencer lost (any pending operations for "
+             "fence devices will be considered failed)");
 
     g_hash_table_iter_init(&iter, rsc_list);
-    while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & rsc)) {
-        if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
-            /* If we registered this fence device, we don't know whether the
-             * fencer still has the registration or not. Cause future probes to
-             * return PCMK_OCF_UNKNOWN_ERROR until the resource is stopped or
-             * started successfully. This is especially important if the
-             * controller also went away (possibly due to a cluster layer
-             * restart) and won't receive our client notification of any
-             * monitors finalized below.
-             */
-            if (rsc->st_probe_rc == pcmk_ok) {
-                rsc->st_probe_rc = pcmk_err_generic;
-            }
+    while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &rsc)) {
+        if (!pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH,
+                          pcmk__str_none)) {
+            continue;
+        }
 
-            if (rsc->active) {
-                cmd_list = g_list_append(cmd_list, rsc->active);
-            }
-            if (rsc->recurring_ops) {
-                cmd_list = g_list_concat(cmd_list, rsc->recurring_ops);
-            }
-            if (rsc->pending_ops) {
-                cmd_list = g_list_concat(cmd_list, rsc->pending_ops);
-            }
-            rsc->pending_ops = rsc->recurring_ops = NULL;
+        /* If we registered this fence device, we don't know whether the
+         * fencer still has the registration or not. Cause future probes to
+         * return an error until the resource is stopped or started
+         * successfully. This is especially important if the controller also
+         * went away (possibly due to a cluster layer restart) and won't
+         * receive our client notification of any monitors finalized below.
+         */
+        if (rsc->fence_probe_result.execution_status == PCMK_EXEC_DONE) {
+            pcmk__set_result(&rsc->fence_probe_result, CRM_EX_ERROR,
+                             PCMK_EXEC_NOT_CONNECTED,
+                             "Lost connection to fencer");
+        }
+
+        // Consider any active, pending, or recurring operations as failed
+
+        for (GList *op = rsc->recurring_ops; op != NULL; op = op->next) {
+            lrmd_cmd_t *cmd = op->data;
+
+            /* This won't free a recurring op but instead restart its timer.
+             * If cmd is rsc->active, this will set rsc->active to NULL, so we
+             * don't have to worry about finalizing it a second time below.
+             */
+            stonith_action_complete(cmd,
+                                    CRM_EX_ERROR, PCMK_EXEC_NOT_CONNECTED,
+                                    "Lost connection to fencer");
+        }
+
+        if (rsc->active != NULL) {
+            rsc->pending_ops = g_list_prepend(rsc->pending_ops, rsc->active);
+        }
+        while (rsc->pending_ops != NULL) {
+            // This will free the op and remove it from rsc->pending_ops
+            stonith_action_complete((lrmd_cmd_t *) rsc->pending_ops->data,
+                                    CRM_EX_ERROR, PCMK_EXEC_NOT_CONNECTED,
+                                    "Lost connection to fencer");
         }
     }
-
-    if (!cmd_list) {
-        return;
-    }
-
-    crm_err("Connection to fencer failed, finalizing %d pending operations",
-            g_list_length(cmd_list));
-    for (cmd_iter = cmd_list; cmd_iter; cmd_iter = cmd_iter->next) {
-        stonith_action_complete(cmd_iter->data, -ENOTCONN);
-    }
-    g_list_free(cmd_list);
 }
 
 /*!
@@ -1159,14 +1138,15 @@ stonith_connection_failed(void)
  * Start a stonith resource by registering it with the fencer.
  * (Stonith agents don't have a start command.)
  *
- * \param[in] stonith_api  Connection to fencer
- * \param[in] rsc          Stonith resource to start
- * \param[in] cmd          Start command to execute
+ * \param[in,out] stonith_api  Connection to fencer
+ * \param[in]     rsc          Stonith resource to start
+ * \param[in]     cmd          Start command to execute
  *
  * \return pcmk_ok on success, -errno otherwise
  */
 static int
-execd_stonith_start(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
+execd_stonith_start(stonith_t *stonith_api, const lrmd_rsc_t *rsc,
+                    const lrmd_cmd_t *cmd)
 {
     char *key = NULL;
     char *value = NULL;
@@ -1204,8 +1184,8 @@ execd_stonith_start(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
  * Stop a stonith resource by unregistering it with the fencer.
  * (Stonith agents don't have a stop command.)
  *
- * \param[in] stonith_api  Connection to fencer
- * \param[in] rsc          Stonith resource to stop
+ * \param[in,out] stonith_api  Connection to fencer
+ * \param[in]     rsc          Stonith resource to stop
  *
  * \return pcmk_ok on success, -errno otherwise
  */
@@ -1223,9 +1203,9 @@ execd_stonith_stop(stonith_t *stonith_api, const lrmd_rsc_t *rsc)
  * \internal
  * \brief Initiate a stonith resource agent recurring "monitor" action
  *
- * \param[in] stonith_api  Connection to fencer
- * \param[in] rsc          Stonith resource to monitor
- * \param[in] cmd          Monitor command being executed
+ * \param[in,out] stonith_api  Connection to fencer
+ * \param[in,out] rsc          Stonith resource to monitor
+ * \param[in]     cmd          Monitor command being executed
  *
  * \return pcmk_ok if monitor was successfully initiated, -errno otherwise
  */
@@ -1248,19 +1228,30 @@ execd_stonith_monitor(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
 }
 
 static void
-lrmd_rsc_execute_stonith(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
+execute_stonith_action(lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
 {
     int rc = 0;
     bool do_monitor = FALSE;
 
     stonith_t *stonith_api = get_stonith_connection();
 
-    if (!stonith_api) {
-        rc = -ENOTCONN;
+    if (pcmk__str_eq(cmd->action, "monitor", pcmk__str_casei)
+        && (cmd->interval_ms == 0)) {
+        // Probes don't require a fencer connection
+        stonith_action_complete(cmd, rsc->fence_probe_result.exit_status,
+                                rsc->fence_probe_result.execution_status,
+                                rsc->fence_probe_result.exit_reason);
+        return;
+
+    } else if (stonith_api == NULL) {
+        stonith_action_complete(cmd, PCMK_OCF_UNKNOWN_ERROR,
+                                PCMK_EXEC_NOT_CONNECTED,
+                                "No connection to fencer");
+        return;
 
     } else if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)) {
         rc = execd_stonith_start(stonith_api, rsc, cmd);
-        if (rc == 0) {
+        if (rc == pcmk_ok) {
             do_monitor = TRUE;
         }
 
@@ -1268,11 +1259,13 @@ lrmd_rsc_execute_stonith(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
         rc = execd_stonith_stop(stonith_api, rsc);
 
     } else if (pcmk__str_eq(cmd->action, "monitor", pcmk__str_casei)) {
-        if (cmd->interval_ms > 0) {
-            do_monitor = TRUE;
-        } else {
-            rc = rsc->st_probe_rc;
-        }
+        do_monitor = TRUE;
+
+    } else {
+        stonith_action_complete(cmd, PCMK_OCF_UNIMPLEMENT_FEATURE,
+                                PCMK_EXEC_ERROR,
+                                "Invalid fence device action (bug?)");
+        return;
     }
 
     if (do_monitor) {
@@ -1283,11 +1276,14 @@ lrmd_rsc_execute_stonith(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
         }
     }
 
-    stonith_action_complete(cmd, rc);
+    stonith_action_complete(cmd,
+                            ((rc == pcmk_ok)? CRM_EX_OK : CRM_EX_ERROR),
+                            stonith__legacy2status(rc),
+                            ((rc == -pcmk_err_generic)? NULL : pcmk_strerror(rc)));
 }
 
-static int
-lrmd_rsc_execute_service_lib(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
+static void
+execute_nonstonith_action(lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
 {
     svc_action_t *action = NULL;
     GHashTable *params_copy = NULL;
@@ -1304,7 +1300,8 @@ lrmd_rsc_execute_service_lib(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
         && pcmk__str_eq(cmd->action, "stop", pcmk__str_casei)) {
 
         cmd->result.exit_status = PCMK_OCF_OK;
-        goto exec_done;
+        cmd_finalize(cmd, rsc);
+        return;
     }
 #endif
 
@@ -1319,39 +1316,48 @@ lrmd_rsc_execute_service_lib(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
     if (action == NULL) {
         pcmk__set_result(&(cmd->result), PCMK_OCF_UNKNOWN_ERROR,
                          PCMK_EXEC_ERROR, strerror(ENOMEM));
-        goto exec_done;
+        cmd_finalize(cmd, rsc);
+        return;
     }
 
     if (action->rc != PCMK_OCF_UNKNOWN) {
         pcmk__set_result(&(cmd->result), action->rc, action->status,
                          services__exit_reason(action));
         services_action_free(action);
-        goto exec_done;
+        cmd_finalize(cmd, rsc);
+        return;
     }
 
     action->cb_data = cmd;
 
     if (services_action_async(action, action_complete)) {
-        /* When services_action_async() returns TRUE, the callback might have
-         * been called -- in this case action_complete(), which might free cmd,
-         * so cmd cannot be used here.
+        /* The services library has taken responsibility for the action. It
+         * could be pending, blocked, or merged into a duplicate recurring
+         * action, in which case the action callback (action_complete())
+         * will be called when the action completes, otherwise the callback has
+         * already been called.
+         *
+         * action_complete() calls cmd_finalize() which can free cmd, so cmd
+         * cannot be used here.
          */
-        return TRUE;
+    } else {
+        /* This is a recurring action that is not being cancelled and could not
+         * be initiated. It has been rescheduled, and the action callback
+         * (action_complete()) has been called, which in this case has already
+         * called cmd_finalize(), which in this case should only reset (not
+         * free) cmd.
+         */
+
+        pcmk__set_result(&(cmd->result), action->rc, action->status,
+                         services__exit_reason(action));
+        services_action_free(action);
     }
-
-    pcmk__set_result(&(cmd->result), action->rc, action->status,
-                     services__exit_reason(action));
-    services_action_free(action);
-    action = NULL;
-
-  exec_done:
-    cmd_finalize(cmd, rsc);
-    return TRUE;
 }
 
 static gboolean
-lrmd_rsc_execute(lrmd_rsc_t * rsc)
+execute_resource_action(gpointer user_data)
 {
+    lrmd_rsc_t *rsc = (lrmd_rsc_t *) user_data;
     lrmd_cmd_t *cmd = NULL;
 
     CRM_CHECK(rsc != NULL, return FALSE);
@@ -1393,20 +1399,14 @@ lrmd_rsc_execute(lrmd_rsc_t * rsc)
     log_execute(cmd);
 
     if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
-        lrmd_rsc_execute_stonith(rsc, cmd);
+        execute_stonith_action(rsc, cmd);
     } else {
-        lrmd_rsc_execute_service_lib(rsc, cmd);
+        execute_nonstonith_action(rsc, cmd);
     }
 
     return TRUE;
 }
 
-static gboolean
-lrmd_rsc_dispatch(gpointer user_data)
-{
-    return lrmd_rsc_execute(user_data);
-}
-
 void
 free_rsc(gpointer data)
 {
@@ -1472,7 +1472,7 @@ process_lrmd_signon(pcmk__client_t *client, xmlNode *request, int call_id,
                     xmlNode **reply)
 {
     int rc = pcmk_ok;
-    const char *is_ipc_provider = crm_element_value(request, F_LRMD_IS_IPC_PROVIDER);
+    time_t now = time(NULL);
     const char *protocol_version = crm_element_value(request, F_LRMD_PROTOCOL_VERSION);
 
     if (compare_version(protocol_version, LRMD_MIN_PROTOCOL_VERSION) < 0) {
@@ -1481,9 +1481,12 @@ process_lrmd_signon(pcmk__client_t *client, xmlNode *request, int call_id,
         rc = -EPROTO;
     }
 
-    if (crm_is_true(is_ipc_provider)) {
+    if (pcmk__xe_attr_is_true(request, F_LRMD_IS_IPC_PROVIDER)) {
 #ifdef PCMK__COMPILE_REMOTE
-        if ((client->remote != NULL) && client->remote->tls_handshake_complete) {
+        if ((client->remote != NULL)
+            && pcmk_is_set(client->flags,
+                           pcmk__client_tls_handshake_complete)) {
+
             // This is a remote connection from a cluster node's controller
             ipc_proxy_add_provider(client);
         } else {
@@ -1498,6 +1501,7 @@ process_lrmd_signon(pcmk__client_t *client, xmlNode *request, int call_id,
     crm_xml_add(*reply, F_LRMD_OPERATION, CRM_OP_REGISTER);
     crm_xml_add(*reply, F_LRMD_CLIENTID, client->id);
     crm_xml_add(*reply, F_LRMD_PROTOCOL_VERSION, LRMD_PROTOCOL_VERSION);
+    crm_xml_add_ll(*reply, PCMK__XA_UPTIME, now - start_time);
 
     return rc;
 }
@@ -1574,7 +1578,7 @@ process_lrmd_rsc_unregister(pcmk__client_t *client, uint32_t id,
 
     if (rsc->active) {
         /* let the caller know there are still active ops on this rsc to watch for */
-        crm_trace("Operation (0x%p) still in progress for unregistered resource %s",
+        crm_trace("Operation (%p) still in progress for unregistered resource %s",
                   rsc->active, rsc_id);
         rc = -EINPROGRESS;
     }
