@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2021 the Pacemaker project contributors
+ * Copyright 2015-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -17,15 +17,17 @@
 
 typedef struct pcmk__time_us pcmk__time_hr_t;
 
-pcmk__time_hr_t *pcmk__time_hr_convert(pcmk__time_hr_t *target, crm_time_t *dt);
-void pcmk__time_set_hr_dt(crm_time_t *target, pcmk__time_hr_t *hr_dt);
-pcmk__time_hr_t *pcmk__time_timeval_hr_convert(pcmk__time_hr_t *target,
-                                               struct timeval *tv);
+pcmk__time_hr_t *pcmk__time_hr_convert(pcmk__time_hr_t *target,
+                                       const crm_time_t *dt);
+void pcmk__time_set_hr_dt(crm_time_t *target, const pcmk__time_hr_t *hr_dt);
+pcmk__time_hr_t *pcmk__time_hr_now(time_t *epoch);
 pcmk__time_hr_t *pcmk__time_hr_new(const char *date_time);
 void pcmk__time_hr_free(pcmk__time_hr_t *hr_dt);
-char *pcmk__time_format_hr(const char *format, pcmk__time_hr_t *hr_dt);
-const char *pcmk__epoch2str(time_t *when);
+char *pcmk__time_format_hr(const char *format, const pcmk__time_hr_t *hr_dt);
+char *pcmk__epoch2str(const time_t *source, uint32_t flags);
+char *pcmk__timespec2str(const struct timespec *ts, uint32_t flags);
 const char *pcmk__readable_interval(guint interval_ms);
+crm_time_t *pcmk__copy_timet(time_t source);
 
 struct pcmk__time_us {
     int years;
