@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2021 the Pacemaker project contributors
+ * Copyright 2015-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -17,29 +17,37 @@
 #include <crm/common/mainloop.h>        // mainloop_io_t, ipc_client_callbacks
 #include <crm/common/output_internal.h> // pcmk__output_t
 #include <crm/common/remote_internal.h> // pcmk__remote_t
-#include <crm/lrmd.h>                   // lrmd_t, lrmd_event_data_t
+#include <crm/lrmd.h>           // lrmd_t, lrmd_event_data_t, lrmd_rsc_info_t
 
 int lrmd__new(lrmd_t **api, const char *nodename, const char *server, int port);
 
-int lrmd_send_attribute_alert(lrmd_t *lrmd, GList *alert_list,
+int lrmd_send_attribute_alert(lrmd_t *lrmd, const GList *alert_list,
                               const char *node, uint32_t nodeid,
                               const char *attr_name, const char *attr_value);
-int lrmd_send_node_alert(lrmd_t *lrmd, GList *alert_list,
+int lrmd_send_node_alert(lrmd_t *lrmd, const GList *alert_list,
                          const char *node, uint32_t nodeid, const char *state);
-int lrmd_send_fencing_alert(lrmd_t *lrmd, GList *alert_list,
+int lrmd_send_fencing_alert(lrmd_t *lrmd, const GList *alert_list,
                             const char *target, const char *task,
                             const char *desc, int op_rc);
-int lrmd_send_resource_alert(lrmd_t *lrmd, GList *alert_list,
-                             const char *node, lrmd_event_data_t *op);
+int lrmd_send_resource_alert(lrmd_t *lrmd, const GList *alert_list,
+                             const char *node, const lrmd_event_data_t *op);
 
 int lrmd__remote_send_xml(pcmk__remote_t *session, xmlNode *msg, uint32_t id,
                           const char *msg_type);
 
+int lrmd__metadata_async(const lrmd_rsc_info_t *rsc,
+                         void (*callback)(int pid,
+                                          const pcmk__action_result_t *result,
+                                          void *user_data),
+                         void *user_data);
+
 void lrmd__set_result(lrmd_event_data_t *event, enum ocf_exitcode rc,
                       int op_status, const char *exit_reason);
 
 void lrmd__reset_result(lrmd_event_data_t *event);
 
+time_t lrmd__uptime(lrmd_t *lrmd);
+
 /* Shared functions for IPC proxy back end */
 
 typedef struct remote_proxy_s {
