@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,13 +7,65 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef XML_TAGS__H
-#  define XML_TAGS__H
+#ifndef PCMK__CRM_MSG_XML__H
+#  define PCMK__CRM_MSG_XML__H
+
+#  include <crm/common/xml.h>
+
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/msg_xml_compat.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/* This file defines constants for various XML syntax (mainly element and
+ * attribute names).
+ *
+ * For consistency, new constants should start with "PCMK_", followed by "XE"
+ * for XML element names, "XA" for XML attribute names, and "META" for meta
+ * attribute names. Old names that don't follow this policy should eventually be
+ * deprecated and replaced with names that do.
+ */
+
+/*
+ * XML elements
+ */
+
+#define PCMK_XE_DATE_EXPRESSION             "date_expression"
+#define PCMK_XE_OP_EXPRESSION               "op_expression"
+
+/* This has been deprecated as a CIB element (an alias for <clone> with
+ * "promotable" set to "true") since 2.0.0.
+ */
+#define PCMK_XE_PROMOTABLE_LEGACY           "master"
+
+#define PCMK_XE_RSC_EXPRESSION              "rsc_expression"
+
+
+/*
+ * XML attributes
+ */
+
+/* These have been deprecated as CIB <clone> element attributes (aliases for
+ * "promoted-max" and "promoted-node-max") since 2.0.0.
+ */
+#define PCMK_XA_PROMOTED_MAX_LEGACY         "master-max"
+#define PCMK_XA_PROMOTED_NODE_MAX_LEGACY    "master-node-max"
+
+
+/*
+ * Meta attributes
+ */
+
+#define PCMK_META_ENABLED                   "enabled"
+
+
+/*
+ * Older constants that don't follow current naming
+ */
+
 #  ifndef F_ORIG
 #    define F_ORIG    "src"
 #  endif
@@ -79,7 +131,6 @@ extern "C" {
 #  define XML_ATTR_CRM_VERSION		"crm_feature_set"
 #  define XML_ATTR_DIGEST		"digest"
 #  define XML_ATTR_VALIDATION		"validate-with"
-#  define XML_ATTR_RA_VERSION		"ra-version"
 
 #  define XML_ATTR_QUORUM_PANIC		"no-quorum-panic"
 #  define XML_ATTR_HAVE_QUORUM		"have-quorum"
@@ -94,6 +145,7 @@ extern "C" {
 #  define XML_ATTR_VERSION		"version"
 #  define XML_ATTR_DESC			"description"
 #  define XML_ATTR_ID			"id"
+#  define XML_ATTR_NAME			"name"
 #  define XML_ATTR_IDREF			"id-ref"
 #  define XML_ATTR_ID_LONG		"long-id"
 #  define XML_ATTR_TYPE			"type"
@@ -116,7 +168,6 @@ extern "C" {
 #  define XML_ATTR_RESPONSE		"response"
 
 #  define XML_ATTR_UNAME		"uname"
-#  define XML_ATTR_UUID			"id"
 #  define XML_ATTR_REFERENCE		"reference"
 
 #  define XML_CRM_TAG_PING		"ping_response"
@@ -130,6 +181,7 @@ extern "C" {
 #  define XML_PING_ATTR_PACEMAKERDSTATE_RUNNING "running"
 #  define XML_PING_ATTR_PACEMAKERDSTATE_SHUTTINGDOWN "shutting_down"
 #  define XML_PING_ATTR_PACEMAKERDSTATE_SHUTDOWNCOMPLETE "shutdown_complete"
+#  define XML_PING_ATTR_PACEMAKERDSTATE_REMOTE "remote"
 
 #  define XML_TAG_FRAGMENT		"cib_fragment"
 
@@ -170,9 +222,6 @@ extern "C" {
 #  define XML_TAG_ATTR_SETS	   	"instance_attributes"
 #  define XML_TAG_META_SETS	   	"meta_attributes"
 #  define XML_TAG_ATTRS			"attributes"
-#  define XML_TAG_RSC_VER_ATTRS	"rsc_versioned_attrs"
-#  define XML_TAG_OP_VER_ATTRS         "op_versioned_attrs"
-#  define XML_TAG_OP_VER_META          "op_versioned_meta"
 #  define XML_TAG_PARAMS		"parameters"
 #  define XML_TAG_PARAM			"param"
 #  define XML_TAG_UTILIZATION		"utilization"
@@ -181,7 +230,6 @@ extern "C" {
 #  define XML_CIB_TAG_RESOURCE	  	"primitive"
 #  define XML_CIB_TAG_GROUP	  	"group"
 #  define XML_CIB_TAG_INCARNATION	"clone"
-#  define XML_CIB_TAG_MASTER        "master" // deprecated since 2.0.0
 #  define XML_CIB_TAG_CONTAINER		"bundle"
 
 #  define XML_CIB_TAG_RSC_TEMPLATE	"template"
@@ -197,8 +245,6 @@ extern "C" {
 #  define XML_RSC_ATTR_PROMOTABLE       "promotable"
 #  define XML_RSC_ATTR_PROMOTED_MAX     "promoted-max"
 #  define XML_RSC_ATTR_PROMOTED_NODEMAX "promoted-node-max"
-#  define XML_RSC_ATTR_MASTER_MAX       "master-max"      // deprecated since 2.0.0
-#  define XML_RSC_ATTR_MASTER_NODEMAX   "master-node-max" // deprecated since 2.0.0
 #  define XML_RSC_ATTR_MANAGED		"is-managed"
 #  define XML_RSC_ATTR_TARGET_ROLE	"target-role"
 #  define XML_RSC_ATTR_UNIQUE		"globally-unique"
@@ -208,7 +254,6 @@ extern "C" {
 #  define XML_RSC_ATTR_FAIL_TIMEOUT	"failure-timeout"
 #  define XML_RSC_ATTR_MULTIPLE		"multiple-active"
 #  define XML_RSC_ATTR_REQUIRES		"requires"
-#  define XML_RSC_ATTR_PROVIDES		"provides"
 #  define XML_RSC_ATTR_CONTAINER	"container"
 #  define XML_RSC_ATTR_INTERNAL_RSC	"internal_rsc"
 #  define XML_RSC_ATTR_MAINTENANCE	"maintenance"
@@ -218,6 +263,7 @@ extern "C" {
 #  define XML_RSC_ATTR_REMOTE_RA_ADDR   "addr"
 #  define XML_RSC_ATTR_REMOTE_RA_SERVER "server"
 #  define XML_RSC_ATTR_REMOTE_RA_PORT   "port"
+#  define XML_RSC_ATTR_CRITICAL         "critical"
 
 #  define XML_REMOTE_ATTR_RECONNECT_INTERVAL "reconnect_interval"
 
@@ -236,7 +282,9 @@ extern "C" {
 #  define XML_AGENT_ATTR_CLASS		"class"
 #  define XML_AGENT_ATTR_PROVIDER	"provider"
 
+//! \deprecated Do not use (will be removed in a future release)
 #  define XML_CIB_ATTR_REPLACE       	"replace"
+
 #  define XML_CIB_ATTR_SOURCE       	"source"
 
 #  define XML_CIB_ATTR_PRIORITY     	"priority"
@@ -326,7 +374,12 @@ extern "C" {
 #  define XML_COLOC_ATTR_TARGET		"with-rsc"
 #  define XML_COLOC_ATTR_TARGET_ROLE	"with-rsc-role"
 #  define XML_COLOC_ATTR_NODE_ATTR	"node-attribute"
+#  define XML_COLOC_ATTR_INFLUENCE          "influence"
+
+//! \deprecated Deprecated since 2.1.5
 #  define XML_COLOC_ATTR_SOURCE_INSTANCE	"rsc-instance"
+
+//! \deprecated Deprecated since 2.1.5
 #  define XML_COLOC_ATTR_TARGET_INSTANCE	"with-rsc-instance"
 
 #  define XML_LOC_ATTR_SOURCE           "rsc"
@@ -336,10 +389,14 @@ extern "C" {
 #  define XML_ORDER_ATTR_THEN		"then"
 #  define XML_ORDER_ATTR_FIRST_ACTION	"first-action"
 #  define XML_ORDER_ATTR_THEN_ACTION	"then-action"
-#  define XML_ORDER_ATTR_FIRST_INSTANCE	"first-instance"
-#  define XML_ORDER_ATTR_THEN_INSTANCE	"then-instance"
 #  define XML_ORDER_ATTR_KIND		"kind"
 
+//! \deprecated Deprecated since 2.1.5
+#  define XML_ORDER_ATTR_FIRST_INSTANCE	"first-instance"
+
+//! \deprecated Deprecated since 2.1.5
+#  define XML_ORDER_ATTR_THEN_INSTANCE	"then-instance"
+
 #  define XML_TICKET_ATTR_TICKET	"ticket"
 #  define XML_TICKET_ATTR_LOSS_POLICY	"loss-policy"
 
@@ -420,8 +477,6 @@ extern "C" {
 #  define XML_DIFF_PATH                 "path"
 #  define XML_DIFF_POSITION             "position"
 
-#  include <crm/common/xml.h>
-
 #  define ID(x) crm_element_value(x, XML_ATTR_ID)
 #  define TYPE(x) crm_element_name(x)
 
