@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -55,7 +55,23 @@ typedef struct resource_object_functions_s {
     pe_node_t *(*location) (const pe_resource_t*, GList**, int);
     void (*free) (pe_resource_t*);
     void (*count) (pe_resource_t*);
-    gboolean (*is_filtered) (pe_resource_t*, GList *, gboolean);
+    gboolean (*is_filtered) (const pe_resource_t*, GList *, gboolean);
+
+    /*!
+     * \brief
+     * \internal Find a node (and optionally count all) where resource is active
+     *
+     * \param[in]  rsc          Resource to check
+     * \param[out] count_all    If not NULL, set this to count of active nodes
+     * \param[out] count_clean  If not NULL, set this to count of clean nodes
+     *
+     * \return A node where the resource is active, preferring the source node
+     *         if the resource is involved in a partial migration or a clean,
+     *         online node if the resource's "requires" is "quorum" or
+     *         "nothing", or NULL if the resource is inactive.
+     */
+    pe_node_t *(*active_node)(const pe_resource_t *rsc, unsigned int *count_all,
+                              unsigned int *count_clean);
 } resource_object_functions_t;
 
 typedef struct resource_alloc_functions_s resource_alloc_functions_t;
@@ -247,7 +263,7 @@ struct pe_node_shared_s {
 
 struct pe_node_s {
     int weight;
-    gboolean fixed;
+    gboolean fixed; //!< \deprecated Will be removed in a future release
     int count;
     struct pe_node_shared_s *details;
     int rsc_discover_mode;
@@ -274,6 +290,7 @@ struct pe_node_s {
 #  define pe_rsc_critical                   0x00008000ULL
 
 #  define pe_rsc_failed                     0x00010000ULL
+#  define pe_rsc_detect_loop                0x00020000ULL
 #  define pe_rsc_runnable                   0x00040000ULL
 #  define pe_rsc_start_pending              0x00080000ULL
 
@@ -287,6 +304,7 @@ struct pe_node_s {
 #  define pe_rsc_allow_migrate              0x00800000ULL
 
 #  define pe_rsc_failure_ignored            0x01000000ULL
+#  define pe_rsc_replica_container          0x02000000ULL
 #  define pe_rsc_maintenance                0x04000000ULL
 #  define pe_rsc_is_container               0x08000000ULL
 
@@ -358,6 +376,13 @@ struct pe_resource_s {
     gboolean is_remote_node;
     gboolean exclusive_discover;
 
+    /* Pay special attention to whether you want to use rsc_cons_lhs and
+     * rsc_cons directly, which include only colocations explicitly involving
+     * this resource, or call libpacemaker's pcmk__with_this_colocations() and
+     * pcmk__this_with_colocations() functions, which may return relevant
+     * colocations involving the resource's ancestors as well.
+     */
+
     //!@{
     //! This field should be treated as internal to Pacemaker
     GList *rsc_cons_lhs;      // List of pcmk__colocation_t*
@@ -387,8 +412,10 @@ struct pe_resource_s {
     pe_resource_t *container;
     GList *fillers;
 
+    // @COMPAT These should be made const at next API compatibility break
     pe_node_t *pending_node;    // Node on which pending_task is happening
     pe_node_t *lock_node;       // Resource is shutdown-locked to this node
+
     time_t lock_time;           // When shutdown lock started
 
     /* Resource parameters may have node-attribute-based rules, which means the
