@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2022 the Pacemaker project contributors
+ * Copyright 2019-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -79,25 +79,36 @@ typedef struct {
 } pcmk_injections_t;
 
 /*!
- * \brief Get controller status
+ * \brief Get and output controller status
  *
- * \param[in,out] xml                The destination for the result, as an XML tree.
- * \param[in]     dest_node          Destination node for request
- * \param[in]     message_timeout_ms Message timeout
+ * \param[in,out] xml                 Destination for the result, as an XML tree
+ * \param[in]     node_name           Name of node whose status is desired
+ *                                    (\p NULL for DC)
+ * \param[in]     message_timeout_ms  How long to wait for a reply from the
+ *                                    \p pacemaker-controld API. If 0,
+ *                                    \p pcmk_ipc_dispatch_sync will be used.
+ *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
+ *                                    be used.
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_controller_status(xmlNodePtr *xml, char *dest_node, unsigned int message_timeout_ms);
+int pcmk_controller_status(xmlNodePtr *xml, const char *node_name,
+                           unsigned int message_timeout_ms);
 
 /*!
- * \brief Get designated controller
+ * \brief Get and output designated controller node name
  *
- * \param[in,out] xml                The destination for the result, as an XML tree.
- * \param[in]     message_timeout_ms Message timeout
+ * \param[in,out] xml                 Destination for the result, as an XML tree
+ * \param[in]     message_timeout_ms  How long to wait for a reply from the
+ *                                    \p pacemaker-controld API. If 0,
+ *                                    \p pcmk_ipc_dispatch_sync will be used.
+ *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
+ *                                    be used.
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_designated_controller(xmlNodePtr *xml, unsigned int message_timeout_ms);
+int pcmk_designated_controller(xmlNodePtr *xml,
+                               unsigned int message_timeout_ms);
 
 /*!
  * \brief Free a :pcmk_injections_t structure
@@ -106,6 +117,68 @@ int pcmk_designated_controller(xmlNodePtr *xml, unsigned int message_timeout_ms)
  */
 void pcmk_free_injections(pcmk_injections_t *injections);
 
+/*!
+ * \brief Get and optionally output node info corresponding to a node ID from
+ *        the controller
+ *
+ * \param[in,out] xml                 Destination for the result, as an XML tree
+ * \param[in,out] node_id             ID of node whose name to get. If \p NULL
+ *                                    or 0, get the local node name. If not
+ *                                    \p NULL, store the true node ID here on
+ *                                    success.
+ * \param[out]    node_name           If not \p NULL, where to store the node
+ *                                    name
+ * \param[out]    uuid                If not \p NULL, where to store the node
+ *                                    UUID
+ * \param[out]    state               If not \p NULL, where to store the
+ *                                    membership state
+ * \param[out]    is_remote           If not \p NULL, where to store whether the
+ *                                    node is a Pacemaker Remote node
+ * \param[out]    have_quorum         If not \p NULL, where to store whether the
+ *                                    node has quorum
+ * \param[in]     show_output         Whether to output the node info
+ * \param[in]     message_timeout_ms  How long to wait for a reply from the
+ *                                    \p pacemaker-controld API. If 0,
+ *                                    \p pcmk_ipc_dispatch_sync will be used.
+ *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
+ *                                    be used.
+ *
+ * \return Standard Pacemaker return code
+ *
+ * \note The caller is responsible for freeing \p *node_name, \p *uuid, and
+ *       \p *state using \p free().
+ */
+int pcmk_query_node_info(xmlNodePtr *xml, uint32_t *node_id, char **node_name,
+                         char **uuid, char **state, bool *have_quorum,
+                         bool *is_remote, bool show_output,
+                         unsigned int message_timeout_ms);
+
+/*!
+ * \brief Get the node name corresponding to a node ID from the controller
+ *
+ * \param[in,out] xml                 Destination for the result, as an XML tree
+ * \param[in,out] node_id             ID of node whose name to get (or 0 for the
+ *                                    local node)
+ * \param[out]    node_name           If not \p NULL, where to store the node
+ *                                    name
+ * \param[in]     message_timeout_ms  How long to wait for a reply from the
+ *                                    \p pacemaker-controld API. If 0,
+ *                                    \p pcmk_ipc_dispatch_sync will be used.
+ *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
+ *                                    be used.
+ *
+ * \return Standard Pacemaker return code
+ *
+ * \note The caller is responsible for freeing \p *node_name using \p free().
+ */
+static inline int
+pcmk_query_node_name(xmlNodePtr *xml, uint32_t node_id, char **node_name,
+                     unsigned int message_timeout_ms)
+{
+    return pcmk_query_node_info(xml, &node_id, node_name, NULL, NULL, NULL,
+                                NULL, false, message_timeout_ms);
+}
+
 /*!
  * \brief Get and output \p pacemakerd status
  *
@@ -114,10 +187,8 @@ void pcmk_free_injections(pcmk_injections_t *injections);
  * \param[in]     message_timeout_ms  How long to wait for a reply from the
  *                                    \p pacemakerd API. If 0,
  *                                    \p pcmk_ipc_dispatch_sync will be used.
- *                                    If positive, \p pcmk_ipc_dispatch_main
- *                                    will be used, and a new mainloop will be
- *                                    created for this purpose (freed before
- *                                    return).
+ *                                    Otherwise, \p pcmk_ipc_dispatch_poll will
+ *                                    be used.
  *
  * \return Standard Pacemaker return code
  */
@@ -127,67 +198,66 @@ int pcmk_pacemakerd_status(xmlNodePtr *xml, const char *ipc_name,
 /*!
  * \brief Calculate and output resource operation digests
  *
- * \param[out] xml        Where to store XML with result
- * \param[in]  rsc        Resource to calculate digests for
- * \param[in]  node       Node whose operation history should be used
- * \param[in]  overrides  Hash table of configuration parameters to override
- * \param[in]  data_set   Cluster working set (with status)
+ * \param[out]    xml        Where to store XML with result
+ * \param[in,out] rsc        Resource to calculate digests for
+ * \param[in]     node       Node whose operation history should be used
+ * \param[in]     overrides  Hash table of configuration parameters to override
+ * \param[in]     data_set   Cluster working set (with status)
  *
  * \return Standard Pacemaker return code
  */
 int pcmk_resource_digests(xmlNodePtr *xml, pe_resource_t *rsc,
-                          pe_node_t *node, GHashTable *overrides,
+                          const pe_node_t *node, GHashTable *overrides,
                           pe_working_set_t *data_set);
 
-/**
- * \brief Simulate a cluster's response to events.
+/*!
+ * \brief Simulate a cluster's response to events
  *
- * This high-level function essentially implements crm_simulate(8).  It operates
- * on an input CIB file and various lists of events that can be simulated.  It
+ * This high-level function essentially implements crm_simulate(8). It operates
+ * on an input CIB file and various lists of events that can be simulated. It
  * optionally writes out a variety of artifacts to show the results of the
- * simulation.  Output can be modified with various flags.
+ * simulation. Output can be modified with various flags.
  *
- * \param[in,out] xml          The destination for the result, as an XML tree.
- * \param[in,out] data_set     Working set for the cluster.
- * \param[in]     events       A structure containing cluster events
+ * \param[in,out] xml          The destination for the result, as an XML tree
+ * \param[in,out] data_set     Working set for the cluster
+ * \param[in]     injections   A structure containing cluster events
  *                             (node up/down, tickets, injected operations)
  * \param[in]     flags        A bitfield of :pcmk_sim_flags to modify
- *                             operation of the simulation.
+ *                             operation of the simulation
  * \param[in]     section_opts Which portions of the cluster status output
  *                             should be displayed?
- * \param[in]     use_date     The date to set the cluster's time to
- *                             (may be NULL).
+ * \param[in]     use_date     Date to set the cluster's time to (may be NULL)
  * \param[in]     input_file   The source CIB file, which may be overwritten by
- *                             this function (may be NULL).
+ *                             this function (may be NULL)
  * \param[in]     graph_file   Where to write the XML-formatted transition graph
  *                             (may be NULL, in which case no file will be
- *                             written).
+ *                             written)
  * \param[in]     dot_file     Where to write the dot(1) formatted transition
  *                             graph (may be NULL, in which case no file will
- *                             be written).  See \p pcmk__write_sim_dotfile().
+ *                             be written)
  *
  * \return Standard Pacemaker return code
  */
 int pcmk_simulate(xmlNodePtr *xml, pe_working_set_t *data_set,
-                  pcmk_injections_t *injections, unsigned int flags,
-                  unsigned int section_opts, char *use_date, char *input_file,
-                  char *graph_file, char *dot_file);
+                  const pcmk_injections_t *injections, unsigned int flags,
+                  unsigned int section_opts, const char *use_date,
+                  const char *input_file, const char *graph_file,
+                  const char *dot_file);
 
 /*!
  * \brief Get nodes list
  *
- * \param[in,out] xml                The destination for the result, as an XML tree.
- * \param[in]     node_types         Node type(s) to return (default: all)
+ * \param[in,out] xml         The destination for the result, as an XML tree
+ * \param[in]     node_types  Node type(s) to return (default: all)
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_list_nodes(xmlNodePtr *xml, char *node_types);
+int pcmk_list_nodes(xmlNodePtr *xml, const char *node_types);
 
 /*!
- * \brief Output the current status of the cluster, formatted in the same way
- *        that `crm_mon --output-as=xml` would.
+ * \brief Output cluster status formatted like `crm_mon --output-as=xml`
  *
- * \param[in,out] xml The destination for the result, as an XML tree.
+ * \param[in,out] xml  The destination for the result, as an XML tree
  *
  * \return Standard Pacemaker return code
  */
@@ -227,7 +297,7 @@ pcmk_check_rule(xmlNodePtr *xml, xmlNodePtr input, const crm_time_t *date,
     return pcmk_check_rules(xml, input, date, rule_ids);
 }
 
-/*
+/*!
  * \enum pcmk_rc_disp_flags
  * \brief Bit flags to control which fields of result code info are displayed
  */
@@ -238,7 +308,7 @@ enum pcmk_rc_disp_flags {
     pcmk_rc_disp_desc = (1 << 2),   //!< Display result code description
 };
 
-/*
+/*!
  * \brief Display the name and/or description of a result code
  *
  * \param[in,out] xml    The destination for the result, as an XML tree
@@ -272,19 +342,19 @@ int pcmk_list_result_codes(xmlNodePtr *xml, enum pcmk_result_type type,
 /*!
  * \brief Ask the cluster to perform fencing
  *
- * \param[in] st        A connection to the fencer API
- * \param[in] target    The node that should be fenced
- * \param[in] action    The fencing action (on, off, reboot) to perform
- * \param[in] name      Who requested the fence action?
- * \param[in] timeout   How long to wait for the operation to complete (in ms)
- * \param[in] tolerance If a successful action for \p target happened within
- *                      this many ms, return 0 without performing the action
- *                      again
- * \param[in] delay     Apply this delay (in milliseconds) before initiating the
- *                      fencing action (a value of -1 applies no delay and also
- *                      disables any fencing delay from pcmk_delay_base and
- *                      pcmk_delay_max)
- * \param[out] reason   If not NULL, where to put descriptive failure reason
+ * \param[in,out] st        A connection to the fencer API
+ * \param[in]     target    The node that should be fenced
+ * \param[in]     action    The fencing action (on, off, reboot) to perform
+ * \param[in]     name      Who requested the fence action?
+ * \param[in]     timeout   How long to wait for operation to complete (in ms)
+ * \param[in]     tolerance If a successful action for \p target happened within
+ *                          this many ms, return 0 without performing the action
+ *                          again
+ * \param[in]     delay     Apply this delay (in milliseconds) before initiating
+ *                          fencing action (-1 applies no delay and also
+ *                          disables any fencing delay from pcmk_delay_base and
+ *                          pcmk_delay_max)
+ * \param[out]     reason   If not NULL, where to put descriptive failure reason
  *
  * \return Standard Pacemaker return code
  * \note If \p reason is not NULL, the caller is responsible for freeing its
@@ -295,35 +365,33 @@ int pcmk_request_fencing(stonith_t *st, const char *target, const char *action,
                          unsigned int tolerance, int delay, char **reason);
 
 /*!
- * \brief List the fencing operations that have occurred for a specific node.
+ * \brief List the fencing operations that have occurred for a specific node
  *
  * \note If \p xml is not NULL, it will be freed first and the previous
  *       contents lost.
  *
- * \param[in,out] xml       The destination for the result, as an XML tree.
- * \param[in]     st        A connection to the STONITH API.
- * \param[in]     target    The node to get history for.
- * \param[in]     timeout   How long to wait for the operation to complete (in ms).
- * \param[in]     quiet     Suppress most output.
- * \param[in]     verbose   Include additional output.
- * \param[in]     broadcast Gather fencing history from all nodes.
- * \param[in]     cleanup   Clean up fencing history after listing.
+ * \param[in,out] xml       The destination for the result, as an XML tree
+ * \param[in,out] st        A connection to the fencer API
+ * \param[in]     target    The node to get history for
+ * \param[in]     timeout   How long to wait for operation to complete (in ms)
+ * \param[in]     quiet     Suppress most output
+ * \param[in]     verbose   Include additional output
+ * \param[in]     broadcast Gather fencing history from all nodes
+ * \param[in]     cleanup   Clean up fencing history after listing
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_fence_history(xmlNodePtr *xml, stonith_t *st, char *target,
+int pcmk_fence_history(xmlNodePtr *xml, stonith_t *st, const char *target,
                        unsigned int timeout, bool quiet, int verbose,
                        bool broadcast, bool cleanup);
 
 /*!
- * \brief List all installed STONITH agents.
+ * \brief List all installed fence agents
  *
- * \note If \p xml is not NULL, it will be freed first and the previous
- *       contents lost.
- *
- * \param[in,out] xml     The destination for the result, as an XML tree.
- * \param[in]     st      A connection to the STONITH API.
- * \param[in]     timeout How long to wait for the operation to complete (in ms).
+ * \param[in,out] xml      The destination for the result, as an XML tree (if
+ *                         not NULL, previous contents will be freed and lost)
+ * \param[in,out] st       A connection to the fencer API
+ * \param[in]     timeout  How long to wait for operation to complete (in ms)
  *
  * \return Standard Pacemaker return code
  */
@@ -332,27 +400,23 @@ int pcmk_fence_installed(xmlNodePtr *xml, stonith_t *st, unsigned int timeout);
 /*!
  * \brief When was a device last fenced?
  *
- * \note If \p xml is not NULL, it will be freed first and the previous
- *       contents lost.
- *
- * \param[in,out] xml       The destination for the result, as an XML tree.
- * \param[in]     target    The node that was fenced.
- * \param[in]     as_nodeid
+ * \param[in,out] xml        The destination for the result, as an XML tree (if
+ *                           not NULL, previous contents will be freed and lost)
+ * \param[in]     target     The node that was fenced
+ * \param[in]     as_nodeid  If true, \p target has node ID rather than name
  *
  * \return Standard Pacemaker return code
  */
 int pcmk_fence_last(xmlNodePtr *xml, const char *target, bool as_nodeid);
 
 /*!
- * \brief List nodes that can be fenced.
+ * \brief List nodes that can be fenced
  *
- * \note If \p xml is not NULL, it will be freed first and the previous
- *       contents lost.
- *
- * \param[in,out] xml        The destination for the result, as an XML tree
- * \param[in]     st         A connection to the STONITH API
+ * \param[in,out] xml        The destination for the result, as an XML tree (if
+ *                           not NULL, previous contents will be freed and lost)
+ * \param[in,out] st         A connection to the fencer API
  * \param[in]     device_id  Resource ID of fence device to check
- * \param[in]     timeout    How long to wait for the operation to complete (in ms)
+ * \param[in]     timeout    How long to wait for operation to complete (in ms)
  *
  * \return Standard Pacemaker return code
  */
@@ -360,89 +424,81 @@ int pcmk_fence_list_targets(xmlNodePtr *xml, stonith_t *st,
                             const char *device_id, unsigned int timeout);
 
 /*!
- * \brief Get metadata for a resource.
+ * \brief Get metadata for a fence agent
  *
  * \note If \p xml is not NULL, it will be freed first and the previous
  *       contents lost.
  *
- * \param[in,out] xml     The destination for the result, as an XML tree.
- * \param[in]     st      A connection to the STONITH API.
- * \param[in]     agent   The fence agent to get metadata for.
- * \param[in]     timeout How long to wait for the operation to complete (in ms).
+ * \param[in,out] xml      The destination for the result, as an XML tree (if
+ *                         not NULL, previous contents will be freed and lost)
+ * \param[in,out] st       A connection to the fencer API
+ * \param[in]     agent    The fence agent to get metadata for
+ * \param[in]     timeout  How long to wait for operation to complete (in ms)
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_fence_metadata(xmlNodePtr *xml, stonith_t *st, char *agent,
+int pcmk_fence_metadata(xmlNodePtr *xml, stonith_t *st, const char *agent,
                         unsigned int timeout);
 
 /*!
- * \brief List registered fence devices.
+ * \brief List registered fence devices
  *
- * \note If \p xml is not NULL, it will be freed first and the previous
- *       contents lost.
- *
- * \param[in,out] xml     The destination for the result, as an XML tree.
- * \param[in]     st      A connection to the STONITH API.
- * \param[in]     target  If not NULL, only return devices that can fence
- *                        this node.
- * \param[in]     timeout How long to wait for the operation to complete (in ms).
+ * \param[in,out] xml      The destination for the result, as an XML tree (if
+ *                         not NULL, previous contents will be freed and lost)
+ * \param[in,out] st       A connection to the fencer API
+ * \param[in]     target   If not NULL, return only devices that can fence this
+ * \param[in]     timeout  How long to wait for operation to complete (in ms)
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_fence_registered(xmlNodePtr *xml, stonith_t *st, char *target,
+int pcmk_fence_registered(xmlNodePtr *xml, stonith_t *st, const char *target,
                           unsigned int timeout);
 
 /*!
- * \brief Register a fencing level for a specific node, node regex, or attribute.
+ * \brief Register a fencing topology level
  *
- * \p target can take three different forms:
- *   - name=value, in which case \p target is an attribute.
- *   - @pattern, in which case \p target is a node regex.
- *   - Otherwise, \p target is a node name.
- *
- * \param[in] st          A connection to the STONITH API.
- * \param[in] target      The object to register a fencing level for.
- * \param[in] fence_level Index number of level to add.
- * \param[in] devices     Devices to use in level.
+ * \param[in,out] st           A connection to the fencer API
+ * \param[in]     target       What fencing level targets (as "name=value" to
+ *                             target by given node attribute, or "@pattern" to
+ *                             target by node name pattern, or a node name)
+ * \param[in]     fence_level  Index number of level to add
+ * \param[in]     devices      Devices to use in level
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_fence_register_level(stonith_t *st, char *target, int fence_level,
-                              stonith_key_value_t *devices);
+int pcmk_fence_register_level(stonith_t *st, const char *target,
+                              int fence_level,
+                              const stonith_key_value_t *devices);
 
 /*!
- * \brief Unregister a fencing level for a specific node, node regex, or attribute.
+ * \brief Unregister a fencing topology level
  *
- * \p target can take three different forms:
- *   - name=value, in which case \p target is an attribute.
- *   - @pattern, in which case \p target is a node regex.
- *   - Otherwise, \p target is a node name.
- *
- * \param[in] st          A connection to the STONITH API.
- * \param[in] target      The object to unregister a fencing level for.
- * \param[in] fence_level Index number of level to remove.
+ * \param[in,out] st           A connection to the fencer API
+ * \param[in]     target       What fencing level targets (as "name=value" to
+ *                             target by given node attribute, or "@pattern" to
+ *                             target by node name pattern, or a node name)
+ * \param[in]     fence_level  Index number of level to remove
  *
  * \return Standard Pacemaker return code
  */
-int pcmk_fence_unregister_level(stonith_t *st, char *target, int fence_level);
+int pcmk_fence_unregister_level(stonith_t *st, const char *target,
+                                int fence_level);
 
 /*!
- * \brief Validate a STONITH device configuration.
+ * \brief Validate a fence device configuration
  *
- * \note If \p xml is not NULL, it will be freed first and the previous
- *       contents lost.
- *
- * \param[in,out] xml     The destination for the result, as an XML tree.
- * \param[in]     st      A connection to the STONITH API.
- * \param[in]     agent   The agent to validate (for example, "fence_xvm").
- * \param[in]     id      STONITH device ID (may be NULL).
- * \param[in]     params  STONITH device configuration parameters.
- * \param[in]     timeout How long to wait for the operation to complete (in ms).
+ * \param[in,out] xml      The destination for the result, as an XML tree (if
+ *                         not NULL, previous contents will be freed and lost)
+ * \param[in,out] st       A connection to the fencer API
+ * \param[in]     agent    The agent to validate (for example, "fence_xvm")
+ * \param[in]     id       Fence device ID (may be NULL)
+ * \param[in]     params   Fence device configuration parameters
+ * \param[in]     timeout  How long to wait for operation to complete (in ms)
  *
  * \return Standard Pacemaker return code
  */
 int pcmk_fence_validate(xmlNodePtr *xml, stonith_t *st, const char *agent,
-                        const char *id, stonith_key_value_t *params,
+                        const char *id, const stonith_key_value_t *params,
                         unsigned int timeout);
 #endif
 
