@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -32,7 +32,6 @@ void pcmk__bundle_add_utilization(const pe_resource_t *rsc,
                                   GList *all_rscs, GHashTable *utilization);
 void pcmk__bundle_shutdown_lock(pe_resource_t *rsc);
 
-pe_node_t *pcmk__clone_allocate(pe_resource_t *rsc, const pe_node_t *prefer);
 void clone_create_actions(pe_resource_t *rsc);
 void clone_internal_constraints(pe_resource_t *rsc);
 void clone_rsc_location(pe_resource_t *rsc, pe__location_t *constraint);
@@ -40,20 +39,12 @@ enum pe_action_flags clone_action_flags(pe_action_t *action,
                                         const pe_node_t *node);
 void clone_expand(pe_resource_t *rsc);
 bool clone_create_probe(pe_resource_t *rsc, pe_node_t *node);
-extern void clone_append_meta(pe_resource_t * rsc, xmlNode * xml);
+void clone_append_meta(const pe_resource_t *rsc, xmlNode *xml);
 void pcmk__clone_add_utilization(const pe_resource_t *rsc,
                                  const pe_resource_t *orig_rsc,
                                  GList *all_rscs, GHashTable *utilization);
 void pcmk__clone_shutdown_lock(pe_resource_t *rsc);
 
-uint32_t pcmk__multi_update_actions(pe_action_t *first,
-                                    pe_action_t *then,
-                                    const pe_node_t *node, uint32_t flags,
-                                    uint32_t filter, uint32_t type,
-                                    pe_working_set_t *data_set);
-
 void pcmk__log_transition_summary(const char *filename);
-void clone_create_pseudo_actions(pe_resource_t *rsc, GList *children,
-                                 notify_data_t **start_notify,
-                                 notify_data_t **stop_notify);
+
 #endif
