@@ -66,7 +66,7 @@ char *strndup(const char *str, size_t len);
 #    define USE_GNU
 #  endif
 
-#  if SUPPORT_DBUS
+#  if HAVE_DBUS
 #    ifndef HAVE_DBUSBASICVALUE
 #      include <stdint.h>
 #      include <dbus/dbus.h>
@@ -109,53 +109,49 @@ typedef union
   int fd;              /**< as Unix file descriptor */
 } DBusBasicValue;
 #    endif  // !defined(HAVE_DBUSBASICVALUE)
-#  endif    // !defined(SUPPORT_DBUS)
+#  endif    // !defined(HAVE_DBUS)
 
 // Replacement constants for Linux-specific errno values
 
 #  include <errno.h>
 
 #  ifndef ENOTUNIQ
+#    define PCMK__ENOTUNIQ
 #    define ENOTUNIQ  190
 #  endif
 
 #  ifndef ECOMM
+#    define PCMK__ECOMM
 #    define ECOMM     191
 #  endif
 
 #  ifndef ELIBACC
+#    define PCMK__ELIBACC
 #    define ELIBACC   192
 #  endif
 
 #  ifndef EREMOTEIO
+#    define PCMK__EREMOTIO
 #    define EREMOTEIO 193
 #  endif
 
-#  ifndef EUNATCH
-#    define EUNATCH   194
-#  endif
-
 #  ifndef ENOKEY
+#    define PCMK__ENOKEY
 #    define ENOKEY    195
 #  endif
 
 #  ifndef ENODATA
+#    define PCMK__ENODATA
 #    define ENODATA   196
 #  endif
 
 #  ifndef ETIME
+#    define PCMK__ETIME
 #    define ETIME     197
 #  endif
 
-#  ifndef ENOSR
-#    define ENOSR     198
-#  endif
-
-#  ifndef ENOSTR
-#    define ENOSTR    199
-#  endif
-
 #  ifndef EKEYREJECTED
+#    define PCMK__EKEYREJECTED
 #    define EKEYREJECTED 200
 #  endif
 
