@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2019 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2022 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -15,7 +17,7 @@
 
 #include <crm/common/mainloop.h>
 #include <crm/cluster/internal.h>
-#include <crm/cluster/election.h>
+#include <crm/cluster/election_internal.h>
 #include <crm/crm.h>
 
 #define STORM_INTERVAL   2      /* in seconds */
@@ -34,18 +36,18 @@ struct election_s {
     time_t last_election_loss; // When dampening period ends
 };
 
-static void election_complete(election_t *e)
+static void
+election_complete(election_t *e)
 {
     e->state = election_won;
-
-    if(e->cb) {
+    if (e->cb != NULL) {
         e->cb(e);
     }
-
     election_reset(e);
 }
 
-static gboolean election_timer_cb(gpointer user_data)
+static gboolean
+election_timer_cb(gpointer user_data)
 {
     election_t *e = user_data;
 
@@ -54,13 +56,17 @@ static gboolean election_timer_cb(gpointer user_data)
     return FALSE;
 }
 
+/*!
+ * \brief Get current state of an election
+ *
+ * \param[in] e  Election object
+ *
+ * \return Current state of \e
+ */
 enum election_result
-election_state(election_t *e)
+election_state(const election_t *e)
 {
-    if(e) {
-        return e->state;
-    }
-    return election_error;
+    return (e == NULL)? election_error : e->state;
 }
 
 /*!
@@ -116,13 +122,13 @@ election_init(const char *name, const char *uname, guint period_ms, GSourceFunc
  * This discards any recorded vote from a specified peer. Election users should
  * call this whenever a voting peer becomes inactive.
  *
- * \param[in] e      Election object
- * \param[in] uname  Name of peer to disregard
+ * \param[in,out] e      Election object
+ * \param[in]     uname  Name of peer to disregard
  */
 void
 election_remove(election_t *e, const char *uname)
 {
-    if(e && uname && e->voted) {
+    if ((e != NULL) && (uname != NULL) && (e->voted != NULL)) {
         crm_trace("Discarding %s (no-)vote from lost peer %s", e->name, uname);
         g_hash_table_remove(e->voted, uname);
     }
@@ -131,7 +137,7 @@ election_remove(election_t *e, const char *uname)
 /*!
  * \brief Stop election timer and disregard all votes
  *
- * \param[in] e      Election object
+ * \param[in,out] e  Election object
  */
 void
 election_reset(election_t *e)
@@ -153,12 +159,12 @@ election_reset(election_t *e)
  * Free all memory associated with an election object, stopping its
  * election timer (if running).
  *
- * \param[in] e      Election object
+ * \param[in,out] e  Election object
  */
 void
 election_fini(election_t *e)
 {
-    if(e) {
+    if (e != NULL) {
         election_reset(e);
         crm_trace("Destroying %s", e->name);
         mainloop_timer_del(e->timeout);
@@ -171,7 +177,7 @@ election_fini(election_t *e)
 static void
 election_timeout_start(election_t *e)
 {
-    if(e) {
+    if (e != NULL) {
         mainloop_timer_start(e->timeout);
     }
 }
@@ -179,12 +185,12 @@ election_timeout_start(election_t *e)
 /*!
  * \brief Stop an election's timer, if running
  *
- * \param[in] e      Election object
+ * \param[in,out] e  Election object
  */
 void
 election_timeout_stop(election_t *e)
 {
-    if(e) {
+    if (e != NULL) {
         mainloop_timer_stop(e->timeout);
     }
 }
@@ -192,13 +198,13 @@ election_timeout_stop(election_t *e)
 /*!
  * \brief Change an election's timeout (restarting timer if running)
  *
- * \param[in] e      Election object
- * \param[in] period New timeout
+ * \param[in,out] e       Election object
+ * \param[in]     period  New timeout
  */
 void
 election_timeout_set_period(election_t *e, guint period)
 {
-    if(e) {
+    if (e != NULL) {
         mainloop_timer_set_period(e->timeout, period);
     } else {
         crm_err("No election defined");
@@ -206,7 +212,7 @@ election_timeout_set_period(election_t *e, guint period)
 }
 
 static int
-crm_uptime(struct timeval *output)
+get_uptime(struct timeval *output)
 {
     static time_t expires = 0;
     static struct rusage info;
@@ -241,11 +247,11 @@ crm_uptime(struct timeval *output)
 }
 
 static int
-crm_compare_age(struct timeval your_age)
+compare_age(struct timeval your_age)
 {
     struct timeval our_age;
 
-    crm_uptime(&our_age); /* If an error occurred, our_age will be compared as {0,0} */
+    get_uptime(&our_age); /* If an error occurred, our_age will be compared as {0,0} */
 
     if (our_age.tv_sec > your_age.tv_sec) {
         crm_debug("Win: %ld vs %ld (seconds)", (long)our_age.tv_sec, (long)your_age.tv_sec);
@@ -272,7 +278,8 @@ crm_compare_age(struct timeval your_age)
  * Broadcast a "vote" election message containing the local node's ID,
  * (incremented) election counter, and uptime, and start the election timer.
  *
- * \param[in] e      Election object
+ * \param[in,out] e  Election object
+ *
  * \note Any nodes agreeing to the candidacy will send a "no-vote" reply, and if
  *       all active peers do so, or if the election times out, the local node
  *       wins the election. (If we lose to any peer vote, we will stop the
@@ -306,7 +313,7 @@ election_vote(election_t *e)
     crm_xml_add(vote, F_CRM_ELECTION_OWNER, our_node->uuid);
     crm_xml_add_int(vote, F_CRM_ELECTION_ID, e->count);
 
-    crm_uptime(&age);
+    get_uptime(&age);
     crm_xml_add_timeval(vote, F_CRM_ELECTION_AGE_S, F_CRM_ELECTION_AGE_US, &age);
 
     send_cluster_message(NULL, crm_msg_crmd, vote, TRUE);
@@ -323,7 +330,7 @@ election_vote(election_t *e)
  * If all known peers have sent no-vote messages, stop the election timer, set
  * the election state to won, and call any registered win callback.
  *
- * \param[in] e      Election object
+ * \param[in,out] e  Election object
  *
  * \return TRUE if local node has won, FALSE otherwise
  * \note If all known peers have sent no-vote messages, but the election owner
@@ -338,7 +345,7 @@ election_check(election_t *e)
     int voted_size = 0;
     int num_members = 0;
 
-    if(e == NULL) {
+    if (e == NULL) {
         crm_trace("Election check requested, but no election available");
         return FALSE;
     }
@@ -403,7 +410,7 @@ struct vote {
 /*!
  * \brief Unpack an election message
  *
- * \param[in] e        Election object
+ * \param[in] e        Election object (for logging only)
  * \param[in] message  Election message XML
  * \param[out] vote    Parsed fields from message
  *
@@ -412,7 +419,8 @@ struct vote {
  *       the message argument.
  */
 static bool
-parse_election_message(election_t *e, xmlNode *message, struct vote *vote)
+parse_election_message(const election_t *e, const xmlNode *message,
+                       struct vote *vote)
 {
     CRM_CHECK(message && vote, return FALSE);
 
@@ -482,7 +490,7 @@ record_vote(election_t *e, struct vote *vote)
 
     CRM_ASSERT(e && vote && vote->from && vote->op);
     if (e->voted == NULL) {
-        e->voted = crm_str_table_new();
+        e->voted = pcmk__strkey_table(free, free);
     }
 
     voter_copy = strdup(vote->from);
@@ -510,9 +518,9 @@ send_no_vote(crm_node_t *peer, struct vote *vote)
 /*!
  * \brief Process an election message (vote or no-vote) from a peer
  *
- * \param[in] e        Election object
- * \param[in] vote     Election message XML from peer
- * \param[in] can_win  Whether to consider the local node eligible for winning
+ * \param[in,out] e        Election object
+ * \param[in]     message  Election message XML from peer
+ * \param[in]     can_win  Whether local node is eligible to win
  *
  * \return Election state after new vote is considered
  * \note If the peer message is a vote, and we prefer the peer to win, this will
@@ -523,7 +531,7 @@ send_no_vote(crm_node_t *peer, struct vote *vote)
  *       this function, and then compare the result.
  */
 enum election_result
-election_count_vote(election_t *e, xmlNode *message, bool can_win)
+election_count_vote(election_t *e, const xmlNode *message, bool can_win)
 {
     int log_level = LOG_INFO;
     gboolean done = FALSE;
@@ -545,7 +553,7 @@ election_count_vote(election_t *e, xmlNode *message, bool can_win)
                    && pcmk__str_eq(our_node->uuid, vote.election_owner,
                                    pcmk__str_none);
 
-    if(can_win == FALSE) {
+    if (!can_win) {
         reason = "Not eligible";
         we_lose = TRUE;
 
@@ -588,7 +596,7 @@ election_count_vote(election_t *e, xmlNode *message, bool can_win)
 
     } else {
         // A peer vote requires a comparison to determine which node is better
-        int age_result = crm_compare_age(vote.age);
+        int age_result = compare_age(vote.age);
         int version_result = compare_version(vote.version, CRM_FEATURE_SET);
 
         if (version_result < 0) {
@@ -710,7 +718,7 @@ election_count_vote(election_t *e, xmlNode *message, bool can_win)
 /*!
  * \brief Reset any election dampening currently in effect
  *
- * \param[in] e        Election object to clear
+ * \param[in,out] e        Election object to clear
  */
 void
 election_clear_dampening(election_t *e)
