@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -45,9 +45,11 @@ typedef time_t log_time_t;
 unsigned int crm_log_level = LOG_INFO;
 unsigned int crm_trace_nonlog = 0;
 bool pcmk__is_daemon = false;
+char *pcmk__our_nodename = NULL;
 
 static unsigned int crm_log_priority = LOG_NOTICE;
 static GLogFunc glib_log_default = NULL;
+static pcmk__output_t *logger_out = NULL;
 
 static gboolean crm_tracing_enabled(void);
 
@@ -64,12 +66,11 @@ crm_glib_handler(const gchar * log_domain, GLogLevelFlags flags, const gchar * m
                                       LOG_DEBUG, __LINE__, crm_trace_nonlog);
     }
 
-
     switch (msg_level) {
         case G_LOG_LEVEL_CRITICAL:
             log_level = LOG_CRIT;
 
-            if (crm_is_callsite_active(glib_cs, LOG_DEBUG, 0) == FALSE) {
+            if (!crm_is_callsite_active(glib_cs, LOG_DEBUG, crm_trace_nonlog)) {
                 /* log and record how we got here */
                 crm_abort(__FILE__, __func__, __LINE__, message, TRUE, TRUE);
             }
@@ -131,8 +132,21 @@ crm_log_deinit(void)
 
 #define FMT_MAX 256
 
+/*!
+ * \internal
+ * \brief Set the log format string based on the passed-in method
+ *
+ * \param[in] method        The detail level of the log output
+ * \param[in] daemon        The daemon ID included in error messages
+ * \param[in] use_pid       Cached result of getpid() call, for efficiency
+ * \param[in] use_nodename  Cached result of uname() call, for efficiency
+ *
+ */
+
+/* XXX __attribute__((nonnull)) for use_nodename parameter */
 static void
-set_format_string(int method, const char *daemon)
+set_format_string(int method, const char *daemon, pid_t use_pid,
+                  const char *use_nodename)
 {
     if (method == QB_LOG_SYSLOG) {
         // The system log gets a simplified, user-friendly format
@@ -146,17 +160,10 @@ set_format_string(int method, const char *daemon)
         char fmt[FMT_MAX];
 
         if (method > QB_LOG_STDERR) {
-            struct utsname res;
-            const char *nodename = "localhost";
-
-            if (uname(&res) == 0) {
-                nodename = res.nodename;
-            }
-
             // If logging to file, prefix with timestamp, node name, daemon ID
             offset += snprintf(fmt + offset, FMT_MAX - offset,
                                TIMESTAMP_FORMAT_SPEC " %s %-20s[%lu] ",
-                               nodename, daemon, (unsigned long) getpid());
+                                use_nodename, daemon, (unsigned long) use_pid);
         }
 
         // Add function name (in parentheses)
@@ -180,7 +187,7 @@ set_format_string(int method, const char *daemon)
 static bool
 logfile_disabled(const char *filename)
 {
-    return pcmk__str_eq(filename, "none", pcmk__str_casei)
+    return pcmk__str_eq(filename, PCMK__VALUE_NONE, pcmk__str_casei)
            || pcmk__str_eq(filename, "/dev/null", pcmk__str_none);
 }
 
@@ -289,8 +296,8 @@ static void
 setenv_logfile(const char *filename)
 {
     // Some resource agents will log only if environment variable is set
-    if (pcmk__env_option("logfile") == NULL) {
-        pcmk__set_env_option("logfile", filename);
+    if (pcmk__env_option(PCMK__ENV_LOGFILE) == NULL) {
+        pcmk__set_env_option(PCMK__ENV_LOGFILE, filename);
     }
 }
 
@@ -377,6 +384,31 @@ pcmk__add_logfile(const char *filename)
     return pcmk_rc_ok;
 }
 
+/*!
+ * \brief Add multiple additional log files
+ *
+ * \param[in] log_files  Array of log files to add
+ * \param[in] out        Output object to use for error reporting
+ *
+ * \return Standard Pacemaker return code
+ */
+void
+pcmk__add_logfiles(gchar **log_files, pcmk__output_t *out)
+{
+    if (log_files == NULL) {
+        return;
+    }
+
+    for (gchar **fname = log_files; *fname != NULL; fname++) {
+        int rc = pcmk__add_logfile(*fname);
+
+        if (rc != pcmk_rc_ok) {
+            out->err(out, "Logging to %s is disabled: %s",
+                     *fname, pcmk_rc_str(rc));
+        }
+    }
+}
+
 static int blackbox_trigger = 0;
 static volatile char *blackbox_file_prefix = NULL;
 
@@ -465,7 +497,7 @@ crm_disable_blackbox(int nsig)
  * @TODO actually make this async-safe
  */
 void
-crm_write_blackbox(int nsig, struct qb_log_callsite *cs)
+crm_write_blackbox(int nsig, const struct qb_log_callsite *cs)
 {
     static volatile int counter = 1;
     static volatile time_t last = 0;
@@ -710,8 +742,24 @@ crm_priority2int(const char *name)
 }
 
 
+/*!
+ * \internal
+ * \brief Set the identifier for the current process
+ *
+ * If the identifier crm_system_name is not already set, then it is set as follows:
+ * - it is passed to the function via the "entity" parameter, or
+ * - it is derived from the executable name
+ *
+ * The identifier can be used in logs, IPC, and more.
+ *
+ * This method also sets the PCMK_service environment variable.
+ *
+ * \param[in] entity  If not NULL, will be assigned to the identifier
+ * \param[in] argc    The number of command line parameters
+ * \param[in] argv    The command line parameter values
+ */
 static void
-set_identity(const char *entity, int argc, char **argv)
+set_identity(const char *entity, int argc, char *const *argv)
 {
     if (crm_system_name != NULL) {
         return; // Already set, don't overwrite
@@ -740,61 +788,88 @@ set_identity(const char *entity, int argc, char **argv)
 }
 
 void
-crm_log_preinit(const char *entity, int argc, char **argv)
+crm_log_preinit(const char *entity, int argc, char *const *argv)
 {
     /* Configure libqb logging with nothing turned on */
 
+    struct utsname res;
     int lpc = 0;
     int32_t qb_facility = 0;
+    pid_t pid = getpid();
+    const char *nodename = "localhost";
+    static bool have_logging = false;
 
-    static bool have_logging = FALSE;
-
-    if(have_logging == FALSE) {
-        have_logging = TRUE;
-
-        crm_xml_init(); /* Sets buffer allocation strategy */
-
-        if (crm_trace_nonlog == 0) {
-            crm_trace_nonlog = g_quark_from_static_string("Pacemaker non-logging tracepoint");
-        }
-
-        umask(S_IWGRP | S_IWOTH | S_IROTH);
-
-        /* Redirect messages from glib functions to our handler */
-        glib_log_default = g_log_set_default_handler(crm_glib_handler, NULL);
-
-        /* and for good measure... - this enum is a bit field (!) */
-        g_log_set_always_fatal((GLogLevelFlags) 0); /*value out of range */
-
-        /* Set crm_system_name, which is used as the logging name. It may also
-         * be used for other purposes such as an IPC client name.
-         */
-        set_identity(entity, argc, argv);
-
-        qb_facility = qb_log_facility2int("local0");
-        qb_log_init(crm_system_name, qb_facility, LOG_ERR);
-        crm_log_level = LOG_CRIT;
-
-        /* Nuke any syslog activity until it's asked for */
-        qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_ENABLED, QB_FALSE);
-#ifdef HAVE_qb_log_conf_QB_LOG_CONF_MAX_LINE_LEN
-        // Shorter than default, generous for what we *should* send to syslog
-        qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_MAX_LINE_LEN, 256);
-#endif
-
-        /* Set format strings and disable threading
-         * Pacemaker and threads do not mix well (due to the amount of forking)
-         */
-        qb_log_tags_stringify_fn_set(crm_quark_to_string);
-        for (lpc = QB_LOG_SYSLOG; lpc < QB_LOG_TARGET_MAX; lpc++) {
-            qb_log_ctl(lpc, QB_LOG_CONF_THREADED, QB_FALSE);
-#ifdef HAVE_qb_log_conf_QB_LOG_CONF_ELLIPSIS
-            // End truncated lines with '...'
-            qb_log_ctl(lpc, QB_LOG_CONF_ELLIPSIS, QB_TRUE);
-#endif
-            set_format_string(lpc, crm_system_name);
-        }
+    if (have_logging) {
+        return;
     }
+
+    have_logging = true;
+
+    crm_xml_init(); /* Sets buffer allocation strategy */
+
+    if (crm_trace_nonlog == 0) {
+        crm_trace_nonlog = g_quark_from_static_string("Pacemaker non-logging tracepoint");
+    }
+
+    umask(S_IWGRP | S_IWOTH | S_IROTH);
+
+    /* Redirect messages from glib functions to our handler */
+    glib_log_default = g_log_set_default_handler(crm_glib_handler, NULL);
+
+    /* and for good measure... - this enum is a bit field (!) */
+    g_log_set_always_fatal((GLogLevelFlags) 0); /*value out of range */
+
+    /* Set crm_system_name, which is used as the logging name. It may also
+     * be used for other purposes such as an IPC client name.
+     */
+    set_identity(entity, argc, argv);
+
+    qb_facility = qb_log_facility2int("local0");
+    qb_log_init(crm_system_name, qb_facility, LOG_ERR);
+    crm_log_level = LOG_CRIT;
+
+    /* Nuke any syslog activity until it's asked for */
+    qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_ENABLED, QB_FALSE);
+#ifdef HAVE_qb_log_conf_QB_LOG_CONF_MAX_LINE_LEN
+    // Shorter than default, generous for what we *should* send to syslog
+    qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_MAX_LINE_LEN, 256);
+#endif
+    if (uname(memset(&res, 0, sizeof(res))) == 0 && *res.nodename != '\0') {
+        nodename = res.nodename;
+    }
+
+    /* Set format strings and disable threading
+     * Pacemaker and threads do not mix well (due to the amount of forking)
+     */
+    qb_log_tags_stringify_fn_set(crm_quark_to_string);
+    for (lpc = QB_LOG_SYSLOG; lpc < QB_LOG_TARGET_MAX; lpc++) {
+        qb_log_ctl(lpc, QB_LOG_CONF_THREADED, QB_FALSE);
+#ifdef HAVE_qb_log_conf_QB_LOG_CONF_ELLIPSIS
+        // End truncated lines with '...'
+        qb_log_ctl(lpc, QB_LOG_CONF_ELLIPSIS, QB_TRUE);
+#endif
+        set_format_string(lpc, crm_system_name, pid, nodename);
+    }
+
+#ifdef ENABLE_NLS
+    /* Enable translations (experimental). Currently we only have a few
+     * proof-of-concept translations for some option help. The goal would be to
+     * offer translations for option help and man pages rather than logs or
+     * documentation, to reduce the burden of maintaining them.
+     */
+
+    // Load locale information for the local host from the environment
+    setlocale(LC_ALL, "");
+
+    // Tell gettext where to find Pacemaker message catalogs
+    CRM_ASSERT(bindtextdomain(PACKAGE, PCMK__LOCALE_DIR) != NULL);
+
+    // Tell gettext to use the Pacemaker message catalogs
+    CRM_ASSERT(textdomain(PACKAGE) != NULL);
+
+    // Tell gettext that the translated strings are stored in UTF-8
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
+#endif
 }
 
 gboolean
@@ -802,7 +877,7 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
              int argc, char **argv, gboolean quiet)
 {
     const char *syslog_priority = NULL;
-    const char *facility = pcmk__env_option("logfacility");
+    const char *facility = pcmk__env_option(PCMK__ENV_LOGFACILITY);
     const char *f_copy = facility;
 
     pcmk__is_daemon = daemon;
@@ -820,12 +895,12 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         if (pcmk__is_daemon) {
             facility = "daemon";
         } else {
-            facility = "none";
+            facility = PCMK__VALUE_NONE;
         }
-        pcmk__set_env_option("logfacility", facility);
+        pcmk__set_env_option(PCMK__ENV_LOGFACILITY, facility);
     }
 
-    if (pcmk__str_eq(facility, "none", pcmk__str_casei)) {
+    if (pcmk__str_eq(facility, PCMK__VALUE_NONE, pcmk__str_casei)) {
         quiet = TRUE;
 
 
@@ -833,13 +908,13 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_FACILITY, qb_log_facility2int(facility));
     }
 
-    if (pcmk__env_option_enabled(crm_system_name, "debug")) {
+    if (pcmk__env_option_enabled(crm_system_name, PCMK__ENV_DEBUG)) {
         /* Override the default setting */
         crm_log_level = LOG_DEBUG;
     }
 
     /* What lower threshold do we have for sending to syslog */
-    syslog_priority = pcmk__env_option("logpriority");
+    syslog_priority = pcmk__env_option(PCMK__ENV_LOGPRIORITY);
     if (syslog_priority) {
         crm_log_priority = crm_priority2int(syslog_priority);
     }
@@ -852,7 +927,7 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
     }
 
     /* Should we log to stderr */ 
-    if (pcmk__env_option_enabled(crm_system_name, "stderr")) {
+    if (pcmk__env_option_enabled(crm_system_name, PCMK__ENV_STDERR)) {
         /* Override the default setting */
         to_stderr = TRUE;
     }
@@ -860,9 +935,9 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
 
     // Log to a file if we're a daemon or user asked for one
     {
-        const char *logfile = pcmk__env_option("logfile");
+        const char *logfile = pcmk__env_option(PCMK__ENV_LOGFILE);
 
-        if (!pcmk__str_eq("none", logfile, pcmk__str_casei)
+        if (!pcmk__str_eq(PCMK__VALUE_NONE, logfile, pcmk__str_casei)
             && (pcmk__is_daemon || (logfile != NULL))) {
             // Daemons always get a log file, unless explicitly set to "none"
             pcmk__add_logfile(logfile);
@@ -870,14 +945,14 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
     }
 
     if (pcmk__is_daemon
-        && pcmk__env_option_enabled(crm_system_name, "blackbox")) {
+        && pcmk__env_option_enabled(crm_system_name, PCMK__ENV_BLACKBOX)) {
         crm_enable_blackbox(0);
     }
 
     /* Summary */
     crm_trace("Quiet: %d, facility %s", quiet, f_copy);
-    pcmk__env_option("logfile");
-    pcmk__env_option("logfacility");
+    pcmk__env_option(PCMK__ENV_LOGFILE);
+    pcmk__env_option(PCMK__ENV_LOGFACILITY);
 
     crm_update_callsites();
 
@@ -895,7 +970,6 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
 
     if (pcmk__is_daemon) {
         int user = getuid();
-        const char *base = CRM_CORE_DIR;
         struct passwd *pwent = getpwuid(user);
 
         if (pwent == NULL) {
@@ -904,21 +978,11 @@ crm_log_init(const char *entity, uint8_t level, gboolean daemon, gboolean to_std
         } else if (!pcmk__strcase_any_of(pwent->pw_name, "root", CRM_DAEMON_USER, NULL)) {
             crm_trace("Don't change active directory for regular user: %s", pwent->pw_name);
 
-        } else if (chdir(base) < 0) {
-            crm_perror(LOG_INFO, "Cannot change active directory to %s", base);
+        } else if (chdir(CRM_CORE_DIR) < 0) {
+            crm_perror(LOG_INFO, "Cannot change active directory to " CRM_CORE_DIR);
 
         } else {
-            crm_info("Changed active directory to %s", base);
-#if 0
-            {
-                char path[512];
-
-                snprintf(path, 512, "%s-%lu", crm_system_name, (unsigned long) getpid());
-                mkdir(path, 0750);
-                chdir(path);
-                crm_info("Changed active directory to %s/%s/%s", base, pwent->pw_name, path);
-            }
-#endif
+            crm_info("Changed active directory to " CRM_CORE_DIR);
         }
 
         /* Original meanings from signal(7)
@@ -1060,7 +1124,54 @@ pcmk__cli_init_logging(const char *name, unsigned int verbosity)
     }
 }
 
+/*!
+ * \brief Log XML line-by-line in a formatted fashion
+ *
+ * \param[in] level  Priority at which to log the messages
+ * \param[in] text   Prefix for each line
+ * \param[in] xml    XML to log
+ *
+ * \note This does nothing when \p level is \p LOG_STDOUT.
+ * \note Do not call this function directly. It should be called only from the
+ *       \p do_crm_log_xml() macro.
+ */
+void
+pcmk_log_xml_impl(uint8_t level, const char *text, const xmlNode *xml)
+{
+    if (xml == NULL) {
+        do_crm_log(level, "%s%sNo data to dump as XML",
+                   pcmk__s(text, ""), pcmk__str_empty(text)? "" : " ");
+
+    } else {
+        if (logger_out == NULL) {
+            CRM_CHECK(pcmk__log_output_new(&logger_out) == pcmk_rc_ok, return);
+        }
+
+        pcmk__output_set_log_level(logger_out, level);
+        pcmk__xml_show(logger_out, text, xml, 1,
+                       pcmk__xml_fmt_pretty
+                       |pcmk__xml_fmt_open
+                       |pcmk__xml_fmt_children
+                       |pcmk__xml_fmt_close);
+    }
+}
+
+/*!
+ * \internal
+ * \brief Free the logging library's internal log output object
+ */
+void
+pcmk__free_common_logger(void)
+{
+    if (logger_out != NULL) {
+        logger_out->finish(logger_out, CRM_EX_OK, true, NULL);
+        pcmk__output_free(logger_out);
+        logger_out = NULL;
+    }
+}
+
 // Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
 
 #include <crm/common/logging_compat.h>
 
@@ -1077,4 +1188,5 @@ crm_add_logfile(const char *filename)
     return pcmk__add_logfile(filename) == pcmk_rc_ok;
 }
 
+// LCOV_EXCL_STOP
 // End deprecated API
