@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2020 the Pacemaker project contributors
+ * Copyright 2008-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -23,7 +23,7 @@
 #include <netdb.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <inttypes.h>  /* X32T ~ PRIx32 */
+#include <inttypes.h>   // PRIx32
 
 #include <glib.h>
 #include <bzlib.h>
@@ -106,8 +106,8 @@ localized_remote_header(pcmk__remote_t *remote)
 
         CRM_LOG_ASSERT(endian == ENDIAN_LOCAL);
         if(endian != ENDIAN_LOCAL) {
-            crm_err("Invalid message detected, endian mismatch: %" X32T
-                    " is neither %" X32T " nor the swab'd %" X32T,
+            crm_err("Invalid message detected, endian mismatch: %" PRIx32
+                    " is neither %" PRIx32 " nor the swab'd %" PRIx32,
                     ENDIAN_LOCAL, header->endian, endian);
             return NULL;
         }
@@ -163,43 +163,34 @@ pcmk__tls_client_handshake(pcmk__remote_t *remote, int timeout_ms)
  * \param[in] session  TLS session to affect
  */
 static void
-set_minimum_dh_bits(gnutls_session_t *session)
+set_minimum_dh_bits(const gnutls_session_t *session)
 {
-    const char *dh_min_bits_s = getenv("PCMK_dh_min_bits");
+    int dh_min_bits;
 
-    if (dh_min_bits_s) {
-        int dh_min_bits = crm_parse_int(dh_min_bits_s, "0");
+    pcmk__scan_min_int(getenv("PCMK_dh_min_bits"), &dh_min_bits, 0);
 
-        /* This function is deprecated since GnuTLS 3.1.7, in favor of letting
-         * the priority string imply the DH requirements, but this is the only
-         * way to give the user control over compatibility with older servers.
-         */
-        if (dh_min_bits > 0) {
-            crm_info("Requiring server use a Diffie-Hellman prime of at least %d bits",
-                     dh_min_bits);
-            gnutls_dh_set_prime_bits(*session, dh_min_bits);
-        }
+    /* This function is deprecated since GnuTLS 3.1.7, in favor of letting
+     * the priority string imply the DH requirements, but this is the only
+     * way to give the user control over compatibility with older servers.
+     */
+    if (dh_min_bits > 0) {
+        crm_info("Requiring server use a Diffie-Hellman prime of at least %d bits",
+                 dh_min_bits);
+        gnutls_dh_set_prime_bits(*session, dh_min_bits);
     }
 }
 
 static unsigned int
 get_bound_dh_bits(unsigned int dh_bits)
 {
-    const char *dh_min_bits_s = getenv("PCMK_dh_min_bits");
-    const char *dh_max_bits_s = getenv("PCMK_dh_max_bits");
-    int dh_min_bits = 0;
-    int dh_max_bits = 0;
+    int dh_min_bits;
+    int dh_max_bits;
 
-    if (dh_min_bits_s) {
-        dh_min_bits = crm_parse_int(dh_min_bits_s, "0");
-    }
-    if (dh_max_bits_s) {
-        dh_max_bits = crm_parse_int(dh_max_bits_s, "0");
-        if ((dh_min_bits > 0) && (dh_max_bits > 0)
-            && (dh_max_bits < dh_min_bits)) {
-            crm_warn("Ignoring PCMK_dh_max_bits because it is less than PCMK_dh_min_bits");
-            dh_max_bits = 0;
-        }
+    pcmk__scan_min_int(getenv("PCMK_dh_min_bits"), &dh_min_bits, 0);
+    pcmk__scan_min_int(getenv("PCMK_dh_max_bits"), &dh_max_bits, 0);
+    if ((dh_max_bits > 0) && (dh_max_bits < dh_min_bits)) {
+        crm_warn("Ignoring PCMK_dh_max_bits less than PCMK_dh_min_bits");
+        dh_max_bits = 0;
     }
     if ((dh_min_bits > 0) && (dh_bits < dh_min_bits)) {
         return dh_min_bits;
@@ -316,16 +307,12 @@ pcmk__init_tls_dh(gnutls_dh_params_t *dh_params)
         goto error;
     }
 
-#ifdef HAVE_GNUTLS_SEC_PARAM_TO_PK_BITS
     dh_bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH,
                                           GNUTLS_SEC_PARAM_NORMAL);
     if (dh_bits == 0) {
         rc = GNUTLS_E_DH_PRIME_UNACCEPTABLE;
         goto error;
     }
-#else
-    dh_bits = 1024;
-#endif
     dh_bits = get_bound_dh_bits(dh_bits);
 
     crm_info("Generating Diffie-Hellman parameters with %u-bit prime for TLS",
@@ -355,7 +342,7 @@ error:
  *         if some data was successfully read but more data is needed)
  */
 int
-pcmk__read_handshake_data(pcmk__client_t *client)
+pcmk__read_handshake_data(const pcmk__client_t *client)
 {
     int rc = 0;
 
@@ -492,8 +479,8 @@ remote_send_iovs(pcmk__remote_t *remote, struct iovec *iov, int iovs)
  * \internal
  * \brief Send an XML message over a Pacemaker Remote connection
  *
- * \param[in] remote  Pacemaker Remote connection to use
- * \param[in] msg     XML to send
+ * \param[in,out] remote  Pacemaker Remote connection to use
+ * \param[in]     msg     XML to send
  *
  * \return Standard Pacemaker return code
  */
@@ -544,7 +531,7 @@ pcmk__remote_send_xml(pcmk__remote_t *remote, xmlNode *msg)
  * \internal
  * \brief Obtain the XML from the currently buffered remote connection message
  *
- * \param[in] remote  Remote connection possibly with message available
+ * \param[in,out] remote  Remote connection possibly with message available
  *
  * \return Newly allocated XML object corresponding to message data, or NULL
  * \note This effectively removes the message from the connection buffer.
@@ -613,7 +600,7 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
 }
 
 static int
-get_remote_socket(pcmk__remote_t *remote)
+get_remote_socket(const pcmk__remote_t *remote)
 {
 #ifdef HAVE_GNUTLS_GNUTLS_H
     if (remote->tls_session) {
@@ -643,7 +630,7 @@ get_remote_socket(pcmk__remote_t *remote)
  *         the specified timeout)
  */
 int
-pcmk__remote_ready(pcmk__remote_t *remote, int timeout_ms)
+pcmk__remote_ready(const pcmk__remote_t *remote, int timeout_ms)
 {
     struct pollfd fds = { 0, };
     int sock = 0;
@@ -686,7 +673,7 @@ pcmk__remote_ready(pcmk__remote_t *remote, int timeout_ms)
  * \internal
  * \brief Read bytes from non-blocking remote connection
  *
- * \param[in] remote  Remote connection to read
+ * \param[in,out] remote  Remote connection to read
  *
  * \return Standard Pacemaker return code (of particular interest, pcmk_rc_ok if
  *         a full message has been received, or EAGAIN for a partial message)
@@ -794,9 +781,9 @@ read_available_remote_data(pcmk__remote_t *remote)
  * \internal
  * \brief Read one message from a remote connection
  *
- * \param[in]  remote      Remote connection to read
- * \param[in]  timeout_ms  Fail if message not read in this many milliseconds
- *                         (10s will be used if 0, and 60s if negative)
+ * \param[in,out] remote      Remote connection to read
+ * \param[in]     timeout_ms  Fail if message not read in this many milliseconds
+ *                            (10s will be used if 0, and 60s if negative)
  *
  * \return Standard Pacemaker return code
  */
@@ -1134,7 +1121,7 @@ pcmk__connect_remote(const char *host, int port, int timeout, int *timer_id,
             ((struct sockaddr_in *)(void*)addr)->sin_port = htons(port);
         }
 
-        memset(buffer, 0, DIMOF(buffer));
+        memset(buffer, 0, PCMK__NELEM(buffer));
         pcmk__sockaddr2str(addr, buffer);
         crm_info("Attempting remote connection to %s:%d", buffer, port);
 
@@ -1176,16 +1163,17 @@ async_cleanup:
  *          as long as its sa_family member is set correctly.
  */
 void
-pcmk__sockaddr2str(void *sa, char *s)
+pcmk__sockaddr2str(const void *sa, char *s)
 {
-    switch (((struct sockaddr*)sa)->sa_family) {
+    switch (((const struct sockaddr *) sa)->sa_family) {
         case AF_INET:
-            inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr),
+            inet_ntop(AF_INET, &(((const struct sockaddr_in *) sa)->sin_addr),
                       s, INET6_ADDRSTRLEN);
             break;
 
         case AF_INET6:
-            inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr),
+            inet_ntop(AF_INET6,
+                      &(((const struct sockaddr_in6 *) sa)->sin6_addr),
                       s, INET6_ADDRSTRLEN);
             break;
 
@@ -1259,7 +1247,7 @@ pcmk__accept_remote_connection(int ssock, int *csock)
  * \return Remote connection TCP port number
  */
 int
-crm_default_remote_port()
+crm_default_remote_port(void)
 {
     static int port = 0;
 
