@@ -1,27 +1,33 @@
 /*
- * Copyright 2020-2021 the Pacemaker project contributors
+ * Copyright 2020-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
- * This source code is licensed under the GNU Lesser General Public License
- * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
+#include "crm/common/results.h"
 #include <crm_internal.h>
 
-#include <stdarg.h>
-#include <stddef.h>
-#include <stdint.h>
-#include <setjmp.h>
-#include <cmocka.h>
+#include <crm/common/unittest_internal.h>
 
 static void
 empty_input_string(void **state)
 {
     long long start, end;
 
-    assert_int_equal(pcmk__parse_ll_range(NULL, &start, &end), pcmk_rc_unknown_format);
-    assert_int_equal(pcmk__parse_ll_range("", &start, &end), pcmk_rc_unknown_format);
+    assert_int_equal(pcmk__parse_ll_range(NULL, &start, &end), ENODATA);
+    assert_int_equal(pcmk__parse_ll_range("", &start, &end), ENODATA);
+}
+
+static void
+null_input_variables(void **state)
+{
+    long long start, end;
+
+    pcmk__assert_asserts(pcmk__parse_ll_range("1234", NULL, &end));
+    pcmk__assert_asserts(pcmk__parse_ll_range("1234", &start, NULL));
 }
 
 static void
@@ -39,7 +45,7 @@ only_separator(void **state)
 {
     long long start, end;
 
-    assert_int_equal(pcmk__parse_ll_range("-", &start, &end), pcmk_rc_unknown_format);
+    assert_int_equal(pcmk__parse_ll_range("-", &start, &end), pcmk_rc_bad_input);
     assert_int_equal(start, PCMK__PARSE_INT_DEFAULT);
     assert_int_equal(end, PCMK__PARSE_INT_DEFAULT);
 }
@@ -72,6 +78,8 @@ range_start_and_end(void **state)
     assert_int_equal(pcmk__parse_ll_range("2000-2020", &start, &end), pcmk_rc_ok);
     assert_int_equal(start, 2000);
     assert_int_equal(end, 2020);
+
+    assert_int_equal(pcmk__parse_ll_range("2000-2020-2030", &start, &end), pcmk_rc_bad_input);
 }
 
 static void
@@ -79,28 +87,31 @@ garbage(void **state)
 {
     long long start, end;
 
-    assert_int_equal(pcmk__parse_ll_range("2000x-", &start, &end), pcmk_rc_unknown_format);
+    assert_int_equal(pcmk__parse_ll_range("2000x-", &start, &end), pcmk_rc_bad_input);
     assert_int_equal(start, PCMK__PARSE_INT_DEFAULT);
     assert_int_equal(end, PCMK__PARSE_INT_DEFAULT);
 
-    assert_int_equal(pcmk__parse_ll_range("-x2000", &start, &end), pcmk_rc_unknown_format);
+    assert_int_equal(pcmk__parse_ll_range("-x2000", &start, &end), pcmk_rc_bad_input);
     assert_int_equal(start, PCMK__PARSE_INT_DEFAULT);
     assert_int_equal(end, PCMK__PARSE_INT_DEFAULT);
 }
 
-int main(int argc, char **argv)
+static void
+strtoll_errors(void **state)
 {
-    const struct CMUnitTest tests[] = {
-        cmocka_unit_test(empty_input_string),
-        cmocka_unit_test(missing_separator),
-        cmocka_unit_test(only_separator),
-        cmocka_unit_test(no_range_end),
-        cmocka_unit_test(no_range_start),
-        cmocka_unit_test(range_start_and_end),
+    long long start, end;
 
-        cmocka_unit_test(garbage),
-    };
-
-    cmocka_set_message_output(CM_OUTPUT_TAP);
-    return cmocka_run_group_tests(tests, NULL, NULL);
+    assert_int_equal(pcmk__parse_ll_range("20000000000000000000-", &start, &end), EOVERFLOW);
+    assert_int_equal(pcmk__parse_ll_range("100-20000000000000000000", &start, &end), EOVERFLOW);
 }
+
+PCMK__UNIT_TEST(NULL, NULL,
+                cmocka_unit_test(empty_input_string),
+                cmocka_unit_test(null_input_variables),
+                cmocka_unit_test(missing_separator),
+                cmocka_unit_test(only_separator),
+                cmocka_unit_test(no_range_end),
+                cmocka_unit_test(no_range_start),
+                cmocka_unit_test(range_start_and_end),
+                cmocka_unit_test(strtoll_errors),
+                cmocka_unit_test(garbage))
