@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -28,10 +28,10 @@ CRM_TRACE_INIT_DATA(pe_rules);
 /*!
  * \brief Evaluate any rules contained by given XML element
  *
- * \param[in]  xml          XML element to check for rules
- * \param[in]  node_hash    Node attributes to use when evaluating expressions
- * \param[in]  now          Time to use when evaluating expressions
- * \param[out] next_change  If not NULL, set to when evaluation will change
+ * \param[in,out] xml          XML element to check for rules
+ * \param[in]     node_hash    Node attributes to use to evaluate expressions
+ * \param[in]     now          Time to use when evaluating expressions
+ * \param[out]    next_change  If not NULL, set to when evaluation will change
  *
  * \return TRUE if no rules, or any of rules present is in effect, else FALSE
  */
@@ -75,12 +75,12 @@ pe_test_rule(xmlNode *rule, GHashTable *node_hash, enum rsc_role_e role,
  * date expression. Given any one of those, evaluate it and return whether it
  * passed.
  *
- * \param[in]  expr         Rule subelement XML
- * \param[in]  node_hash    Node attributes to use when evaluating expression
- * \param[in]  role         Resource role to use when evaluating expression
- * \param[in]  now          Time to use when evaluating expression
- * \param[out] next_change  If not NULL, set to when evaluation will change
- * \param[in]  match_data   If not NULL, resource back-references and params
+ * \param[in,out] expr         Rule subelement XML
+ * \param[in]     node_hash    Node attributes to use when evaluating expression
+ * \param[in]     role         Resource role to use when evaluating expression
+ * \param[in]     now          Time to use when evaluating expression
+ * \param[out]    next_change  If not NULL, set to when evaluation will change
+ * \param[in]     match_data   If not NULL, resource back-references and params
  *
  * \return TRUE if expression is in effect under given conditions, else FALSE
  */
@@ -110,31 +110,26 @@ find_expression_type(xmlNode * expr)
     attr = crm_element_value(expr, XML_EXPR_ATTR_ATTRIBUTE);
     tag = crm_element_name(expr);
 
-    if (pcmk__str_eq(tag, "date_expression", pcmk__str_casei)) {
+    if (pcmk__str_eq(tag, PCMK_XE_DATE_EXPRESSION, pcmk__str_none)) {
         return time_expr;
 
-    } else if (pcmk__str_eq(tag, "rsc_expression", pcmk__str_casei)) {
+    } else if (pcmk__str_eq(tag, PCMK_XE_RSC_EXPRESSION, pcmk__str_none)) {
         return rsc_expr;
 
-    } else if (pcmk__str_eq(tag, "op_expression", pcmk__str_casei)) {
+    } else if (pcmk__str_eq(tag, PCMK_XE_OP_EXPRESSION, pcmk__str_none)) {
         return op_expr;
 
-    } else if (pcmk__str_eq(tag, XML_TAG_RULE, pcmk__str_casei)) {
+    } else if (pcmk__str_eq(tag, XML_TAG_RULE, pcmk__str_none)) {
         return nested_rule;
 
-    } else if (!pcmk__str_eq(tag, "expression", pcmk__str_casei)) {
+    } else if (!pcmk__str_eq(tag, XML_TAG_EXPRESSION, pcmk__str_none)) {
         return not_expr;
 
-    } else if (pcmk__strcase_any_of(attr, CRM_ATTR_UNAME, CRM_ATTR_KIND, CRM_ATTR_ID, NULL)) {
+    } else if (pcmk__str_any_of(attr, CRM_ATTR_UNAME, CRM_ATTR_KIND, CRM_ATTR_ID, NULL)) {
         return loc_expr;
 
-    } else if (pcmk__str_eq(attr, CRM_ATTR_ROLE, pcmk__str_casei)) {
+    } else if (pcmk__str_eq(attr, CRM_ATTR_ROLE, pcmk__str_none)) {
         return role_expr;
-
-#if ENABLE_VERSIONED_ATTRS
-    } else if (pcmk__str_eq(attr, CRM_ATTR_RA_VERSION, pcmk__str_casei)) {
-        return version_expr;
-#endif
     }
 
     return attr_expr;
@@ -158,7 +153,7 @@ find_expression_type(xmlNode * expr)
  */
 
 static int
-phase_of_the_moon(crm_time_t * now)
+phase_of_the_moon(const crm_time_t *now)
 {
     uint32_t epact, diy, goldn;
     uint32_t y;
@@ -174,7 +169,8 @@ phase_of_the_moon(crm_time_t * now)
 }
 
 static int
-check_one(xmlNode *cron_spec, const char *xml_field, uint32_t time_field) {
+check_one(const xmlNode *cron_spec, const char *xml_field, uint32_t time_field)
+{
     int rc = pcmk_rc_undetermined;
     const char *value = crm_element_value(cron_spec, xml_field);
     long long low, high;
@@ -184,7 +180,7 @@ check_one(xmlNode *cron_spec, const char *xml_field, uint32_t time_field) {
         goto bail;
     }
 
-    if (pcmk__parse_ll_range(value, &low, &high) == pcmk_rc_unknown_format) {
+    if (pcmk__parse_ll_range(value, &low, &high) != pcmk_rc_ok) {
        goto bail;
     } else if (low == high) {
         /* A single number was given, not a range. */
@@ -240,7 +236,7 @@ check_passes(int rc) {
 } while (0)
 
 int
-pe_cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
+pe_cron_range_satisfied(const crm_time_t *now, const xmlNode *cron_spec)
 {
     uint32_t h, m, s, y, d, w;
 
@@ -265,6 +261,11 @@ pe_cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
     CHECK_ONE(cron_spec, "weekdays", d);
 
     CHECK_ONE(cron_spec, "moon", phase_of_the_moon(now));
+    if (crm_element_value(cron_spec, "moon") != NULL) {
+        pcmk__config_warn("Support for 'moon' in date_spec elements "
+                          "(such as %s) is deprecated and will be removed "
+                          "in a future release of Pacemaker", ID(cron_spec));
+    }
 
     /* If we get here, either no fields were specified (which is success), or all
      * the fields that were specified had their conditions met (which is also a
@@ -274,7 +275,7 @@ pe_cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
 }
 
 static void
-update_field(crm_time_t *t, xmlNode *xml, const char *attr,
+update_field(crm_time_t *t, const xmlNode *xml, const char *attr,
             void (*time_fn)(crm_time_t *, int))
 {
     long long value;
@@ -285,12 +286,10 @@ update_field(crm_time_t *t, xmlNode *xml, const char *attr,
     }
 }
 
-crm_time_t *
-pe_parse_xml_duration(crm_time_t * start, xmlNode * duration_spec)
+static crm_time_t *
+parse_xml_duration(const crm_time_t *start, const xmlNode *duration_spec)
 {
-    crm_time_t *end = crm_time_new_undefined();
-
-    crm_time_set(end, start);
+    crm_time_t *end = pcmk_copy_time(start);
 
     update_field(end, duration_spec, "years", crm_time_add_years);
     update_field(end, duration_spec, "months", crm_time_add_months);
@@ -409,71 +408,11 @@ populate_hash(xmlNode * nvpair_list, GHashTable * hash, gboolean overwrite, xmlN
     }
 }
 
-#if ENABLE_VERSIONED_ATTRS
-static xmlNode*
-get_versioned_rule(xmlNode * attr_set)
-{
-    xmlNode * rule = NULL;
-    xmlNode * expr = NULL;
-
-    for (rule = pcmk__xe_first_child(attr_set); rule != NULL;
-         rule = pcmk__xe_next(rule)) {
-
-        if (pcmk__str_eq((const char *)rule->name, XML_TAG_RULE,
-                         pcmk__str_none)) {
-            for (expr = pcmk__xe_first_child(rule); expr != NULL;
-                 expr = pcmk__xe_next(expr)) {
-
-                if (find_expression_type(expr) == version_expr) {
-                    return rule;
-                }
-            }
-        }
-    }
-
-    return NULL;
-}
-
-static void
-add_versioned_attributes(xmlNode * attr_set, xmlNode * versioned_attrs)
-{
-    xmlNode *attr_set_copy = NULL;
-    xmlNode *rule = NULL;
-    xmlNode *expr = NULL;
-
-    if (!attr_set || !versioned_attrs) {
-        return;
-    }
-
-    attr_set_copy = copy_xml(attr_set);
-
-    rule = get_versioned_rule(attr_set_copy);
-    if (!rule) {
-        free_xml(attr_set_copy);
-        return;
-    }
-
-    expr = pcmk__xe_first_child(rule);
-    while (expr != NULL) {
-        if (find_expression_type(expr) != version_expr) {
-            xmlNode *node = expr;
-
-            expr = pcmk__xe_next(expr);
-            free_xml(node);
-        } else {
-            expr = pcmk__xe_next(expr);
-        }
-    }
-
-    add_node_nocopy(versioned_attrs, NULL, attr_set_copy);
-}
-#endif
-
 typedef struct unpack_data_s {
     gboolean overwrite;
     void *hash;
     crm_time_t *next_change;
-    pe_rule_eval_data_t *rule_data;
+    const pe_rule_eval_data_t *rule_data;
     xmlNode *top;
 } unpack_data_t;
 
@@ -488,48 +427,25 @@ unpack_attr_set(gpointer data, gpointer user_data)
         return;
     }
 
-#if ENABLE_VERSIONED_ATTRS
-    if (get_versioned_rule(pair->attr_set) && !(unpack_data->rule_data->node_hash &&
-        g_hash_table_lookup_extended(unpack_data->rule_data->node_hash,
-                                     CRM_ATTR_RA_VERSION, NULL, NULL))) {
-        // we haven't actually tested versioned expressions yet
-        return;
-    }
-#endif
-
     crm_trace("Adding attributes from %s (score %d) %s overwrite",
               pair->name, pair->score,
               (unpack_data->overwrite? "with" : "without"));
     populate_hash(pair->attr_set, unpack_data->hash, unpack_data->overwrite, unpack_data->top);
 }
 
-#if ENABLE_VERSIONED_ATTRS
-static void
-unpack_versioned_attr_set(gpointer data, gpointer user_data)
-{
-    sorted_set_t *pair = data;
-    unpack_data_t *unpack_data = user_data;
-
-    if (pe_eval_rules(pair->attr_set, unpack_data->rule_data,
-                      unpack_data->next_change)) {
-        add_versioned_attributes(pair->attr_set, unpack_data->hash);
-    }
-}
-#endif
-
 /*!
  * \internal
  * \brief Create a sorted list of nvpair blocks
  *
- * \param[in]  top           XML document root (used to expand id-ref's)
- * \param[in]  xml_obj       XML element containing blocks of nvpair elements
- * \param[in]  set_name      If not NULL, only get blocks of this element type
- * \param[in]  always_first  If not NULL, sort block with this ID as first
+ * \param[in,out] top           XML document root (used to expand id-ref's)
+ * \param[in]     xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]     set_name      If not NULL, only get blocks of this element
+ * \param[in]     always_first  If not NULL, sort block with this ID as first
  *
  * \return List of sorted_set_t entries for nvpair blocks
  */
 static GList *
-make_pairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+make_pairs(xmlNode *top, const xmlNode *xml_obj, const char *set_name,
            const char *always_first)
 {
     GList *unsorted = NULL;
@@ -566,24 +482,22 @@ make_pairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
 }
 
 /*!
- * \internal
  * \brief Extract nvpair blocks contained by an XML element into a hash table
  *
- * \param[in]  top           XML document root (used to expand id-ref's)
- * \param[in]  xml_obj       XML element containing blocks of nvpair elements
- * \param[in]  set_name      If not NULL, only use blocks of this element type
- * \param[out] hash          Where to store extracted name/value pairs
- * \param[in]  always_first  If not NULL, process block with this ID first
- * \param[in]  overwrite     Whether to replace existing values with same name
- * \param[in]  rule_data     Matching parameters to use when unpacking
- * \param[out] next_change   If not NULL, set to when rule evaluation will change
- * \param[in]  unpack_func   Function to call to unpack each block
+ * \param[in,out] top           XML document root (used to expand id-ref's)
+ * \param[in]     xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]     set_name      If not NULL, only use blocks of this element
+ * \param[in]     rule_data     Matching parameters to use when unpacking
+ * \param[out]    hash          Where to store extracted name/value pairs
+ * \param[in]     always_first  If not NULL, process block with this ID first
+ * \param[in]     overwrite     Whether to replace existing values with same name
+ * \param[out]    next_change   If not NULL, set to when evaluation will change
  */
-static void
-unpack_nvpair_blocks(xmlNode *top, xmlNode *xml_obj, const char *set_name,
-                     void *hash, const char *always_first, gboolean overwrite,
-                     pe_rule_eval_data_t *rule_data, crm_time_t *next_change,
-                     GFunc unpack_func)
+void
+pe_eval_nvpairs(xmlNode *top, const xmlNode *xml_obj, const char *set_name,
+                const pe_rule_eval_data_t *rule_data, GHashTable *hash,
+                const char *always_first, gboolean overwrite,
+                crm_time_t *next_change)
 {
     GList *pairs = make_pairs(top, xml_obj, set_name, always_first);
 
@@ -596,36 +510,26 @@ unpack_nvpair_blocks(xmlNode *top, xmlNode *xml_obj, const char *set_name,
             .rule_data = rule_data
         };
 
-        g_list_foreach(pairs, unpack_func, &data);
+        g_list_foreach(pairs, unpack_attr_set, &data);
         g_list_free_full(pairs, free);
     }
 }
 
-void
-pe_eval_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
-                pe_rule_eval_data_t *rule_data, GHashTable *hash,
-                const char *always_first, gboolean overwrite,
-                crm_time_t *next_change)
-{
-    unpack_nvpair_blocks(top, xml_obj, set_name, hash, always_first,
-                         overwrite, rule_data, next_change, unpack_attr_set);
-}
-
 /*!
  * \brief Extract nvpair blocks contained by an XML element into a hash table
  *
- * \param[in]  top           XML document root (used to expand id-ref's)
- * \param[in]  xml_obj       XML element containing blocks of nvpair elements
- * \param[in]  set_name      Element name to identify nvpair blocks
- * \param[in]  node_hash     Node attributes to use when evaluating rules
- * \param[out] hash          Where to store extracted name/value pairs
- * \param[in]  always_first  If not NULL, process block with this ID first
- * \param[in]  overwrite     Whether to replace existing values with same name
- * \param[in]  now           Time to use when evaluating rules
- * \param[out] next_change   If not NULL, set to when rule evaluation will change
+ * \param[in,out] top           XML document root (used to expand id-ref's)
+ * \param[in]     xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]     set_name      Element name to identify nvpair blocks
+ * \param[in]     node_hash     Node attributes to use when evaluating rules
+ * \param[out]    hash          Where to store extracted name/value pairs
+ * \param[in]     always_first  If not NULL, process block with this ID first
+ * \param[in]     overwrite     Whether to replace existing values with same name
+ * \param[in]     now           Time to use when evaluating rules
+ * \param[out]    next_change   If not NULL, set to when evaluation will change
  */
 void
-pe_unpack_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+pe_unpack_nvpairs(xmlNode *top, const xmlNode *xml_obj, const char *set_name,
                   GHashTable *node_hash, GHashTable *hash,
                   const char *always_first, gboolean overwrite,
                   crm_time_t *now, crm_time_t *next_change)
@@ -643,19 +547,17 @@ pe_unpack_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
                     always_first, overwrite, next_change);
 }
 
-#if ENABLE_VERSIONED_ATTRS
-void
-pe_eval_versioned_attributes(xmlNode *top, xmlNode *xml_obj, const char *set_name,
-                             pe_rule_eval_data_t *rule_data, xmlNode *hash,
-                             crm_time_t *next_change)
-{
-    unpack_nvpair_blocks(top, xml_obj, set_name, hash, NULL, FALSE, rule_data,
-                         next_change, unpack_versioned_attr_set);
-}
-#endif
-
+/*!
+ * \brief Expand any regular expression submatches (%0-%9) in a string
+ *
+ * \param[in] string      String possibly containing submatch variables
+ * \param[in] match_data  If not NULL, regular expression matches
+ *
+ * \return Newly allocated string identical to \p string with submatches
+ *         expanded, or NULL if there were no matches
+ */
 char *
-pe_expand_re_matches(const char *string, pe_re_match_data_t *match_data)
+pe_expand_re_matches(const char *string, const pe_re_match_data_t *match_data)
 {
     size_t len = 0;
     int i;
@@ -711,33 +613,18 @@ pe_expand_re_matches(const char *string, pe_re_match_data_t *match_data)
     return result;
 }
 
-#if ENABLE_VERSIONED_ATTRS
-GHashTable*
-pe_unpack_versioned_parameters(xmlNode *versioned_params, const char *ra_version)
-{
-    GHashTable *hash = pcmk__strkey_table(free, free);
-
-    if (versioned_params && ra_version) {
-        GHashTable *node_hash = pcmk__strkey_table(free, free);
-        xmlNode *attr_set = pcmk__xe_first_child(versioned_params);
-
-        if (attr_set) {
-            g_hash_table_insert(node_hash, strdup(CRM_ATTR_RA_VERSION),
-                                strdup(ra_version));
-            pe_unpack_nvpairs(NULL, versioned_params,
-                              crm_element_name(attr_set), node_hash, hash, NULL,
-                              FALSE, NULL, NULL);
-        }
-
-        g_hash_table_destroy(node_hash);
-    }
-
-    return hash;
-}
-#endif
-
+/*!
+ * \brief Evaluate rules
+ *
+ * \param[in,out] ruleset      XML possibly containing rule sub-elements
+ * \param[in]     rule_data
+ * \param[out]    next_change  If not NULL, set to when evaluation will change
+ *
+ * \return TRUE if there are no rules or
+ */
 gboolean
-pe_eval_rules(xmlNode *ruleset, pe_rule_eval_data_t *rule_data, crm_time_t *next_change)
+pe_eval_rules(xmlNode *ruleset, const pe_rule_eval_data_t *rule_data,
+              crm_time_t *next_change)
 {
     // If there are no rules, pass by default
     gboolean ruleset_default = TRUE;
@@ -760,8 +647,18 @@ pe_eval_rules(xmlNode *ruleset, pe_rule_eval_data_t *rule_data, crm_time_t *next
     return ruleset_default;
 }
 
+/*!
+ * \brief Evaluate all of a rule's expressions
+ *
+ * \param[in,out] rule         XML containing a rule definition or its id-ref
+ * \param[in]     rule_data    Matching parameters to check against rule
+ * \param[out]    next_change  If not NULL, set to when evaluation will change
+ *
+ * \return TRUE if \p rule_data passes \p rule, otherwise FALSE
+ */
 gboolean
-pe_eval_expr(xmlNode *rule, pe_rule_eval_data_t *rule_data, crm_time_t *next_change)
+pe_eval_expr(xmlNode *rule, const pe_rule_eval_data_t *rule_data,
+             crm_time_t *next_change)
 {
     xmlNode *expr = NULL;
     gboolean test = TRUE;
@@ -802,8 +699,18 @@ pe_eval_expr(xmlNode *rule, pe_rule_eval_data_t *rule_data, crm_time_t *next_cha
     return passed;
 }
 
+/*!
+ * \brief Evaluate a single rule expression, including any subexpressions
+ *
+ * \param[in,out] expr         XML containing a rule expression
+ * \param[in]     rule_data    Matching parameters to check against expression
+ * \param[out]    next_change  If not NULL, set to when evaluation will change
+ *
+ * \return TRUE if \p rule_data passes \p expr, otherwise FALSE
+ */
 gboolean
-pe_eval_subexpr(xmlNode *expr, pe_rule_eval_data_t *rule_data, crm_time_t *next_change)
+pe_eval_subexpr(xmlNode *expr, const pe_rule_eval_data_t *rule_data,
+                crm_time_t *next_change)
 {
     gboolean accept = FALSE;
     const char *uname = NULL;
@@ -847,19 +754,6 @@ pe_eval_subexpr(xmlNode *expr, pe_rule_eval_data_t *rule_data, crm_time_t *next_
             accept = pe__eval_op_expr(expr, rule_data);
             break;
 
-#if ENABLE_VERSIONED_ATTRS
-        case version_expr:
-            if (rule_data->node_hash &&
-                g_hash_table_lookup_extended(rule_data->node_hash,
-                                             CRM_ATTR_RA_VERSION, NULL, NULL)) {
-                accept = pe__eval_attr_expr(expr, rule_data);
-            } else {
-                // we are going to test it when we have ra-version
-                accept = TRUE;
-            }
-            break;
-#endif
-
         default:
             CRM_CHECK(FALSE /* bad type */ , return FALSE);
             accept = FALSE;
@@ -1038,7 +932,7 @@ accept_attr_expr(const char *l_val, const char *r_val, const char *type,
  */
 static const char *
 expand_value_source(const char *value, const char *value_source,
-                    pe_match_data_t *match_data)
+                    const pe_match_data_t *match_data)
 {
     GHashTable *table = NULL;
 
@@ -1072,7 +966,7 @@ expand_value_source(const char *value, const char *value_source,
  * \return TRUE if rule_data satisfies the expression, FALSE otherwise
  */
 gboolean
-pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
+pe__eval_attr_expr(const xmlNode *expr, const pe_rule_eval_data_t *rule_data)
 {
     gboolean attr_allocated = FALSE;
     const char *h_val = NULL;
@@ -1089,10 +983,13 @@ pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
     type = crm_element_value(expr, XML_EXPR_ATTR_TYPE);
     value_source = crm_element_value(expr, XML_EXPR_ATTR_VALUE_SOURCE);
 
-    if (attr == NULL || op == NULL) {
-        pe_err("Invalid attribute or operation in expression"
-               " (\'%s\' \'%s\' \'%s\')", crm_str(attr), crm_str(op), crm_str(value));
+    if (attr == NULL) {
+        pe_err("Expression %s invalid: " XML_EXPR_ATTR_ATTRIBUTE
+               " not specified", pcmk__s(ID(expr), "without ID"));
         return FALSE;
+    } else if (op == NULL) {
+        pe_err("Expression %s invalid: " XML_EXPR_ATTR_OPERATION
+               " not specified", pcmk__s(ID(expr), "without ID"));
     }
 
     if (rule_data->match_data != NULL) {
@@ -1133,7 +1030,8 @@ pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
  * \return Standard Pacemaker return code
  */
 int
-pe__eval_date_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data, crm_time_t *next_change)
+pe__eval_date_expr(const xmlNode *expr, const pe_rule_eval_data_t *rule_data,
+                   crm_time_t *next_change)
 {
     crm_time_t *start = NULL;
     crm_time_t *end = NULL;
@@ -1161,7 +1059,7 @@ pe__eval_date_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data, crm_time_t *
     }
 
     if (start != NULL && end == NULL && duration_spec != NULL) {
-        end = pe_parse_xml_duration(start, duration_spec);
+        end = parse_xml_duration(start, duration_spec);
     }
 
     if (pcmk__str_eq(op, "in_range", pcmk__str_null_matches | pcmk__str_casei)) {
@@ -1215,7 +1113,8 @@ pe__eval_date_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data, crm_time_t *
 }
 
 gboolean
-pe__eval_op_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data) {
+pe__eval_op_expr(const xmlNode *expr, const pe_rule_eval_data_t *rule_data)
+{
     const char *name = crm_element_value(expr, XML_NVPAIR_ATTR_NAME);
     const char *interval_s = crm_element_value(expr, XML_LRM_ATTR_INTERVAL);
     guint interval;
@@ -1256,7 +1155,7 @@ pe__eval_op_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data) {
  * \return TRUE if rule_data->role satisfies the expression, FALSE otherwise
  */
 gboolean
-pe__eval_role_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
+pe__eval_role_expr(const xmlNode *expr, const pe_rule_eval_data_t *rule_data)
 {
     gboolean accept = FALSE;
     const char *op = NULL;
@@ -1299,7 +1198,7 @@ pe__eval_role_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
 }
 
 gboolean
-pe__eval_rsc_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
+pe__eval_rsc_expr(const xmlNode *expr, const pe_rule_eval_data_t *rule_data)
 {
     const char *class = crm_element_value(expr, XML_AGENT_ATTR_CLASS);
     const char *provider = crm_element_value(expr, XML_AGENT_ATTR_PROVIDER);
@@ -1409,8 +1308,8 @@ unpack_instance_attributes(xmlNode *top, xmlNode *xml_obj, const char *set_name,
         .op_data = NULL
     };
 
-    unpack_nvpair_blocks(top, xml_obj, set_name, hash, always_first,
-                         overwrite, &rule_data, NULL, unpack_attr_set);
+    pe_eval_nvpairs(top, xml_obj, set_name, &rule_data, hash, always_first,
+                    overwrite, NULL);
 }
 
 // LCOV_EXCL_STOP
