@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 the Pacemaker project contributors
+ * Copyright 2004-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -27,6 +27,8 @@
 #include <crm/pengine/internal.h>
 #include <pacemaker-internal.h>
 
+#define ATTR_SET_ELEMENT "attr_set_element"
+
 typedef struct node_info_s {
     const char *node_name;
     bool promoted;
@@ -76,19 +78,20 @@ int cli_resource_fail(pcmk_ipc_api_t *controld_api, const char *host_uname,
 GList *cli_resource_search(pe_resource_t *rsc, const char *requested_name,
                              pe_working_set_t *data_set);
 int cli_resource_delete(pcmk_ipc_api_t *controld_api, const char *host_uname,
-                        pe_resource_t *rsc, const char *operation,
+                        const pe_resource_t *rsc, const char *operation,
                         const char *interval_spec, bool just_failures,
                         pe_working_set_t *data_set, gboolean force);
 int cli_cleanup_all(pcmk_ipc_api_t *controld_api, const char *node_name,
                     const char *operation, const char *interval_spec,
                     pe_working_set_t *data_set);
-int cli_resource_restart(pcmk__output_t *out, pe_resource_t *rsc, pe_node_t *node,
-                         const char *move_lifetime, int timeout_ms, cib_t *cib,
-                         int cib_options, gboolean promoted_role_only, gboolean force);
-int cli_resource_move(pe_resource_t *rsc, const char *rsc_id, const char *host_name,
-                      const char *move_lifetime, cib_t *cib, int cib_options,
-                      pe_working_set_t *data_set, gboolean promoted_role_only,
-                      gboolean force);
+int cli_resource_restart(pcmk__output_t *out, pe_resource_t *rsc,
+                         const pe_node_t *node, const char *move_lifetime,
+                         int timeout_ms, cib_t *cib, int cib_options,
+                         gboolean promoted_role_only, gboolean force);
+int cli_resource_move(const pe_resource_t *rsc, const char *rsc_id,
+                      const char *host_name, const char *move_lifetime,
+                      cib_t *cib, int cib_options, pe_working_set_t *data_set,
+                      gboolean promoted_role_only, gboolean force);
 crm_exit_t cli_resource_execute_from_params(pcmk__output_t *out, const char *rsc_name,
                                             const char *rsc_class, const char *rsc_prov,
                                             const char *rsc_type, const char *rsc_action,
@@ -104,13 +107,11 @@ int cli_resource_update_attribute(pe_resource_t *rsc, const char *requested_name
                                   const char *attr_set, const char *attr_set_type,
                                   const char *attr_id, const char *attr_name,
                                   const char *attr_value, gboolean recursive,
-                                  cib_t *cib, int cib_options,
-                                  pe_working_set_t *data_set, gboolean force);
+                                  cib_t *cib, int cib_options, gboolean force);
 int cli_resource_delete_attribute(pe_resource_t *rsc, const char *requested_name,
                                   const char *attr_set, const char *attr_set_type,
                                   const char *attr_id, const char *attr_name,
-                                  cib_t *cib, int cib_options,
-                                  pe_working_set_t *data_set, gboolean force);
+                                  cib_t *cib, int cib_options, gboolean force);
 
 int update_working_set_xml(pe_working_set_t *data_set, xmlNode **xml);
 int wait_till_stable(pcmk__output_t *out, int timeout_ms, cib_t * cib);
