@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2022 the Pacemaker project contributors
+ * Copyright 2012-2023 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -748,7 +748,7 @@ main(int argc, char **argv)
         g_strfreev(processed_args);
         pcmk__free_arg_context(context);
         /* FIXME:  When crm_ticket is converted to use formatted output, this can go. */
-        pcmk__cli_help('v', CRM_EX_OK);
+        pcmk__cli_help('v');
     }
 
     data_set = pe_new_working_set();
@@ -990,10 +990,6 @@ main(int argc, char **argv)
 
     cib__clean_up_connection(&cib_conn);
 
-    if (rc == pcmk_rc_no_quorum) {
-        g_set_error(&error, PCMK__RC_ERROR, rc, "Use --force to ignore quorum");
-    }
-
     g_strfreev(processed_args);
     pcmk__free_arg_context(context);
     g_free(options.attr_default);
@@ -1005,7 +1001,7 @@ main(int argc, char **argv)
     g_free(options.ticket_id);
     g_free(options.xml_file);
 
-    pcmk__output_and_clear_error(error, NULL);
+    pcmk__output_and_clear_error(&error, NULL);
 
     crm_exit(exit_code);
 }
